Package: bbmle
Title: Tools for General Maximum Likelihood Estimation
Description: Methods and functions for fitting maximum likelihood models in R. This package modifies and extends the 'mle' classes in the 'stats4' package.
Version: 1.0.25.1
Authors@R: c(person("Ben","Bolker",email="bolker@mcmaster.ca",role=c("aut","cre"),
	   comment=c(ORCID="0000-0002-2127-0443")),
	   person("R Development Core Team",role=c("aut")),
	   person("Iago Giné-Vázquez", role=c("ctb"))
	  )
Depends: R (>= 3.0.0), stats4
Imports: stats, numDeriv, lattice, MASS, methods, bdsmatrix, Matrix,
        mvtnorm
Suggests: emdbook, rms, ggplot2, RUnit, MuMIn, AICcmodavg, Hmisc,
        optimx (>= 2013.8.6), knitr, testthat
VignetteBuilder: knitr
BuildVignettes: yes
License: GPL
URL: https://github.com/bbolker/bbmle
Collate: 'mle2-class.R' 'mle2-methods.R' 'mle.R' 'confint.R'
        'predict.R' 'profile.R' 'update.R' 'dists.R' 'IC.R' 'slice.R'
        'impsamp.R' 'TMB.R'
RoxygenNote: 7.1.0
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2023-12-08 23:45:52 UTC; bolker
Author: Ben Bolker [aut, cre] (<https://orcid.org/0000-0002-2127-0443>),
  R Development Core Team [aut],
  Iago Giné-Vázquez [ctb]
Maintainer: Ben Bolker <bolker@mcmaster.ca>
Repository: CRAN
Date/Publication: 2023-12-09 01:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 06:18:41 UTC; unix

export(mle2,call.to.char,namedrop,parnames,"parnames<-",relist2)
export(sbinom,snorm,sbeta,snbinom,spois,sbetabinom,dnorm_n,slnorm)
export(ICtab,AICtab,BICtab,AICctab)
export(stdEr,vcov)
export(slice,sliceOld,slice1D,slice2D)
export(proffun)
export(pop_pred_samp)
export(mle2.options)
exportClasses(mle2,summary.mle2)
exportMethods(AIC, AICc, qAICc, qAIC,
               profile, coef, confint, logLik,
              update, vcov, anova, deviance, residuals,
              simulate, predict, formula, plot, stdEr, summary)
importClassesFrom(stats4,mle)
importFrom(stats4,coef,confint,logLik,BIC,summary,profile,vcov,AIC, update, plot)
importFrom(stats,
           anova,deviance,residuals,
           simulate,predict,formula,napredict,na.omit,na.exclude,
           dnorm)
importFrom(methods,setMethod,is)
importFrom(lattice,xyplot,splom,diag.panel.splom,panel.abline,panel.number,panel.points,panel.xyplot)  ## for slice methods
importFrom(Matrix,nearPD)
importFrom(numDeriv,hessian,grad,jacobian)
importFrom(grDevices, dev.interactive)
importFrom(graphics, abline, lines, par, points, text)
importFrom(methods, new)
importFrom(stats, approx, approxfun, as.formula, constrOptim,
             deriv, model.matrix, na.omit, nlm, nlminb,
             optimize, pchisq, pnorm, printCoefmat, qbeta,
             qbinom, qchisq, qnbinom, qnorm, qpois, setNames,
           spline, uniroot, update.formula)
importFrom(MASS, mvrnorm, ginv)
importFrom(bdsmatrix, gchol)
importFrom(mvtnorm,dmvnorm)
S3method(as.data.frame,ICtab)
S3method(as.data.frame,profile.mle2)
S3method(print,ICtab)
S3method(slice,mle2)
S3method(plot,slice)
S3method(xyplot,slice)
S3method(splom,slice)

## ----opts,echo=FALSE----------------------------------------------------------
if (require("knitr")) opts_chunk$set(tidy=FALSE)

## ----dfun---------------------------------------------------------------------
dfun <- function(object) {
  with(object,sum((weights * residuals^2)[weights > 0])/df.residual)
}

## ----dobdata------------------------------------------------------------------
## Dobson (1990) Page 93: Randomized Controlled Trial :
counts <- c(18,17,15,20,10,20,25,13,12)
outcome <- gl(3,1,9)
treatment <- gl(3,3)

## ----fitdob-------------------------------------------------------------------
glmOT.D93 <- glm(counts ~ outcome + treatment, family=poisson)
glmO.D93  <- update(glmOT.D93, . ~ . - treatment)
glmT.D93  <- update(glmOT.D93, . ~ . - outcome)
glmX.D93  <- update(glmT.D93, . ~ . - treatment)
glmQOT.D93 <- update(glmOT.D93, family=quasipoisson)
glmQO.D93 <- update(glmO.D93, family=quasipoisson)
glmQT.D93 <- update(glmT.D93, family=quasipoisson)
glmQX.D93 <- update(glmX.D93, family=quasipoisson)

## ----dobll--------------------------------------------------------------------
(sum(dpois(counts,
          lambda=exp(predict(glmOT.D93)),log=TRUE))) ## by hand
(logLik(glmOT.D93))  ## from Poisson fit

## ----dobll2-------------------------------------------------------------------
(-2*(logLik(glmT.D93)-logLik(glmOT.D93)))  ## Poisson fit
(deviance(glmT.D93)-deviance(glmOT.D93))   ## Poisson fit
(deviance(glmQT.D93)-deviance(glmQOT.D93)) ## quasi-fit

## ----dobdisp------------------------------------------------------------------
(dfun(glmOT.D93))
(sum(residuals(glmOT.D93,"pearson")^2)/glmOT.D93$df.residual)
(summary(glmOT.D93)$dispersion)
(summary(glmQOT.D93)$dispersion)

## ----bbmle--------------------------------------------------------------------
library(bbmle)
(qAIC(glmOT.D93,dispersion=dfun(glmOT.D93)))
(qAICc(glmOT.D93,dispersion=dfun(glmOT.D93),nobs=length(counts)))
ICtab(glmOT.D93,glmT.D93,glmO.D93,glmX.D93,
      dispersion=dfun(glmOT.D93),type="qAIC")
ICtab(glmOT.D93,glmT.D93,glmO.D93,glmX.D93,
      dispersion=dfun(glmOT.D93),
      nobs=length(counts),type="qAICc")
detach("package:bbmle")

## ----AICcmodavg---------------------------------------------------------------
if (require("AICcmodavg")) {
    aictab(list(glmOT.D93,glmT.D93,glmO.D93,glmX.D93), 
           modnames=c("OT","T","O","X"),
           c.hat=dfun(glmOT.D93))
    detach("package:AICcmodavg")
}

## ----MuMin--------------------------------------------------------------------
if (require("MuMIn")) {
    packageVersion("MuMIn")
    ## from ?QAIC
    x.quasipoisson <- function(...) {
        res <- quasipoisson(...)
        res$aic <- poisson(...)$aic
        res
    }
    glmQOT2.D93 <- update(glmOT.D93,family="x.quasipoisson",
                          na.action=na.fail)
    (gg <-  dredge(glmQOT2.D93,rank="QAIC", chat=dfun(glmOT.D93)))
    (ggc <- dredge(glmQOT2.D93,rank="QAICc",chat=dfun(glmOT.D93)))
    detach("package:MuMIn")
}

## try to reconstruct error reported by Hofert Jan Marius
## (simpler version)

Lfun <- function(x) {
  (x-5)^2
}



library(bbmle)

lb <- 6
## first try with L-BFGS-B and bounds
m1 <- mle2(Lfun,start=list(x=7),lower=6,method="L-BFGS-B")
coef(m1)
p1 <- profile(m1)
plot(p1)
(c1 <- confint(m1,quietly=TRUE))
## all  OK

m2 <- mle2(Lfun,start=list(x=7),optimizer="optimize",
           lower=lb,upper=10)
coef(m2)
p2 <- profile(m2)
(c2 <- confint(m2))
(c2 <- confint(m2))
plot(p2,show.points=TRUE)

ICtab <- function(...,type=c("AIC","BIC","AICc","qAIC","qAICc"),
                  weights=FALSE,delta=TRUE,base=FALSE,
                  logLik=FALSE,
                  sort=TRUE,nobs=NULL,dispersion=1,mnames,k=2) {
    ## TO DO: allow inclusion of log-likelihood (or negative log-likelihood?)
    ## base or delta? or both?  Should deltas include delta-df as well?
    L <- list(...)
    if (is.list(L[[1]]) && length(L)==1) L <- L[[1]]
    type <- match.arg(type)
    if (dispersion !=1) {
        if (type=="BIC") stop("cannot specify dispersion with BIC")
        if (substr(type,1,1)!="q") {
            type = paste("q",type,sep="")
            warning("dispersion!=1, type changed to ",type)
        }
    }
    if (type=="AICc" || type=="BIC" || type=="qAICc") {
        if (is.null(nobs)) {
            ## if(is.null(attr(L[[1]],"nobs")))
            ## stop("must specify number of observations if corr=TRUE")
            ## nobs <- sapply(L,attr,"nobs")
            nobs <- sapply(L,nobs)
            if (length(unique(nobs))>1)
                stop("nobs different: must have identical data for all objects")
            nobs <- nobs[1]
        }
    }
    ICs <- switch(type,
                  AIC=sapply(L,AIC),
                  BIC=sapply(L,BIC),
                  AICc=sapply(L,AICc,nobs=nobs),
                  qAIC=sapply(L,qAIC,dispersion=dispersion),
                  qAICc=sapply(L,qAICc,nobs=nobs,dispersion=dispersion))
    logLiks <- sapply(L,function(x) c(logLik(x)))
    ## hack: protect against aod method
    if (is.matrix(ICs)) ICs <- ICs["AIC",]  
    getdf <- function(x) {
        if (!is.null(df <- attr(x,"df"))) return(df)
        else if (!is.null(df <- attr(logLik(x),"df"))) return(df)
    }
    dIC <- ICs-min(ICs,na.rm=TRUE)
    dlogLiks <- logLiks-min(logLiks,na.rm=TRUE)
    df <- sapply(L,getdf)
    tab <- data.frame(df=df)
    if (delta) {
        dName <- paste0("d",type)
        tab <- cbind(setNames(data.frame(dIC),dName),tab)
        if (logLik) {
            tab <- cbind(data.frame(dLogLik=dlogLiks),tab)
        }
    }
    if (base) {
        tab <- cbind(setNames(data.frame(ICs),type),tab)
        if (logLik) {
            tab <- cbind(data.frame(logLik=logLiks),tab)
        }
    }
    if (!delta && !base) stop("either 'base' or 'delta' must be TRUE")
    if (weights) {
        dIC_noNA <- na.exclude(dIC)
        wts <- napredict(attr(dIC_noNA,"na.action"),
                                    exp(-dIC_noNA/2)/sum(exp(-dIC_noNA/2)))
        tab <- data.frame(tab,weight=wts)
    }
    if (missing(mnames)) {
        Call <- match.call()
        if (!is.null(names(Call))) {
            xargs <- which(names(Call) %in% names(formals())[-1])
        } else xargs <- numeric(0)
        mnames <- as.character(Call)[c(-1,-xargs)]
    }
    row.names(tab) <- mnames
    if (sort) {
        tab <- tab[order(ICs),]
    }
    class(tab) <- "ICtab"
    tab
}

print.ICtab <- function(x,...,min.weight=0.001) {
    chtab <- format(do.call("cbind",lapply(x,round,1)))
    rownames(chtab) <- attr(x,"row.names")
    chtab[,"df"] <- as.character(round(x$df,1))
    if (!is.null(x$weight))
        chtab[,"weight"] <- format.pval(x$weight,eps=min.weight,
                                        digits=2)
    print(chtab,quote=FALSE)
}

as.data.frame.ICtab <- function(x, row.names = NULL, optional = FALSE, ...){
    attr(x,"class") <- "data.frame"
    as.data.frame(x, row.names = row.names, optional = optional)
}

AICtab <- function(...,mnames) {
    ## fancy footwork to preserve model names
    if (missing(mnames)) mnames <- get.mnames(match.call())
    ICtab(...,mnames=mnames,type="AIC")
}
BICtab <- function(...,mnames) {
    if (missing(mnames)) mnames <- get.mnames(match.call())
    ICtab(...,mnames=mnames,type="BIC")
}

AICctab <- function(...,mnames) {
    if (missing(mnames)) mnames <- get.mnames(match.call())
    ICtab(...,mnames=mnames,type="AICc")
}

setGeneric("AICc", function(object, ..., nobs=NULL, k=2) standardGeneric("AICc"))

setMethod("AICc", "mle2",
          function (object, ..., nobs, k)  {
              L <- list(...)
              if (length(L)) {
                  L = c(list(object), L)
                  # First, we attempt to use the "nobs" attribute
                  if (is.null(nobs)) {
                      nobs <- unlist(lapply(L, attr,"nobs"))
                  }
                  # If that is still null, maybe there's a "nobs" method?
                  if (is.null(nobs)) {
                      nobs <- unlist(lapply(L,nobs))
                  }
                  if (length(unique(nobs))>1)
                      stop("nobs different: must have identical data for all objects")
                  logLiks <- lapply(L, logLik)
                  df <- sapply(logLiks,attr,"df")
                  val <- -2*unlist(logLiks)+k*df+k*df*(df+1)/(nobs-df-1)
                  data.frame(AICc=val,df=df)
              } else {
                  if (is.null(nobs)) {
                      nobs <- attr(object,"nobs")
                  }
                  if (is.null(nobs)) {
                      nobs <- nobs(object)
                  }
                  AICc(object=logLik(object), nobs=nobs, k=k)
              }
          })

setMethod("AICc", signature(object="logLik"),
          function(object, ..., nobs, k){
              # Handles the "nobs" argument
              if (missing(nobs)) {
                  if (is.null(attr(object,"nobs")))
                      stop("number of observations not specified")
                  nobs <- attr(object,"nobs")
              }
              if (length(list(...))>1)
                  warning("additional parameters ignored")

              df <- attr(object,"df")
              -2*c(object)+k*df+k*df*(df+1)/(nobs-df-1)
          })

setMethod("AICc", signature(object="ANY"),
          function(object, ..., nobs, k){
              AICc(object=logLik(object, ...), nobs=nobs, k=k)
          })

setMethod("AIC", "mle2",
          function (object, ..., k = 2) {
              L <- list(...)
              if (length(L)) {
                  L <- c(list(object),L)
                  logLiks <- lapply(L, logLik)
                  AICs <- sapply(logLiks,AIC,k=k)
                  df <- sapply(logLiks,attr,"df")
                  data.frame(AIC=AICs,df=df)
              } else AIC(logLik(object), k = k)
          })

### quasi- methods

setGeneric("qAICc", function(object, ..., nobs=NULL, dispersion, k=2)
           standardGeneric("qAICc"))

setMethod("qAICc", signature(object="ANY"),
          function(object, ..., nobs=NULL, dispersion, k=2){
              qAICc(object=logLik(object), nobs=nobs, dispersion=dispersion, k=k)
          })

setMethod("qAICc", "mle2",
          function (object, ..., nobs, dispersion, k)  {
              L <- list(...)
              if (length(L)) {
                  L <- c(list(object),L)
                  if (missing(nobs)) {
                      nobs <- sapply(L,nobs)
                  }
                  if (missing(dispersion) && is.null(attr(object,"dispersion")))
                      stop("must specify (over)dispersion coefficient")
                  if (length(unique(nobs))>1)
                      stop("nobs different: must have identical data for all obj
ects")
                  if (length(nobs)==0) stop("must specify nobs")
                  nobs <- nobs[1]
                  logLiks <- sapply(L, logLik)/dispersion
                  df <- sapply(L,attr,"df")+1 ## add one for scale parameter
                  val <- -2*logLiks+k*df+k*df*(df+1)/(nobs-df-1)
                  data.frame(AICc=val,df=df)
              } else {
                  df <- attr(object,"df")
                  c(-2*logLik(object)/dispersion+k*df+k*df*(df+1)/(nobs-df-1))
              }
          })

setMethod("qAICc", signature(object="logLik"),
          function(object, ..., nobs, dispersion, k){
              if (missing(nobs)) {
                  if (is.null(attr(object,"nobs")))
                      stop("number of observations not specified")
                  nobs <- attr(object,"nobs")
              }
              if (missing(dispersion)) {
                  if (is.null(attr(object,"dispersion")))
                      stop("dispersion not specified")
                  dispersion <- attr(object,"dispersion")
              }
              df <- attr(object,"df")+1 ## add one for scale parameter
              -2 * c(object)/dispersion + k*df+2*df*(df+1)/(nobs-df-1)
          })

setGeneric("qAIC", function(object, ..., dispersion, k=2)
           standardGeneric("qAIC"))

setMethod("qAIC", signature(object="ANY"),
          function(object, ..., dispersion, k=2){
              qAIC(object=logLik(object), dispersion=dispersion, k)
          })

setMethod("qAIC", signature(object="logLik"),
          function(object, ..., dispersion, k){
              if (missing(dispersion)) {
                  if (is.null(attr(object,"dispersion")))
                      stop("dispersion not specified")
                  dispersion <- attr(object,"dispersion")
              }
              df <- attr(object,"df")
              -2 * c(object)/dispersion + k*df
          })

setMethod("qAIC", "mle2",
          function (object, ..., dispersion, k=2) {
              L <- list(...)
              if (length(L)) {
                  L <- c(list(object),L)
                  if (!all(sapply(L,class)=="mle2"))
                      stop("all objects in list must be class mle2")
                  logLiks <- lapply(L, logLik)
                  AICs <- sapply(logLiks,qAIC, k=k, dispersion=dispersion)
                  df <- sapply(L,attr,"df")
                  data.frame(AIC=AICs,df=df)
              } else {
                  qAIC(logLik(object), k=k, dispersion=dispersion)
              }
          })

library(bbmle)
set.seed(1001)
f <- factor(rep(1:3,each=50))
kvals <- c(1,2,5)
muvals <- c(10,2,5)
y <- rnbinom(length(f),size=kvals[f],mu=muvals[f])
plot(y)

NLL <- function(p) {
  kvals <- p[1:3]
  muvals <- p[4:6]
  -sum(dnbinom(y,size=kvals[f],mu=muvals[f],log=TRUE))
}
parnames(NLL) <- c("k1","k2","k3","mu1","mu2","mu3")
svec <- c(kvals,muvals)
names(svec) <- parnames(NLL)
m1 <- mle2(NLL,start=svec,vecpar=TRUE)

library(bbmle)
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)

## uses default parameters of LL
fit <- mle2(y~dpois(exp(loglam)),
            data=d,
           start=list(loglam=0),control=list(maxit=2))
pp <- profile(fit)

library(bbmle)
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)
LL <- function(ymax=15, xhalf=6)
    -sum(stats::dpois(y, lambda=ymax/(1+x/xhalf), log=TRUE))
options(digits=3)
mfit0 <- mle2(y~dpois(lambda=exp(interc)),
              start=list(interc=log(mean(y))),data=d)
mfit1 <- mle2(y~dpois(lambda=exp(loglambda)),
              start=list(loglambda=log(mean(y))),data=d)

coef(mfit0)
residuals(mfit0)
AIC(mfit0)
BIC(mfit0)
vcov(mfit0)
## fitted(mfit0)  ## fails, looks for default value
predict(mfit0)  ## FIXME: doesn't expand properly (need implicit lambda~1 formula??)
set.seed(1001)
simulate(mfit0)
anova(mfit0,mfit1)
summary(mfit0)
summary(mfit1)

library(bbmle)
old_opt <- options(digits=3)
tracelevel <- 0

## source("~/lib/R/pkgs/bbmle/pkg/R/mle.R

set.seed(1002)
X <- rexp(1000, rate = 0.0001)
f <- function(X, rate) {
  if (tracelevel>0 && rate<0) cat("rate<0: ",rate,"\n")
  -sum(dexp(X, rate = rate, log = TRUE))
}
if (FALSE) {
  ## L-BFGS-B violates bounds, and gets stuck at lower bound
  m <- mle2(minuslogl = f,
            data = list(X = X),
            start = list(rate = 0.01),
            method = "L-BFGS-B",
            control = list(trace = tracelevel, 
                           parscale = 1e-4),
            lower = c(rate = 1e-9))

  profile(m, std.err=0.0001) ## finds new optimum

  fsc <- function(X, rate) {
    -sum(dexp(X, rate = rate*1e-4, log = TRUE))
  }
  msc <- mle2(minuslogl = fsc,
            data = list(X = X),
            start = list(rate = 100),
            method = "L-BFGS-B",
            control = list(trace = tracelevel),
            lower = c(rate = 1e-5))

  ## does it work if we scale by hand?
  ##   no, identical problem
}

## works fine with a better starting point
m <- mle2(minuslogl = f,
          data = list(X = X),
          start = list(rate = 0.001),
          method = "L-BFGS-B",
          control = list(trace = tracelevel,
                         parscale=1e-4),
              lower = c(rate = 1e-9))
vcov(m)
confint(m)


## works OK despite warnings about 1-dimensional opt. with N-M
(m0 <- mle2(minuslogl = f,
          data = list(X = X),
          start = list(rate = 0.01),
          method = "Nelder-Mead",
          control = list(trace = tracelevel, parscale = 1e-4)))
vcov(m0)

confint(m0)
confint(m0,method="quad")
## very similar (good quadratic surface, not surprising)

m1 <- mle2(minuslogl = f,
          data = list(X = X),
          start = list(rate = 0.01),
          method = "BFGS",
          control = list(trace = tracelevel, parscale = 1e-4))


## gets stuck? will have to investigate ...
m2 <- mle2(minuslogl = f,
           data = list(X = X),
           start = list(rate = 0.01),
           optimizer = "optimize",
           lower=1e-9,upper=0.1)

vcov(m2)
options(old_opt)

## FIXME: abstract to general-purpose code?  (i.e. replace 'fitted' by
                                        #    objective function, parameter vector, optimizer, method, control settings,
##   min val, standard error/Hessian, ...
##
## allow starting values to be set by "mle" (always use mle), "prevfit"
##  (default?), and "extrap" (linear extrapolation from previous two fits)
## 

proffun <- function (fitted, which = 1:p, maxsteps = 100,
                     alpha = 0.01, zmax = sqrt(qchisq(1 - alpha/2, p)),
                     del = zmax/5, trace = FALSE, skiperrs=TRUE,
                     std.err,
                     tol.newmin = 0.001,
                     debug=FALSE,
                     prof.lower, prof.upper, skip.hessian=TRUE,
                     continuation = c("none","naive","linear"),
                     try_harder=FALSE, ...) {
    ## fitted: mle2 object
    ## which: which parameters to profile (numeric or char)
    ## maxsteps: steps to take looking for zmax
    ## alpha: max alpha level
    ## zmax: max log-likelihood difference to search to
    ## del: stepsize
    ## trace:
    ## skiperrs:
    continuation <- match.arg(continuation)
    if (fitted@optimizer=="optimx") {
        fitted@call$method <- fitted@details$method.used
    }
    if (fitted@optimizer=="constrOptim")
        stop("profiling not yet working for constrOptim -- sorry")
    Pnames <- names(fitted@coef)
    p <- length(Pnames)
    if (is.character(which)) which <- match(which,Pnames)
    if (any(is.na(which)))
        stop("parameters not found in model coefficients")
    ## global flag for better fit found inside profile fit
    newpars_found <- FALSE
    if (debug) cat("i","bi","B0[i]","sgn","step","del","std.err[i]","\n")
    pfit <- NULL ## save pfit to implement continuation methods
                 ## for subsequent calls to onestep
    onestep <- function(step,bi) {
        if (missing(bi)) {
            bi <- B0[i] + sgn * step * del * std.err[i]
            if (debug) cat(i,bi,B0[i],sgn,step,del,std.err[i],"\n")
        } else if (debug) cat(bi,"\n")
        fix <- list(bi)
        names(fix) <- p.i
        if (is.null(call$fixed)) call$fixed <- fix
        else call$fixed <- c(eval(call$fixed),fix)
        ## 
        if (continuation!="none") {
            if (continuation != "naive")
                stop("only 'naive' continuation implemented")
            if (!is.null(pfit)) {
                for (nm in setdiff(names(call$start),names(call$fixed))) {
                    call$start[[nm]] <- coef(pfit)[nm]
                }
            }
        }
        ## now try to fit ...
        if (skiperrs) {
            pfit0 <- try(eval(call, environment(fitted)), silent=TRUE)
        } else {
            pfit0 <- eval(call, environment(fitted))
        }
        ok <- !inherits(pfit0,"try-error")
        ## don't overwrite pfit in environment until we know it's OK ...
        if (ok) pfit <<- pfit0
        if (debug && ok) cat(coef(pfit),-logLik(pfit),"\n")
        if(skiperrs && !ok) {
            warning(paste("Error encountered in profile:",pfit0))
            return(NA)
        }
        else {
            ## pfit is current (profile) fit,
            ##   fitted is original fit
            ## pfit@min _should_ be > fitted@min
            ## thus zz below should be >0
            zz <- 2*(pfit@min - fitted@min)
            ri <- pv0
            ri[, names(pfit@coef)] <- pfit@coef
            ri[, p.i] <- bi
            ##cat(2*pfit@min,2*fitted@min,zz,
            ##   tol.newmin,zz<(-tol.newmin),"\n")
            if (!is.na(zz) && zz<0) {
                if (zz > (-tol.newmin)) {
                    z <- 0
                    ## HACK for non-monotonic profiles? z <- -sgn*sqrt(abs(zz))
                } else {
                    ## cat() instead of warning(); FIXME use message() instead???
                    ## FIXME:  why??? shouldn't this be a warning?
                    message("Profiling has found a better solution,",
                            "so original fit had not converged:\n")
                    message(sprintf("(new deviance=%1.4g, old deviance=%1.4g, diff=%1.4g)",
                                    2*pfit@min,2*fitted@min,2*(pfit@min-fitted@min)),"\n")
                    message("Returning better fit ...\n")
                    ## need to return parameters all the way up
                    ##   to top level
                    newpars_found <<- TRUE
                    ## return(pfit@fullcoef)
                    if (!try_harder) return(pfit) ## bail out, return full fit
                }
            } else {
                z <- sgn * sqrt(zz)
            }
            pvi <<- rbind(pvi, ri)
            zi <<- c(zi, z) ## nb GLOBAL set
        }
        if (trace) cat(bi, z, "\n")
        return(z)
    } ## end onestep
    ## Profile the likelihood around its maximum
    ## Based on profile.glm in MASS
    ## suppressWarnings (don't want to know e.g. about bad vcov)
    summ <- suppressWarnings(summary(fitted))
    if (missing(std.err)) {
        std.err <- summ@coef[, "Std. Error"]
    } else {
        n <- length(summ@coef)
        if (length(std.err)<n)
            std.err <- rep(std.err,length.out=length(summ@coef))
        if (any(is.na(std.err)))
            std.err[is.na(std.err)] <- summ@coef[is.na(std.err)]
    }
    if (any(is.na(std.err))) {
        std.err[is.na(std.err)] <- sqrt(1/diag(fitted@details$hessian))[is.na(std.err)]
        if (any(is.na(std.err))) {  ## still bad
            stop("Hessian is ill-behaved or missing, ",
                 "can't find an initial estimate of std. error ",
                 "(consider specifying std.err in profile call)")
        }
        ## warn anyway ...
        warning("Non-positive-definite Hessian, ",
                "attempting initial std err estimate from diagonals")
    }
    Pnames <- names(B0 <- fitted@coef)
    pv0 <- t(as.matrix(B0))
    p <- length(Pnames)
    prof <- vector("list", length = length(which))
    names(prof) <- Pnames[which]
    call <- fitted@call
    call$skip.hessian <- skip.hessian ## BMB: experimental
    call$minuslogl <- fitted@minuslogl
    ndeps <- eval.parent(call$control$ndeps)
    parscale <- eval.parent(call$control$parscale)
    nc <- length(fitted@coef)
    xf <- function(x) rep(x,length.out=nc) ## expand to length
    upper <- xf(unlist(eval.parent(call$upper)))
    lower <- xf(unlist(eval.parent(call$lower)))
    if (all(upper==Inf & lower==-Inf)) {
        lower <- upper <- NULL
        ## kluge: lower/upper may have been set to +/- Inf
        ##   in previous rounds, 
        ##  but we don't want them in that case
    }
    if (!missing(prof.lower)) prof.lower <- xf(prof.lower)
    if (!missing(prof.upper)) prof.upper <- xf(prof.upper)
    ## cat("upper\n")
    ## print(upper)
    stop_msg <- list()
    for (i in which) {
        zi <- 0
        pvi <- pv0
        p.i <- Pnames[i]
        wfun <- function(txt) paste(txt," (",p.i,")",sep="")
        ## omit values from control vectors:
        ##   is this necessary/correct?
        if (!is.null(ndeps)) call$control$ndeps <- ndeps[-i]
        if (!is.null(parscale)) call$control$parscale <- parscale[-i]
        if (!is.null(upper) && length(upper)>1) call$upper <- upper[-i]
        if (!is.null(lower) && length(lower)>1) call$lower <- lower[-i]
        stop_msg[[i]] <- list(down="",up="")
        for (sgn in c(-1, 1)) {
            pfit <- NULL ## reset for continuation method
            dir_ind <- (sgn+1)/2+1 ## (-1,1) -> (1,2)
            if (trace) {
                cat("\nParameter:", p.i, c("down", "up")[dir_ind], "\n")
                cat("par val","sqrt(dev diff)\n")
            }
            step <- 0
            z <- 0
            ## This logic was a bit frail in some cases with
            ## high parameter curvature. We should probably at least
            ## do something about cases where the mle2 call fails
            ## because the parameter gets stepped outside the domain.
            ## (We now have.)
            call$start <- as.list(B0)
            lastz <- 0
            valf <- function(b) {
                (!is.null(b) && length(b)>1) ||
                    (length(b)==1 && i==1 && is.finite(b))
            }
            lbound <- if (!missing(prof.lower)) {
                          prof.lower[i]
                      } else if (valf(lower))
                      { lower[i]
                      } else -Inf
            ubound <- if (!missing(prof.upper)) prof.upper[i] else if (valf(upper)) upper[i] else Inf
            stop_bound <- stop_na <- stop_cutoff <- stop_flat <- FALSE
            while ((step <- step + 1) < maxsteps &&
                   ## added is.na() test for try_harder case
                   ## FIXME: add unit test!
                   (is.na(z) || abs(z) < zmax)) {
                       curval <- B0[i] + sgn * step * del * std.err[i]
                       if ((sgn==-1 & curval<lbound) ||
                           (sgn==1 && curval>ubound)) {
                           stop_bound <- TRUE;
                           stop_msg[[i]][[dir_ind]] <- paste(stop_msg[[i]][[dir_ind]],wfun("hit bound"))
                           break
                       }
                       z <- onestep(step)
                       if (newpars_found && !try_harder) return(pfit)
                       ## stop on flat spot, unless try_harder
                       if (step>1 && (identical(oldcurval,curval) || identical(oldz,z))) {
                           stop_flat <- TRUE
                           stop_msg[[i]][[dir_ind]] <- paste(stop_msg[[i]][[dir_ind]],wfun("hit flat spot"),
                                                             sep=";")
                           if (!try_harder) break
                       }
                       oldcurval <- curval
                       oldz <- z
                       if(is.na(z)) {
                           stop_na <- TRUE
                           stop_msg[[i]][[dir_ind]] <- paste(stop_msg[[i]][[dir_ind]],wfun("hit NA"),sep=";")
                           if (!try_harder) break
                       }
                       lastz <- z
                   }
            stop_cutoff <- (!is.na(z) && abs(z)>=zmax)
            stop_maxstep <- (step==maxsteps)
            if (stop_maxstep) stop_msg[[i]][[dir_ind]] <- paste(stop_msg[[i]][[dir_ind]],wfun("max steps"),sep=";")
            if (debug) {
                if (stop_na) message(wfun("encountered NA"),"\n")
                if (stop_cutoff) message(wfun("above cutoff"),"\n")
            }
            if (stop_flat) {
                warning(wfun("stepsize effectively zero/flat profile"))
            } else {
                if (stop_maxstep) warning(wfun("hit maximum number of steps"))
                if(!stop_cutoff) {
                    if (debug) cat(wfun("haven't got to zmax yet, trying harder"),"\n")
                    stop_msg[[i]][[dir_ind]] <- paste(stop_msg[[i]][[dir_ind]],wfun("past cutoff"),sep=";")
                    ## now let's try a bit harder if we came up short
                    for(dstep in c(0.2, 0.4, 0.6, 0.8, 0.9)) {
                        curval <- B0[i] + sgn * (step-1+dstep) * del * std.err[i]
                        if ((sgn==-1 & curval<lbound) ||
                            (sgn==1 && curval>ubound)) break
                        z <- onestep(step - 1 + dstep)
                        if (newpars_found && !try_harder) return(pfit)
                        if(is.na(z) || abs(z) > zmax) break
                        lastz <- z
                        if (newpars_found && !try_harder) return(pfit)
                    }
                    if (!stop_cutoff && stop_bound) {
                        if (debug) cat(wfun("bounded and didn't make it, try at boundary"),"\n")
                        ## bounded and didn't make it, try at boundary
                        if (sgn==-1 && B0[i]>lbound) z <- onestep(bi=lbound)
                        if (newpars_found && !try_harder) return(pfit)
                        if (sgn==1  && B0[i]<ubound) z <- onestep(bi=ubound)
                        if (newpars_found && !try_harder) return(pfit)
                    }
                } else if (length(zi) < 5) { # try smaller steps
                    if (debug) cat(wfun("try smaller steps"),"\n")
                    stop_msg[[i]][[dir_ind]] <- paste(stop_msg[[i]][[dir_ind]],wfun("took more steps"),sep=";")
                    mxstep <- step - 1
                    step <- 0.5
                    while ((step <- step + 1) < mxstep) {
                        z <- onestep(step)
                        if (newpars_found && !try_harder) return(pfit)
                    }
                } ## smaller steps
            } ## !zero stepsize
        } ## step in both directions
        si <- order(pvi[, i])
        prof[[p.i]] <- data.frame(z = zi[si])
        prof[[p.i]]$par.vals <- pvi[si,, drop=FALSE]
    } ## for i in which
    if (newpars_found) return(pfit)
    return(list(prof=prof,summ=summ,stop_msg=stop_msg))
}

setMethod("profile", "mle2",
function(fitted,...) {
    ## cc <- match.call()
    ## cc[[1]] <- quote(proffun)
    ## prof <- eval.parent(cc)
    prof <- proffun(fitted,...)
    if (is(prof,"mle2")) return(prof)
    newprof <- new("profile.mle2", profile = prof$prof, summary = prof$summ)
    attr(newprof,"stop_msg") <- prof$stop_msg
    newprof
})


setMethod("plot", signature(x="profile.mle2", y="missing"),
          function (x, levels, which=1:p, conf = c(99, 95, 90, 80, 50)/100,
                    plot.confstr = TRUE, confstr = NULL, absVal = TRUE, add = FALSE,
                    col.minval="green", lty.minval=2,
                    col.conf="magenta", lty.conf=2,
                    col.prof="blue", lty.prof=1,
                    xlabs=nm, ylab="z",
                    onepage=TRUE,
                    ask=((prod(par("mfcol")) < length(which)) && dev.interactive() &&
                         !onepage),
                    show.points=FALSE,
                    main, xlim, ylim, ...)
{
    ## Plot profiled likelihood
    ## Based on profile.nls (package stats)
    obj <- x@profile
    nm <- names(obj)
    p <- length(nm)
    ## need to save these for i>1 below
    no.xlim <- missing(xlim)
    no.ylim <- missing(ylim)    
    if (is.character(which)) which <- match(which,nm)
    ask_orig <- par(ask=ask)
    op <- list(ask=ask_orig)
    if (onepage) {
        nplots <- length(which)
        ## Q: should we reset par(mfrow), or par(mfg), anyway?
        if (prod(par("mfcol")) < nplots) {
            rows <- ceiling(round(sqrt(nplots)))
            columns <- ceiling(nplots/rows)
            mfrow_orig <- par(mfrow=c(rows,columns))
            op <- c(op,mfrow_orig)
        }
    }
    on.exit(par(op))
    confstr <- NULL
    if (missing(levels)) {
        levels <- sqrt(qchisq(pmax(0, pmin(1, conf)), 1))
        confstr <- paste(format(100 * conf), "%", sep = "")
    }
    if (any(levels <= 0)) {
        levels <- levels[levels > 0]
        warning("levels truncated to positive values only")
    }
    if (is.null(confstr)) {
        confstr <- paste(format(100 * pchisq(levels^2, 1)), "%", sep = "")
    }
    mlev <- max(levels) * 1.05
    ##    opar <- par(mar = c(5, 4, 1, 1) + 0.1)
    if (!missing(xlabs) && length(which)<length(nm)) {
        xl2 = nm
        xl2[which] <- xlabs
        xlabs <- xl2
    }
    if (missing(main)) 
        main <- paste("Likelihood profile:",nm)
    main <- rep(main,length=length(nm))
    for (i in seq(along = nm)[which]) {
        ## <FIXME> This does not need to be monotonic
        ## cat("**",i,obj[[i]]$par.vals[,i],obj[[i]]$z,"\n")
        ## FIXME: reconcile this with confint!
        yvals <- obj[[i]]$par.vals[,nm[i],drop=FALSE]
        avals <- data.frame(x=unname(yvals), y=obj[[i]]$z)
        if (!all(diff(obj[[i]]$z)>0)) {
            warning("non-monotonic profile: reverting to linear interpolation.  Consider setting std.err manually")
            predback <- approxfun(obj[[i]]$z,yvals)
        } else {
            sp <- splines::interpSpline(yvals, obj[[i]]$z,
                                        na.action=na.omit)
            avals <- rbind(avals,as.data.frame(predict(sp)))
            avals <- avals[order(avals$x),]
            bsp <- try(splines::backSpline(sp),silent=TRUE)
            bsp.OK <- !inherits(class(bsp),"try-error")
            if (bsp.OK) {
                predback <- function(y) { predict(bsp,y)$y }
            } else { ## backspline failed
                warning("backspline failed: using uniroot(), confidence limits may be unreliable")
                ## what do we do?
                ## attempt to use uniroot
                predback <- function(y) {
                    pfun0 <- function(z1) {
                        t1 <- try(uniroot(function(z) {
                            predict(sp,z)$y-z1
                        }, range(obj[[i]]$par.vals[,nm[i]])),silent=TRUE)
                        if (inherits(t1,"try-error")) NA else t1$root
                    }
                    sapply(y,pfun0)
                }
            }
        }
        ## </FIXME>
        if (no.xlim) xlim <- sort(predback(c(-mlev, mlev)))
        xvals <- obj[[i]]$par.vals[,nm[i]]
        if (is.na(xlim[1]))
            xlim[1] <- min(xvals)
        if (is.na(xlim[2]))
            xlim[2] <- max(xvals)
        if (absVal) {
            if (!add) {
                if (no.ylim) ylim <- c(0,mlev)
                plot(abs(obj[[i]]$z) ~ xvals, 
                     xlab = xlabs[i],
                     ylab = if (missing(ylab)) expression(abs(z)) else ylab,
                     xlim = xlim, ylim = ylim,
                     type = "n", main=main[i], ...)
            }
            avals$y <- abs(avals$y)
            lines(avals, col = col.prof, lty=lty.prof)
            if (show.points) points(yvals,abs(obj[[i]]$z))
        } else { ## not absVal
            if (!add) {
                if (no.ylim) ylim <- c(-mlev,mlev)
                plot(obj[[i]]$z ~ xvals,  xlab = xlabs[i],
                     ylim = ylim, xlim = xlim,
                     ylab = if (missing(ylab)) expression(z) else ylab,
                     type = "n", main=main[i], ...)
            }
            lines(avals, col = col.prof, lty=lty.prof)
            if (show.points) points(yvals,obj[[i]]$z)
        }
        x0 <- predback(0)
        abline(v = x0, h=0, col = col.minval, lty = lty.minval)
        for (j in 1:length(levels)) {
            lev <- levels[j]
            confstr.lev <- confstr[j]
            ## Note: predict may return NA if we didn't profile
            ## far enough in either direction. That's OK for the
            ## "h" part of the plot, but the horizontal line made
            ## with "l" disappears.
            pred <- predback(c(-lev, lev))
            ## horizontal
            if (absVal) levs=rep(lev,2) else levs=c(-lev,lev)
            lines(pred, levs, type = "h", col = col.conf, lty = 2)
            ## vertical
            pred <- ifelse(is.na(pred), xlim, pred)
            if (absVal) {
                lines(pred, rep(lev, 2), type = "l", col = col.conf, lty = lty.conf)
            } else {
                lines(c(x0,pred[2]), rep(lev, 2), type = "l", col = col.conf, lty = lty.conf)
                lines(c(pred[1],x0), rep(-lev, 2), type = "l", col = col.conf, lty = lty.conf)
            }
            if (plot.confstr) {
                text(labels=confstr.lev,x=x0,y=lev,col=col.conf)
            }
        } ## loop over levels
    } ## loop over variables
    ## par(opar)
})

## from Eric Weese
library(bbmle)
f <- function(x=2,a=1) x^2 - a
f.g <- function(x,a) 2*x
f.g2 <- function(x,a) c(2*x,0)
options(digits=3)
m1 <- mle2(f,fixed=list(a=1))
m2 <- mle2(f,gr=f.g,fixed=list(a=1))
m3 <- mle2(f,gr=f.g2,fixed=list(a=1))
stopifnot(all.equal(coef(m1),coef(m2)))
stopifnot(all.equal(coef(m1),coef(m3)))
tt <- function(x) x@details$hessian
stopifnot(all.equal(tt(m1),tt(m2),tolerance=1e-6))
stopifnot(all.equal(tt(m1),tt(m3),tolerance=1e-6))

library(bbmle)
set.seed(1002)
lymax <- c(0,2)
lhalf <- 0
x <- runif(200)
g <- factor(rep(c("a","b"),each=100))
y <- rnbinom(200,mu=exp(lymax[g])/(1+x/exp(lhalf)),size=2)
d <- data.frame(x,g,y)

fit3 <- mle2(y~dnbinom(mu=exp(lymax)/(1+x/exp(lhalf)),size=exp(logk)),
    parameters=list(lymax~g),
    start=list(lymax=0,lhalf=0,logk=0),data=d)

plot(y~x,col=g)
## true curves
curve(exp(0)/(1+x/exp(0)),add=TRUE)
curve(exp(2)/(1+x/exp(0)),col=2,add=TRUE)
xvec = seq(0,1,length=100)
lines(xvec,predict(fit3,newdata=list(g=factor(rep("a",100),levels=c("a","b")),
                                x = xvec)),col=1,lty=2)
lines(xvec,predict(fit3,newdata=list(g=factor(rep("b",100),levels=c("a","b")),
                                x = xvec)),col=2,lty=2)

p1 = predict(fit3)
## manual prediction
p2A =
with(as.list(coef(fit3)),exp(`lymax.(Intercept)`)/(1+x[1:100]/exp(lhalf)))
p2B = with(as.list(coef(fit3)),exp(`lymax.(Intercept)`+lymax.gb)/(1+x[101:200]/exp(lhalf)))
p2 = c(p2A,p2B)
all(p1==p2)

## implement richards-incidence (="revised superlogistic")
##  with analytic gradients

## from Junling's code:
model_richardson <- function(times, theta, N)
{	
	x0 = theta[1]
	lambda = theta[2]
	K = theta[3] * N
	alpha = theta[4]
	return(K/(1+((K/x0)^alpha-1)*exp(-lambda*alpha*times))^(1/alpha))
}

## equivalent model, in terms of sigma and as a symbolic expression
Rcum <- expression((sigma*N)/(1+(((sigma*N)/x0)^alpha-1)*exp(-lambda*alpha*times))^(1/alpha))

pnames <- c("x0","lambda","sigma","alpha")

## function to compute gradient (and value), derived by R
Rderiv <- deriv(Rcum,pnames, function.arg=c(pnames,"N","times"))

## equivalent (using Rcum): return incidence (incid=TRUE) or cumulative incidence (incid=FALSE)
calc_mean <- function(p,times,N,incid=TRUE) {
  ## this is more 'magic' than I would like it to be ...
  ##  have to create an environment and populate it with the contents of p (and N and times),
  ##  then evaluate the expression in this environment
  pp <- c(as.list(p),list(times=times,N=N))
  ##  e0 <- new.env()
  ## mapply(assign,names(pp),pp,MoreArgs=list(envir=e0))
  cumvals <- eval(Rcum,envir=pp)
  if (incid) diff(cumvals) else cumvals
}

## Poisson likelihood function
likfun <- function(p,dat,times,N,incid=TRUE) {
  -sum(dpois(dat,calc_mean(p,times,N,incid=incid),log=TRUE))
}

## deriv of P(x,lambda) = -sum(dpois(x,lambda,log=TRUE)) wrt lambda == sum(1-lambda/x) = N - lambda/(sum(x))
## deriv of P(x,lambda) wrt p = dP/d(lambda) * d(lambda)/dp

## compute gradient vector
gradlikfun <- function(p,dat,times,N,incid=TRUE) {
  gcall <- do.call(Rderiv,c(as.list(p),list(times=times,N=N))) ## values + gradient matrix
  lambda <- gcall
  attr(lambda,"gradient") <- NULL
  if (incid) lambda <- diff(lambda)
  gmat <- attr(gcall,"gradient") ## extract gradient
  if (incid) gmat <- apply(gmat,2,diff)  ## differences
  totderiv <- sweep(gmat,MARGIN=1,(1-dat/lambda),"*") ## apply chain rule (multiply columns of gmat by dP/dlambda)
  colSums(totderiv)  ## deriv of summed likelihood = sum of derivs of likelihod
}

N <- 1000
p0 <- c(x0=0.1,lambda=1,sigma=0.5,alpha=0.5)
t0 <- 1:10
## deterministic versions of data  (cumulative and incidence)
dcdat <- model_richardson(t0,p0,N)
ddat <- diff(dcdat)

plot(t0,dcdat)
plot(t0[-1],ddat)

set.seed(1001)
ddat <- rpois(length(ddat),ddat)

likfun(p0,ddat,t0,N)
gradlikfun(p0,ddat,t0,N)

library(numDeriv)
grad(likfun,p0,dat=ddat,times=t0,N=N)  ## finite differences
## matches!

library(bbmle)
parnames(likfun) <- names(p0)


m1 <- mle2(likfun,start=p0,gr=gradlikfun,data=list(times=t0,N=N,dat=ddat),
           vecpar=TRUE)

plot(t0[-1],ddat)
lines(t0[-1],calc_mean(coef(m1),times=t0,N=N))

if (FALSE) {
  ## too slow ..
  pp0 <- profile(m1)
  pp0C <- profile(m1,continuation="naive")
} 

pp1 <- profile(m1,which="lambda")

m0 <- mle2(likfun,start=p0,data=list(times=t0,N=N,dat=ddat),
           vecpar=TRUE)

pp0 <- profile(m0,which="lambda")
par(mfrow=c(1,2))
plot(pp1,show.points=TRUE)
plot(pp0,show.points=TRUE)

library(bbmle)

## Simulate data

set.seed(1)
x <- 1:5
y <- 2*x+1
noise <- rnorm(5, 0, 0.1)
mydata <- data.frame(x = x, y=y+noise)

## Model definition

model <- function(a, b) with(mydata, a*x+b)

## Negative log-likelihood

nll <- function(par) with(mydata, {
  a <- par[1]
  b <- par[2]
  sum(0.5*((y-model(a,b))/0.1)^2)
  
})

gr <- function(par) with(mydata, {
  a <- par[1]
  b <- par[2]
  dnllda <- -sum(((y-model(a,b))/0.1)*x/0.1)
  dnlldb <- -sum(((y-model(a,b))/0.1)*1/0.1)
  return(c(dnllda, dnlldb))
})

## optimization

parnames(nll) <- c("a", "b")
parnames(gr) <- c("a", "b")

fit <- mle2(nll, c(a = 1, b=2), gr=gr)

myprof <- profile(fit)
myprof_c <- profile(fit,continuation="naive")
confint(myprof)
confint(myprof_c)

fit <- mle2(nll, c(a = 1, b=2), gr=gr, skip.hessian=TRUE)
myprof2 <- profile(fit,std.err=c(0.1,0.1))

## incomplete!
model2 <- ~a+b*x+c*x^2
f0 <- deriv(model2,"x",function.arg=c("a","b","c"))
## chain rule
f1 <- function() {
## memoize
lastpar <- NULL
lastval <- NULL
}

f2 <- function(par) {
    if (par==lastpar) {
        return(c(lastval))
    }
    lastpar <<- par
    lastval <<- do.call(f0,par)
    f1(par)
}
f2.gr <- function(par) {
    if (par==lastpar) {
        return(attr(lastval,".grad"))
    }
    lastpar <<- par
    lastval <<- do.call(f0,par)
    f1.gr(par)
}
parnames(f2) <- parnames(f2.gr) <- c("a","b","c")

library(bbmle)
f <- function() {
  maxit <- 1000
  d <- data.frame(x=0:10,
                  y=c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8))
  mle2(y~dpois(lambda=exp(lymax)/(1+x/exp(lhalf))),
   start=list(lymax=0,lhalf=0),
   data=d,
   control=list(maxit=maxit),
   parameters=list(lymax~1,lhalf~1))
}

f2 <- function(method="BFGS") {
    d <- data.frame(x=0:10,
                    y=c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8))
    mle2(y~dpois(lambda=exp(lymax)/(1+x/exp(lhalf))),
         start=list(lymax=0,lhalf=0),
         data=d,
         method=method,
         parameters=list(lymax~1,lhalf~1))
}

m1 <- f()
p <- profile(m1)
## FIXME: check results (need to save in an environment-friendly way!)
print(head(as.data.frame(p)),digits=3)

m2 <- f2()
p2 <- profile(m2)
print(head(as.data.frame(p2)),digits=3)

setGeneric("simulate", function(object, nsim=1, seed=NULL, ...) standardGeneric("simulate"))
setMethod("simulate", "mle2",
          function(object, nsim=1, seed, newdata=NULL,
                   newparams=NULL, ...) {
            if (!is.null(seed)) set.seed(seed)
            if (!is.null(newparams)) {
              object@fullcoef <- newparams
            }
            g <- gfun(object,newdata=newdata, nsim=nsim,op="simulate")
            if (nsim>1) {
              g <- matrix(g,ncol=nsim)
            }
            g
          })

setGeneric("predict", function(object, ...) standardGeneric("predict"))
setMethod("predict", "mle2",
          function(object,newdata=NULL,
                   location="mean",newparams=NULL, ...) {
            if (!is.null(newparams)) {
              object@fullcoef <- newparams
            }
            gfun(object,newdata=newdata,location=location,op="predict")
          })

setGeneric("residuals", function(object, ...) standardGeneric("residuals"))
setMethod("residuals", "mle2",
          function(object,
                   type=c("pearson","response"),
                   location="mean",
                   ...) {
            type <- match.arg(type)
            location <- match.arg(location)
            pred <- predict(object,location)
            ## not sure this will work ...
            obs <- with(object@data,
                        get(gsub("~.+","",object@formula)))
            res <- obs-pred
            if (type=="response") return(res)
            vars <- predict(object,location="variance")
            return(res/sqrt(vars))
          })

## general-purpose function for simulation and
##  prediction (the hard part is evaluating the parameters etc.)
##
gfun <- function(object,newdata=NULL,location=c("mean","median","variance"),
                 nsim,
                 op=c("predict","simulate")) {
  ## notes: should operate on formula
  ## pull out call$formula (not character)
  location <- match.arg(location)
  if (class(try(form <- as.formula(object@call$minuslogl)))!="formula")
    stop("can only use predict() if formula specified")
  LHS <- form[[3]]
  ddist = as.character(LHS[[1]])
  spref <- switch(op,predict="s",simulate="r")
  sdist = gsub("^d",spref,ddist)
  arglist = as.list(LHS)[-1]
  if (!exists(sdist) || !is.function(get(sdist)))
    stop("function ",sdist," does not exist")
  ## evaluate parameters
  ## evaluate sdist [ newdata > coef > data ]
##   if (is.null(object@data)) {
##     comb <- newdata
##   } else {
##     nmatch <- match(names(newdata),names(object@data))
##     comb <- object@data
##     comb[na.omit(nmatch)] <- newdata[!is.na(nmatch)]
##     comb <- c(comb,newdata[is.na(nmatch)])
##   }
##   comb <- c(newdata,object@data)
##   comb <- comb[!duplicated(names(comb))]
##   comb <- comb[sapply(comb,length)>0]
##   rvar <- strsplit(object@formula,"~")[[1]][1]
##   comb <- comb[!names(comb)==rvar] ## remove response variable
  parameters <- eval(object@call$parameters)
  if (!is.null(parameters)) {
    vars <- as.character(sapply(parameters,"[[",2))
    models <-  sapply(parameters,function(z) call.to.char(z[[3]]))
    parameters <- parameters[models!="1"]
    npars <- length(parameters)
    if (npars==0) { ## no non-constant parameters
      parameters <- mmats <- vpos <- NULL
    } else {
      mmats <- list()
      vpos <- list()
      for (i in seq(along=parameters)) {
        vname <- vars[i]
        p <- parameters[[i]]
        p[[2]] <- NULL
        mmat <- with(c(newdata,object@data),
                     model.matrix(p,data=environment()))
        ## c(as.list(newdata),as.list(object@data)))
        pnames <- paste(vname,colnames(mmat),sep=".")
        assign(vname,mmat %*% coef(object)[pnames])
      }
    }
  }
  arglist1 <- lapply(arglist,eval,envir=c(newdata,object@data,
                                    as.list(coef(object))),
                     enclos=sys.frame(sys.nframe()))
  ## HACK: need a way to figure out how many data points there
  ##  are, in the *absence* of an explicit data argument
  ## then replicate constant values to the full length
  if (op=="simulate") {
    if (length(object@data)==0)
      stop("need explicit data argument for simulation")
    ndata <- max(sapply(c(newdata,object@data),length)) ## ???
    arglist1 <- c(arglist1,list(n=ndata*nsim))
  }
  vals <- with(as.list(coef(object)),do.call(sdist,arglist1))
  if (op=="predict") return(vals[[location]]) else return(vals)
}

## must go before setAs to avoid warnings
setClass("mle2", slots=c(call = "language",
                         call.orig = "language",
                         coef = "numeric",
                         fullcoef = "numeric",
                         vcov = "matrix",
                         min = "numeric",
                         details = "list",
                         minuslogl = "function",
                         method = "character",
                         data="list",
                         formula="character",
                         optimizer="character"))

setAs("mle","mle2", function(from,to) {
  new("mle2",
      call=from@call,
      call.orig=from@call,
      coef=from@coef,
      fullcoef=from@fullcoef,
      vcov=from@vcov,
      min=from@min,
      details=from@details,
      minuslogl=from@minuslogl,
      method=from@method,
      data=list(),
      formula="",
      optimizer="optim")
})
                

setClass("summary.mle2", slots=c(call = "language",
                               coef = "matrix",
                               m2logL = "numeric"))

setClass("profile.mle2", slots=c(profile="list",
                                 summary="summary.mle2"))


setClass("slice.mle2", slots=c(profile="list",
                               summary="summary.mle2"))

setIs("profile.mle2", "slice.mle2")

set.seed(1001)
x <- runif(10)
y <- 1000+x+rnorm(10,sd=0.1)
d <- data.frame(x,y)

library(bbmle)
## warning
m1 = mle2(y~dnorm(a+b*x,sd=exp(s)),start=list(a=900,b=1,s=log(0.1)),
  control=list(parscale=c(1000,1,0.1)),data=d)

m2 = mle2(y~dnorm(a+b*x,sd=exp(s)),start=list(a=900,b=1,s=log(0.1)),
  control=list(parscale=c(b=1,a=1000,s=0.1)),data=d)

m3 = mle2(y~dnorm(a+b*x,sd=exp(s)),start=list(a=1,b=1,s=log(0.1)),
  method="L-BFGS-B",lower=c(a=1100,b=2,s=-Inf),data=d)

## warning
m4 = mle2(y~dnorm(a+b*x,sd=exp(s)),start=list(b=1,a=1200,s=log(0.1)),
  method="L-BFGS-B",lower=c(2,1100,0.1),data=d)

c1 = coef(m3)[c("a","b","s")]
c2 = coef(m4)[c("a","b","s")]
if (!all(abs(c1-c2)<1e-7)) stop("mismatch")

x <- runif(10)
y <- 1+x+rnorm(10,sd=0.1)
d <- data.frame(x,y)

library(bbmle)
m1 <- mle2(y~dnorm(a+b*x,sd=exp(s)),start=list(a=1,b=1,s=log(0.1)),data=d)

m2 <- mle2(y~dnorm(a+b*x,sd=exp(s)),start=list(a=1,b=1,s=log(0.1)),
  method="L-BFGS-B",lower=c(a=0,b=0,s=-Inf),data=d)

m2F <- mle2(y~dnorm(a+b*x,sd=exp(s)),start=list(a=1,b=1,s=log(0.1)),
  method="L-BFGS-B",lower=c(a=0,b=0,s=-Inf),
  fixed=list(a=1),data=d)

snorm <- function(mean,sd) {
  list(title="Normal",
       mean=mean,sd=sd,
       median=mean,
       mode=mean,
       variance=sd^2,
       sd=sd)
}

sbinom <- function(size,prob) {
  list(title="Binomial",
       prob=prob,size=size,
       mean=prob*size,
       median=qbinom(0.5,size,prob),
       mode=NA,
       variance=size*prob*(1-prob),
       sd=sqrt(size*prob*(1-prob)),
       formula="x*log(prob)+(size-x)*log(1-prob)")
}

sbeta <- function(shape1,shape2) {
  list(title="Beta",
       shape1=shape1,shape2=shape2,
       mean=shape1/(shape1+shape2),
       median=qbeta(0.5,shape1,shape2),
       mode=NA,
       variance=shape1*shape2/((shape1+shape2)^2*(shape1+shape2+1)),
       sd=sqrt(shape1*shape2/((shape1+shape2)^2*(shape1+shape2+1))))
}

snbinom <- function(size,prob,mu) {
    if (missing(mu) && !missing(prob)) {
        mupar <- FALSE
        mu = NA ## FIXME
        warning("STUB in snbinom: calc. mu as a function of prob")
    }
    if (!missing(mu) && missing(prob)) {
        mupar <- TRUE
        prob = size/(size+mu)
    }
    v <- if (mupar) mu+mu^2/size else size*(1-prob)/prob^2
    list(title="Negative binomial",
         prob=prob,mu=mu,size=size,
         mean=if (mupar) mu else size*(1-prob)/prob,
         median= if (mupar) qnbinom(0.5,mu=mu,size) else qnbinom(0.5,prob=prob,size),
         mode=NA,
         variance=v,
         sd=sqrt(v))
}

spois <- function(lambda) {
  list(title="Poisson",
       lambda=lambda,
       mean=lambda,
       median=qpois(0.5,lambda),
       mode=NA,
       variance=lambda,
       sd=sqrt(lambda))      
}

sbetabinom <- function(size,prob,theta) {
  list(title="Beta-binomial",
       prob=prob,size=size,theta=theta,
       mean=prob*size,
       median=NA, ## qbetabinom(0.5,size,prob),
       mode=NA,
       variance=size*prob*(1-prob)/theta,
       sd=sqrt(size*prob*(1-prob)))
}

sgamma <- function(shape,rate=1,scale=1/rate) {
    if (missing(rate)) rate <- 1/scale
    list(title="Gamma",
         mean=shape/rate,sd=sqrt(shape)/rate,
         median=NA,
         mode=NA,
         variance=shape/rate^2)
}

dnorm_n <- function(x,mean,log=FALSE) {
  ssq <- sum((x-mean)^2)
  stats::dnorm(x,mean,sd=sqrt(ssq/length(x)),log=log)
}

slnorm <- function(meanlog, sdlog) {
    list(title="Log-normal",
         median=exp(meanlog),
         mean=exp(meanlog+sdlog^2/2))
}

library(bbmle)
old_opt <- options(digits=3)
if (require(optimx)) {
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)

## breaks, don't try this
## optimx(fn=Lfn,par=c(15,6),method="Rvmmin")

suppressWarnings(m1 <- mle2(minuslogl=y~dpois(lambda=ymax/(1+x/xhalf)),
     start=list(ymax=15,xhalf=6),data=d,
     optimizer="optimx",
           method=c("BFGS","Nelder-Mead","CG")))

## FIXME!! fails (although not with an error, because
##  errors are caught by profiling) due to npar now
## being restricted to >1 in optimx 2012.05.24 ...

suppressWarnings(head(as.data.frame(profile(m1))))
detach("package:optimx")
}
options(old_opt)

## require(methods,quietly=TRUE)  ## for independence from stats4
## require(numDeriv,quietly=TRUE) ## for hessian()

call.to.char <- function(x) {
    ## utility function
    x <- as.list(x)
    if (length(x)>1) x <- x[c(2,1,3)]
    paste(sapply(x,as.character),collapse="")
}

## FIXME: problem with bounds and formulae!
calc_mle2_function <- function(formula,
                               parameters,
                               links,
                               start,
                               parnames,
                               use.deriv=FALSE,
                               data=NULL,
                               trace=FALSE) {
    ## resid=FALSE
    ##  stub: what was I going to use this for ???
    ##  returning residuals rather than mle (e.g. for minpack.nls??)
    RHS <- formula[[3]]
    ddistn <- as.character(RHS[[1]])
    if (ddistn=="dnorm" && !("sd" %in% names(RHS))) {
        warning("using dnorm() with sd implicitly set to 1 is rarely sensible")
    }
    if (ddistn=="dnbinom" && !("mu" %in% names(RHS))) {
    }
    ## need to check on variable order:
    ## should it go according to function/formula,
    ##   not start?
    if (!is.list(data)) stop("must specify data argument",
                             " (as a list or data frame)",
                             " when using formula argument")
    vecstart <- (is.numeric(start))
    if (vecstart) start <- as.list(start) ## expand to a list
    if (missing(parnames) || is.null(parnames)) {
        parnames <- as.list(names(start))
        names(parnames) <- names(start)
    }
    ## hack
    if (!missing(parameters)) {
        ## linear model specified for some parameters
        vars <- as.character(sapply(parameters,"[[",2))
        if (length(parameters)>1) {
            models <-  sapply(parameters,function(z) call.to.char(z[[3]]))
        } else {
            models <- as.character(parameters)
        }
        models <- gsub(" ","",models)
        ## remove intercept-only models
        not.int.only <- models!="1"
        parameters <- parameters[not.int.only]
        vars <- vars[not.int.only]
        npars <- length(parameters)
        if (npars==0) { ## no non-constant parameters
            parameters <- mmats <- vpos <- NULL
        } else {
            ## BUG IN HERE SOMEWHERE, FIXME: SENSITIVE TO ORDER OF 'start'
            mmats <- list() ## model matrices
            vpos <- list()
            pnames0 <- parnames
            names(parnames) <- parnames
            for (i in seq(along=parameters)) {
                vname <- vars[i]      ## name of variable
                p <- parameters[[i]]  ## formula for variable
                p[[2]] <- NULL        ## RHS only
                mmat <- model.matrix(p,data=data)     
                pnames <- paste(vname,colnames(mmat),sep=".")
                parnames[[vname]] <- pnames ## insert into parameter names
                vpos0 <- which(pnames0==vname)
                vposvals <- cumsum(sapply(parnames,length))
                ## fill out start vectors with zeros or replicates as appropriate
                if (length(start[[vname]])==1) {
                    if (length(grep("-1",models[i])>0)) {
                        start[[vname]] <- rep(start[[vname]],length(pnames))
                    } else {
                        start[[vname]] <- c(start[[vname]],
                                            rep(0,length(pnames)-1))
                    }
                }
                ## fix: what if parameters are already correctly specified?
                startpos <- if (vpos0==1) 1 else vposvals[vpos0-1]+1
                vpos[[vname]] <- startpos:vposvals[vpos0]
                mmats[[vname]] <- mmat
            }
        }
    } else parameters <- vars <- mmats <- vpos <- NULL
    if (!missing(links)) {
        stop("parameter link functions not yet implemented")
        for (i in length(links)) {
        }
    }
    parnames <- unlist(parnames)
    start <- as.list(unlist(start)) ## collapse/re-expand (WHY?)
    names(start) <- parnames
    arglist <- as.list(RHS[-1]) ## delete function name
    arglist$parameters <- NULL
    arglist1 <- c(list(x=formula[[2]]),arglist,list(log=TRUE))
    arglist1  ## codetools check kluge
    fn <- function() {
        ## is there a better way to do this?
        ## need to look for parameters etc.
        pars <- unlist(as.list(match.call())[-1])
        if (!is.null(parameters)) {
            for (.i in seq(along=parameters)) {
                assign(vars[.i],mmats[[.i]] %*% pars[vpos[[.i]]])
            }
        }
        ## if (is.null(data) || !is.list(data))
        ## stop("data argument must be specified when using formula interface")
        ## BUG/FIXME: data evaluates to 'FALSE' at this point -- regardless of whether
        ## it has been specified
        ## FIXME: how to make this eval() less fragile???
        ## sys.frame(sys.nframe()) specifies the number of the *current* frame
        ## ... envir=data,enclos=parent.frame()
        ## this actually works OK: fails enigmatically if we
        ## 
        arglist2 <- lapply(arglist1,eval,envir=data,
                           enclos=sys.frame(sys.nframe()))
        if (use.deriv) {
            stop("use.deriv is not yet implemented")
            ## browser()
            ## minor hack -- should store information otherwise -- could have
            ##  different numbers of arguments for different distributions?
            LLform <- get(gsub("^d","s",as.character(RHS[[1]])))(NA,NA)$formula
            avals <- as.list(formula[[3]][-1])
            for (i in seq_along(avals))
                LLform <- gsub(names(avals)[i],avals[[i]],LLform)
            r <- eval(deriv(parse(text=LLform),parnames),envir=c(arglist2,data))
        } else {
            r <- -sum(do.call(ddistn,arglist2))
        }
        ## doesn't work yet -- need to eval arglist in the right env ...
        ## if (debugfn) cat(unlist(arglist),r,"\n")
        if (trace) cat(pars,r,"\n")
        r
    }
    npars <- length(parnames)
    flist <-  vector("list",npars)
    names(flist) <- parnames
    ## add additional parnames?
    ## browser()
    ## flist <- c(flist,setdiff(names(arglist),c("x","log",... ?))
    formals(fn) <- flist
    if (vecstart) start <- unlist(start)
    list(fn=fn,start=start,parameters=parameters,
         fdata=list(vars=vars,mmats=mmats,vpos=vpos,
                    arglist1=arglist1,ddistn=ddistn,parameters=parameters),
         parnames=parnames)
}

## need logic that will identify correctly when
## we need to pass parameters as a vector
mle2 <- function(minuslogl,
                 start,  ## =formals(minuslogl),
                 method,
                 optimizer,
                 fixed=NULL,
                 data=NULL,
                 subset=NULL,
                 default.start=TRUE, 
                 eval.only = FALSE,
                 vecpar = FALSE,
                 parameters=NULL,
                 parnames=NULL,
                 skip.hessian=FALSE,
                 hessian.opts=NULL,
                 use.ginv=TRUE,
                 trace=FALSE,
                 browse_obj=FALSE,
                 gr=NULL,
                 optimfun,
                 ...) {

    if (missing(method)) method <- mle2.options("optim.method")
    if (missing(optimizer)) optimizer <- mle2.options("optimizer")
    L <- list(...)
    if (optimizer=="optimize" && (is.null(L$lower) || is.null(L$upper)))
        stop("lower and upper bounds must be specified when using
'optimize'")
    if (inherits(minuslogl,"formula")) {
        pf <- function(f) {if (is.null(f))
                           {  ""
                           } else {
                               paste(f[2],"~",
                                     gsub(" ","",as.character(f[3])),sep="")
                           }
        }
        if (missing(parameters)) {
            formula <- pf(minuslogl)
        } else {
            formula <- paste(pf(minuslogl),
                             paste(sapply(parameters,pf),collapse=", "),sep=": ")
        }
        tmp <- calc_mle2_function(minuslogl,parameters,
                                  start=start,
                                  parnames=parnames,
                                  data=data,trace=trace)
        minuslogl <- tmp$fn
        start <- tmp$start
        fdata <- tmp$fdata
        parameters <- tmp$parameters
    } else {
        formula <- ""
        fdata <- NULL
    }
    call <- match.call()
    call.orig <- call
    ## ?? still not sure this is the best thing to do, but:
    ##   evaluate all elements of call
    ##    to make sure it will still function in new environments ...
    ## call[-1] <- lapply(call[-1],eval.parent)
    ## call[-1] <- lapply(call[-1],eval,envir=parent.frame(),enclos=parent.frame(2))
    ## FAILS if embedded in a funny environment (e.g. called from lapply)
    ##  why do we need this in the first place?
    ## FIXME: change update(), profile() to re-fit model properly
    ##  rather than evaluating call(), or generally find a less-fragile
    ##  way to do this.  Reverting to original form for now.
    call$data <- eval.parent(call$data)
    call$upper <- eval.parent(call$upper)
    call$lower <- eval.parent(call$lower)
    call$gr <- eval.parent(call$gr)
    ## FIX based on request from Mark Clements
    ## call$control$parscale <- eval.parent(call$control$parscale)
    ## call$control$ndeps <- eval.parent(call$control$ndeps)
    ## call$control$maxit <- eval.parent(call$control$maxit)
    call$control <- eval.parent(call$control)
    call$method <- eval.parent(call$method)
    if(!missing(start))
        if (!is.list(start)) {
            if (is.null(names(start)) || !is.vector(start))
                stop("'start' must be a named vector or named list")
            ## do we want this or not???
            vecpar <- call$vecpar <- TRUE  ## given a vector start: set vecpar=TRUE
            start <- as.list(start)
        }
    ## also check parnames(minuslogl)?
    if (missing(start) && default.start) start <- formals(minuslogl)
    call$start <- eval.parent(call$start)
    if (!is.null(fixed) && !is.list(fixed)) {
        if (is.null(names(fixed)) || !is.vector(fixed))
            stop("'fixed' must be a named vector or named list")
        fixed <- as.list(fixed)
    }
    call$fixed <- eval.parent(call$fixed)
    if (!is.null(data) && !is.list(data)) ##  && !is.environment(data)) 
        stop("'data' must be a list")
    nfix <- names(unlist(namedrop(fixed)))
    if (!is.null(parnames(minuslogl))) {
        nfull <- parnames(minuslogl)
        fullcoef <- vector("list",length(nfull))
        names(fullcoef) <- nfull
    } else {
        fullcoef <- formals(minuslogl)
        nfull <- names(fullcoef)
    }
    if(any(! nfix %in% nfull))
        stop("some named arguments in 'fixed' are not arguments to the specified log-likelihood function")
    if (length(nfix)>0) start[nfix] <- NULL
    fullcoef[nfix] <- fixed
    ## switched namedrop() from outside to inside sapply ?
    nstart <- names(unlist(sapply(namedrop(start),eval.parent)))
    fullcoef[! nfull %in% nfix & ! nfull %in% nstart ] <- NULL  ## delete unnecessary names
    nfull <- names(fullcoef)
    lc <- length(call$lower)
    lu <- length(call$upper)
    npnfix <- sum(!nfull %in% nfix)
    if (!npnfix==0 && (lu>npnfix || lc>npnfix )) {
        warning("length mismatch between lower/upper ",
                "and number of non-fixed parameters: ",
                "# lower=",lc,", # upper=",lu,", # non-fixed=",npnfix)
    }
    template <- lapply(start, eval.parent)  ## preserve list structure!
    if (vecpar) template <- unlist(template)
    start <- sapply(namedrop(start), eval.parent) # expressions are allowed; added namedrop
    nstart <- names(unlist(namedrop(start)))
    ## named <- length(names(fullcoef))
    oo <- match(nstart, names(fullcoef))
    if (any(is.na(oo)))
        stop("some named arguments in 'start' are not arguments to the specified log-likelihood function")
    ## if (named)
    start <- start[order(oo)]
    ## rearrange lower/upper to same order as "start"
    ## FIXME: use names to rearrange if present
    fix_order <- function(c1,name,default=NULL) {
        if (!is.null(c1)) {
            if (length(unique(c1))>1) {  ## not all the same
                if (is.null(names(c1)) && length(unique(c1))>1) {
                    warning(name," not named: rearranging to match 'start'")
                    oo2 <- oo
                } else oo2 <- match(names(unlist(namedrop(c1))),names(fullcoef))
                c1 <- c1[order(oo2)]
            }
        } else c1 <- default
        c1
    }
    call$lower <- fix_order(call$lower,"lower bounds",-Inf)
    call$upper <- fix_order(call$upper,"upper bounds",Inf)
    call$control$parscale <- fix_order(call$control$parscale,"parscale")
    call$control$ndeps <- fix_order(call$control$ndeps,"ndeps")
    if (is.null(call$control)) call$control <- list()
    ## attach(data,warn.conflicts=FALSE)
    ## on.exit(detach(data))
    denv <- local(environment(),c(as.list(data),fdata,list(mleenvset=TRUE)))
    ## denv <- local(new.env(),c(as.list(data),fdata,list(mleenvset=TRUE)))
    argnames.in.data <- names(data)[names(data) %in% names(formals(minuslogl))]
    args.in.data <- lapply(argnames.in.data,get,env=denv)
    names(args.in.data) <- argnames.in.data
    args.in.data  ## codetools kluge
    objectivefunction <- function(p){
        if (browse_obj) browser()
        l <- relist2(p,template) ## redo list structure
        ## if (named)
        names(p) <- nstart[order(oo)] ## make sure to reorder
        ## ??? useless, comes after l is constructed ???
        l[nfix] <- fixed
        ##    cat("p\n"); print(p)
        ## cat("l\n"); print(l)
        ##    cat("data\n"); print(data)
        if (vecpar) {
            ## if (named)
            l <- namedrop(l[nfull])
            l <- unlist(l)
            args <- list(l)
            args <- c(list(l),args.in.data)
        } else { args <- c(l,args.in.data)
        }
        ## eval in environment of minuslogl???
        ## doesn't help, environment(minuslogl) is empty by this time
        ## cat("e3:",length(ls(envir=environment(minuslogl))),"\n")
        ## hack to remove unwanted names ...
        do.call("minuslogl",namedrop(args))
    } ## end of objective function
    objectivefunctiongr <-
        if (!is.null(gr))
            function(p) {
                if (browse_obj) browser()
                l <- relist2(p,template) ## redo list structure
                names(p) <- nstart[order(oo)] ## make sure to reorder
                l[nfix] <- fixed
                if (vecpar) {
                    l <- namedrop(l[nfull])
                    l <- unlist(l)
                    args <- list(l)
                    args <- c(list(l),args.in.data)
                } else { args <- c(l,args.in.data)
                }
                v <- do.call("gr",args)
                if (is.null(names(v))) {
                    if (length(v)==length(l) && !is.null(tt <- names(l))) {
                        ## try to set names from template
                        vnames <- tt
                    } else if (length(v)==length(p) && !is.null(tt <- names(p))) {
                        ## try to set names from params
                        vnames <- tt
                    } else if (!is.null(tt <- parnames(minuslogl))) {
                        ## names were set as an attribute of the function
                        vnames <- tt
                    } else vnames <- names(formals(minuslogl))
                    if (length(vnames)!=length(v))
                        stop("name/length mismatch in gradient function")
                    names(v) <- vnames
                }
                return(v[!names(v) %in% nfix]) ## from Eric Weese
            } ## end of gradient function
    ## FIXME: try to do this by assignment into appropriate
    ##    environments rather than replacing them ...
    ## only set env if environment has not been previously set!
    if (!("mleenvset" %in% ls(envir=environment(minuslogl)))) {
        newenv <- new.env(hash=TRUE,parent=environment(minuslogl))
        d <- as.list(denv)
        mapply(assign,names(d),d,
               MoreArgs=list(envir=newenv))
        environment(minuslogl) <- newenv
        if (!is.null(gr)) {
            newenvgr <- new.env(hash=TRUE,parent=environment(minuslogl))
            mapply(assign,names(d),d,
                   MoreArgs=list(envir=newenvgr))
            environment(gr) <- newenvgr
        }
    }
    if (length(start)==0 || eval.only) {
        if (length(start)==0) start <- numeric(0)
        optimizer <- "none"
        skip.hessian <- TRUE
        oout <- list(par=start, value=objectivefunction(start),
                     hessian = matrix(NA,nrow=length(start),ncol=length(start)),
                     convergence=0)
    } else {
        oout <- switch(optimizer,
                       optim = {
            arglist <- list(...)
            arglist$lower <- arglist$upper <-
                arglist$control <- NULL
            do.call("optim",
                    c(list(par=start,
                           fn=objectivefunction,
                           method=method,
                           hessian=FALSE,
                           gr=objectivefunctiongr,
                           control=call$control,
                           lower=call$lower,
                           upper=call$upper),
                      arglist))
        },
        optimx = {
            ## don't ask, will get us into
            ##   dependency hell
            ## require("optimx")
            arglist <- list(...)
            arglist$lower <- arglist$upper <-
                arglist$control <- NULL
            do.call("optimx",
                    c(list(par=start,
                           fn=objectivefunction,
                           method=method,
                           hessian=FALSE,
                           gr=objectivefunctiongr,
                           control=call$control,
                           lower=call$lower,
                           upper=call$upper),
                      arglist))
        },
        nlm = nlm(f=objectivefunction, p=start, hessian=FALSE, ...),
        ##!skip.hessian,
        ## <http://netlib.bell-labs.com/cm/cs/cstr/153.pdf>
        nlminb = nlminb(start=start,
                        objective=objectivefunction, hessian=NULL, ...),
        constrOptim = constrOptim(theta=start,
                                  f=objectivefunction, method=method, ...),
        optimize=,
        optimise= optimize(f=objectivefunction,
                           interval=c(call$lower,call$upper), ...),
        user = {
            arglist <- list(...)
            arglist$lower <- arglist$upper <-
                arglist$control <- NULL
            do.call(optimfun,
                    c(list(par=start,
                           fn=objectivefunction,
                           method=method,
                           hessian=FALSE,
                           gr=objectivefunctiongr,
                           control=call$control,
                           lower=call$lower,
                           upper=call$upper),
                      arglist))
        },
        stop("unknown optimizer (choices are 'optim', 'nlm', 'nlminb', 'constrOptim', 'user', and 'optimi[sz]e')")
        )
    }
    optimval <- switch(optimizer,
                       optim= , constrOptim=, optimx=, user=, none="value",
                       nlm="minimum",
                       optimize=, optimise=, nlminb="objective")
    if (optimizer=="optimx") {
        fvals <- oout[["value"]]
        conv <- oout[["convcode"]]
        ## best <- if (!any(conv==0)) {
        best <- which.min(fvals)
        ##    } else {
        ## fvals <- fvals[conv==0]
        ## which.min(fvals)
        ## }
        oout <- list(par=as.numeric(unlist(oout[best,1:attr(oout,"npar")])),
                     value=fvals[best],
                     convergence=conv[best],
                     method.used=attr(oout,"details")[,"method"][[best]])
        ## FIXME: should do profiles only with best method for MLE?
    }
    if (optimizer=="nlm") {
        oout$par <- oout$estimate
        oout$convergence <- oout$code
    }
    if (optimizer %in% c("optimise","optimize")) {
        oout$par <- oout$minimum
        oout$convergence <- 0 ## can't detect non-convergence
    }
    if (optimizer %in% c("nlminb","optimise","optimize") ||
        ## optimizer (bobyqa?) may have stripped names -- try to restore them!
        is.null(names(oout$par))) {
        names(oout$par) <- names(start)
    }

    ## compute Hessian
    if (length(oout$par)==0) skip.hessian <- TRUE
    if (!skip.hessian) {
        if ((!is.null(call$upper) || !is.null(call$lower)) &&
            any(oout$par==call$upper) || any(oout$par==call$lower))
            warning("some parameters are on the boundary: variance-covariance calculations based on Hessian may be unreliable")
    }
    namatrix <- matrix(NA,nrow=length(start),ncol=length(start))
    if (!skip.hessian) {
        psc <- call$control$parscale
        if (is.null(gr)) {
            if (is.null(psc)) {
                oout$hessian <- try(hessian(objectivefunction,oout$par,
                                            method.args=hessian.opts))
            } else {
                tmpf <- function(x) {
                    objectivefunction(x*psc)
                }
                oout$hessian <- try(hessian(tmpf,oout$par/psc,
                                            method.args=hessian.opts))/outer(psc,psc)
            }
        } else { ## gradient provided
            if (is.null(psc)) {
                oout$hessian <- try(jacobian(objectivefunctiongr,oout$par,
                                             method.args=hessian.opts))
            } else {
                tmpf <- function(x) {
                    objectivefunctiongr(x*psc)
                }
                oout$hessian <- try(jacobian(tmpf,oout$par/psc,
                                             method.args=hessian.opts))/outer(psc,psc)
            }
        }
    }
    if (skip.hessian || inherits(oout$hessian,"try-error"))
        oout$hessian <- namatrix
    coef <- oout$par
    nc <- names(coef)
    if (skip.hessian) {
        tvcov <- matrix(NA,length(coef),length(coef))
    } else {
        if (length(coef)) {
            if (use.ginv) {
                tmphess <- try(MASS::ginv(oout$hessian),silent=TRUE)
            } else {
                tmphess <- try(solve(oout$hessian,silent=TRUE))
            }
            if (inherits(tmphess,"try-error")) {
                tvcov <- matrix(NA,length(coef),length(coef))
                warning("couldn't invert Hessian")
            } else tvcov <- tmphess
        } else {
            tvcov <- matrix(numeric(0),0,0)
        }
    }
    dimnames(tvcov) <- list(nc,nc)
    min <-  oout[[optimval]]
    ##  if (named)
    fullcoef[nstart[order(oo)]] <- coef
    ## else fullcoef <- coef
    ## compute termination info
    ## FIXME: should we worry about parscale here??
    if (length(coef)) {
        gradvec <- if (!is.null(gr)) {
                       objectivefunctiongr(coef)
                   } else {
                       if (inherits(tt <- try(grad(objectivefunction,coef),silent=TRUE),
                                    "try-error")) NA else tt
                   }
        oout$maxgrad <-  max(abs(gradvec))
        if (!skip.hessian) {
            if (inherits(ev <- try(eigen(oout$hessian)$value,silent=TRUE),
                         "try-error")) ev <- NA
            oout$eratio <- min(Re(ev))/max(Re(ev))
        }
    }
    if (!is.null(conv <- oout$conv) &&
        ((optimizer=="nlm" && conv>2) ||
         (optimizer!="nlm" && conv!=0))) {
        ## warn of convergence failure
        if (is.null(oout$message)) {
            cmsg <- "unknown convergence failure: refer to optimizer documentation"
            if (optimizer=="optim") {
                if (conv==1) cmsg <- "iteration limit 'maxit' reached"
                if (conv==10) cmsg <- "degenerate Nelder-Mead simplex"
            } else if (optimizer=="nlm") {
                if (conv==3) cmsg <- "last global step failed to locate a point lower than 'estimate': see ?nlm"
                if (conv==4) cmsg <- "iteration limit exceeded"
                if (conv==5) cmsg <- "maximum step size 'stepmax' exceeded five consecutive times: see ?nlm"
            }
        } else cmsg <- oout$message
        warning(paste0("convergence failure: code=",conv," (",cmsg,")"))
    }
    m <- new("mle2", call=call, call.orig=call.orig, coef=coef, fullcoef=unlist(fullcoef), vcov=tvcov,
             min=min, details=oout, minuslogl=minuslogl, method=method,
             optimizer=optimizer,data=as.list(data),formula=formula)
    attr(m,"df") = length(m@coef)
    if (!missing(data)) attr(m,"nobs") = length(data[[1]])
    environment(m) <- parent.frame()
    ## to work with BIC as well
    m
}


get.mnames <- function(Call) {
    xargs <- which(names(Call) %in% names(formals(ICtab))[-1])
    mnames <- as.character(Call)[c(-1,-xargs)]
    if (length(mnames)==1) {
        g <- get(mnames)
        if (is.list(g) && length(g)>1) {
            if (is.null(names(g))) mnames <- paste("model",1:length(g),sep="")
            else mnames <- names(g)
            if (any(duplicated(mnames))) stop("model names must be distinct")
        }
    }
    mnames
}


mle2.options <- function(...) {
    single <- FALSE
    args <- list(...)
    setvals <- !is.null(names(args))
    if (!length(args)) args <- names(.Mle2.options)
    if (all(unlist(lapply(args, is.character)))) 
        args <- as.list(unlist(args))
    if (length(args) == 1) {
        if (is.list(args[[1]]) | is.null(args[[1]])) 
            args <- args[[1]]
        else if (!setvals)
            single <- TRUE
    }
    if (setvals) {
        .Mle2.options[names(args)] <<- args
        value <- .Mle2.options[names(args)]
    } else value <- .Mle2.options[unlist(args)]
    if (single) value <- value[[1]]
    if (setvals) invisible(value) else value
}


.Mle2.options = list(optim.method="BFGS",confint = "spline",optimizer="optim")


## .onLoad <- function(lib, pkg) require(methods)


## (not yet) replaced by relist?
## reconstruct list structure:
##   v is a vector, l is the original list
##      to use as a template
relist2 <- function(v,l) {
    if (is.list(v)) v <- unlist(v)
    if (!all(sapply(l,mode)=="numeric")) {
        stop("can't relist non-numeric values")
    }
    lens = sapply(l,length)
    if (all(lens==1))
        return(as.list(v))
    l2 <- split(v,rep(1:length(l),lens))
    names(l2) <- names(l)
    l3 <- mapply(function(x,y) {
        if (!is.null(dim(y))) {
            z=array(x,dim(y)); dimnames(z)=dimnames(y); z
        } else {
            z=x; names(z)=names(y); z
        }
    },l2,l,SIMPLIFY=FALSE)
    names(l3) <- names(l)
    l3
}

namedrop <- function(x) {
    if (!is.list(x)) x
    for (i in seq(along=x)) {
        ## cat(i,length(x),"\n")
        n = names(x[[i]])
        lx = length(x[[i]])
        if (!is.null(n)) {
            if (lx==1) {
                names(x[[i]]) <- NULL
            } else if (length(unique(n))<lx) {
                names(x[[i]]) <- 1:lx
            }
        } ## !is.null(names(x[[i]]))
    } ## loop over elements
    x
}

"parnames<-" <- function(obj,value) {
    attr(obj,"parnames") <- value
    obj
}

parnames <- function(obj) {
    attr(obj,"parnames")
}


### TEST OF NLMINB
if (FALSE) {
    x <- 0:10
    y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
    mle2(y~dpois(lambda=ymax/(1+x/xhalf)),start=list(ymax=25,xhalf=3.06),
         optimizer="nlminb",fixed=list(ymax=38.76),lower=c(0,0),trace=TRUE)

    f = calc_mle2_function(y~dpois(lambda=ymax/(1+x/xhalf)),
                           start=list(ymax=25,xhalf=3.06))
    f2 = function(xhalf) {
        -sum(dpois(y,38.76/(1+x/xhalf),log=TRUE))
    }
    optim(f2,par=3.06,method="BFGS")
    ## optim(f2,par=3.06,method="L-BFGS-B",lower=0) ## error
    nlminb(objective=f2,start=3.06)
    nlminb(objective=f2,start=3.06,lower=0)
    nlminb(objective=f2,start=3.06,lower=1e-8)
}

strwrapx <- function(x, width = 0.9 * getOption("width"),
                     indent = 0, exdent = 0, 
                     prefix = "", simplify = TRUE,
                     parsplit= "\n[ \t\n]*\n", wordsplit = "[ \t\n]") 
{
    if (!is.character(x)) 
        x <- as.character(x)
    indentString <- paste(rep.int(" ", indent), collapse = "")
    exdentString <- paste(rep.int(" ", exdent), collapse = "")
    y <- list()
    ## split at "+" locations
    plussplit = function(w) {
        lapply(w,
               function(z) {
            plusloc = which(strsplit(z,"")[[1]]=="+")
            plussplit =
                apply(cbind(c(1,plusloc+1),
                            c(plusloc,nchar(z,type="width"))),
                      1,
                      function(b) substr(z,b[1],b[2]))
            plussplit
        })}
    ## ugh!
    z <- lapply(strsplit(x, parsplit),
                function(z) { lapply(strsplit(z,wordsplit),
                                     function(x) unlist(plussplit(x)))
    })
    ## print(str(lapply(strsplit(x,parsplit),strsplit,wordsplit)))
    ## print(str(z))
    for (i in seq_along(z)) {
        yi <- character(0)
        for (j in seq_along(z[[i]])) {
            words <- z[[i]][[j]]
            nc <- nchar(words, type = "w")
            if (any(is.na(nc))) {
                nc0 <- nchar(words)
                nc[is.na(nc)] <- nc0[is.na(nc)]
            }
            if (any(nc == 0)) {
                zLenInd <- which(nc == 0)
                zLenInd <- zLenInd[!(zLenInd %in% (grep("\\.$", 
                                                        words) + 1))]
                if (length(zLenInd) > 0) {
                    words <- words[-zLenInd]
                    nc <- nc[-zLenInd]
                }
            }
            if (length(words) == 0) {
                yi <- c(yi, "", prefix)
                next
            }
            currentIndex <- 0
            lowerBlockIndex <- 1
            upperBlockIndex <- integer(0)
            lens <- cumsum(nc + 1)
            first <- TRUE
            maxLength <- width - nchar(prefix, type = "w") - 
                indent
            while (length(lens) > 0) {
                k <- max(sum(lens <= maxLength), 1)
                if (first) {
                    first <- FALSE
                    maxLength <- maxLength + indent - exdent
                }
                currentIndex <- currentIndex + k
                if (nc[currentIndex] == 0) 
                    upperBlockIndex <- c(upperBlockIndex, currentIndex - 
                                                          1)
                else upperBlockIndex <- c(upperBlockIndex, currentIndex)
                if (length(lens) > k) {
                    if (nc[currentIndex + 1] == 0) {
                        currentIndex <- currentIndex + 1
                        k <- k + 1
                    }
                    lowerBlockIndex <- c(lowerBlockIndex, currentIndex + 
                                                          1)
                }
                if (length(lens) > k) 
                    lens <- lens[-(1:k)] - lens[k]
                else lens <- NULL
            }
            nBlocks <- length(upperBlockIndex)
            s <- paste(prefix, c(indentString, rep.int(exdentString, 
                                                       nBlocks - 1)), sep = "")
            for (k in (1:nBlocks)) {
                s[k] <- paste(s[k],
                              paste(words[lowerBlockIndex[k]:upperBlockIndex[k]], 
                                    collapse = " "), sep = "")
            }
            s = gsub("\\+ ","+",s)  ## kluge
            yi <- c(yi, s, prefix)
        }
        y <- if (length(yi)) 
                 c(y, list(yi[-length(yi)]))
             else c(y, "")
    }
    if (simplify) {
        y <- unlist(y)
    }
    return(y)
}

library(bbmle)
library(testthat)
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)
LL <- function(ymax=15, xhalf=6)
    -sum(stats::dpois(y, lambda=ymax/(1+x/xhalf), log=TRUE))
mfit0 <- mle2(y~dpois(lambda=exp(interc)),
              start=list(interc=log(mean(y))),data=d)

mfit1 <- mle2(y~dpois(lambda=exp(loglambda)),
              start=list(loglambda=log(mean(y))),data=d)
              
gfit0 <- glm(y~1,family=poisson)
expect_equal(unname(coef(mfit0)),unname(coef(gfit0)))
expect_equal(logLik(mfit0),logLik(gfit0))
expect_equal(predict(mfit0),  ## only one value for now
             unique(predict(gfit0,type="response")))

## FIXME: residuals are backwards
expect_equal(residuals(mfit0,type="response"),unname(residuals(gfit0,type="response")))
## FIXME: residuals are backwards
expect_equal(residuals(mfit0,type="pearson"),unname(residuals(gfit0,type="pearson")))

library(bbmle)
set.seed(1002)
lymax <- c(0,2)
lhalf <- 0
x <- runif(200)
g <- factor(rep(c("a","b"),each=100))
y <- rnbinom(200,mu=(exp(lymax[g])/(1+x/exp(lhalf)))^2,size=2)
dd <- data.frame(x,g,y)

fit3 <- mle2(y~dnbinom(mu=(exp(lymax)/(1+x/exp(lhalf)))^d,size=exp(logk)),
    parameters=list(lymax~g),
    start=list(lymax=0,lhalf=0,logk=0,d=NA),
    data=dd,
    fixed=list(d=2))

pp <- pop_pred_samp(fit3,PDify=TRUE)
stopifnot(
    !any(is.na(pp)),
    identical(colnames(pp),
              c("lymax.(Intercept)", "lymax.gb", "lhalf", "logk", "d")))

## fix parameters instead of dealing with negative variance
pp2 <- pop_pred_samp(fit3,fix_param="lhalf")
stopifnot(length(unique(pp2[,"lhalf"]))==1)

library(bbmle)

## copied from emdbook
dbetabinom <- function (x, prob, size, theta, shape1, shape2, log = FALSE) 
{
    if (missing(prob) && !missing(shape1) && !missing(shape2)) {
        prob = shape1/(shape1 + shape2)
        theta = shape1 + shape2
    }
    v <- lchoose(size, x) - lbeta(theta * (1 - prob), theta * 
        prob) + lbeta(size - x + theta * (1 - prob), x + theta * 
        prob)
    if (log) 
        v
    else exp(v)
}

ss <- data.frame(taken=c(0,1,2,5),available=c(5,5,5,5),
                 dist=rep(1,4))

SP.bb=mle2(taken~dbetabinom(prob,theta,size=available),
  start=list(prob=0.5,theta=1),data=ss)
SP.bb.dist=mle2(taken~dbetabinom(prob,size=available,theta),
  parameters=list(prob~dist-1,theta~dist-1),
  start=as.list(coef(SP.bb)),data=ss)

SP.bb.dist2=mle2(taken~dbetabinom(prob,size=available,theta),
  parameters=list(prob~dist - 1,theta~dist - 1),
  start=as.list(coef(SP.bb)),data=ss)

## test whether profiling works when custom optimizer is defined
##  inside a function (GH #7)

library(bbmle)
test <- function(t, X) {
  likfun <- function(p) {
    mu <- with(as.list(p), {
      exp(a+b*t)
    })
    -sum(dpois(X, mu, log=TRUE))
  }
  parnames(likfun) <- c("a", "b")
  
  optimfun <- function(par, fn, gr = NULL, ...,
                       method = NULL, lower = -Inf, upper = Inf,
                       control = NULL, hessian = FALSE) {
    ## cat("using custom optimfun!\n")
    optim(par, fn=fn, gr=gr, ...,
          method="BFGS", control=control, hessian=TRUE)
  }
  
  mle2(likfun, start=c(a=1,b=1), optimizer="user", optimfun=optimfun)
}

f <- test(0:5, round(exp(1:6)))
pp <- profile(f,skiperrs=FALSE)
stopifnot(inherits(pp,"profile.mle2"))

library(bbmle)

x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)
oldopts <- options(warn=-1,digits=3)  ## ignore warnings
m1 <- mle2(y~dpois(lambda=ymax/(1+x/xhalf)),
           start=list(ymax=1,xhalf=1),data=d)
m1
y2 <- c(26, 17, 10, 15, 20, 5, 9, 8, 5, 4, 8)
d2 <- data.frame(x,y=y2)

m2 <- update(m1,data=d2)
m2
m3 <- update(m1,.~dpois(lambda=c),start=list(c=5))
m3
options(oldopts)

require(RUnit)
## TODO -- find solution to run these tests on R-forge

##testsuite <- defineTestSuite("phylobase", dirs="/home/francois/Work/R-dev/phylobase/branches/fm-branch/RUnit-tests",
##                             testFileRegexp="^test", testFuncRegexp="^test")
##testRslt <- runTestSuite(testsuite)
##printTextProtocol(testRslt)

library(bbmle)
old_opt <- options(digits=3)
## source("../R/dists.R")
## source("../R/mle.R")

## an attempt to sketch out by hand
##  how one would derive an analytic
##  gradient function for a formula-specified
##  likelihood and use it ...

## chain rule should be:

## deriv(probability distribution)/[prob params] *
##    deriv([prob params])/[model params] *
##   {OPTIONAL} deriv([model params])/[linear model params]

set.seed(1001)
x <- rbinom(50,size=10,prob=0.4)
suppressWarnings(mle2(x~dbinom(prob=p,size=10),start=list(p=0.3),data=data.frame(x)))

## step 1: construct gradient function for simplest example
f <- sbinom(prob=0.1,size=1)$formula

d1 <- deriv(parse(text=f),"prob",function.arg=TRUE)

## step 2: chain rule step #1
mle2(x~dbinom(prob=plogis(logitp),size=10),start=list(logitp=-1),
     data=data.frame(x))

f <- sbinom(prob=NA,size=NA)$formula

## note: plogis is not in derivatives table!!
##  will need to extend by text substitution ...
gsub("plogis(\\([^)]+\\))",
     "(1+exp(\\1))^(-1)",
     "plogis(logitprob)")

f2 <- gsub("plogis(\\([^)]+\\))",
     "(1+exp(\\1))^(-1)","plogis(logitp)")

## start with a single parameter (ignore 'size')
fun1 <- deriv(parse(text=f),c("prob"),function.arg=TRUE)
fun2 <- deriv(parse(text=f2),"logitp", function.arg=TRUE)

size <- 10
a1 <- attr(fun2(logitp=0),"gradient")
a2 <- attr(fun1(prob=plogis(0)),"gradient")

## compute gradient by variable and sum
colSums(apply(a1,2,"*",a2))
## rep(a1,length(x))*a2


## eventually we will want to do something tricky to
## 'memoise' results because optim() requires the
## objective function and gradient to be computed
## *separately*.  Not worth worrying about this in the
## first pass!
options(old_opt)

library(bbmle)
old_opts <- options(digits=3)
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)
suppressWarnings(fits <- lapply(c("optim","nlm","nlminb"),
               mle2,
               minuslogl=y~dpois(lambda=ymax/(1+x/xhalf)),
               start=list(ymax=15,xhalf=6),data=d,
               method="Nelder-Mead")) ## 'method' is ignored by nlm()/nlminb()

sapply(fits,coef)
sapply(fits,logLik)

(fit2 <-  mle2(y~dpois(lambda=25/(1+x/xhalf)),
              start=list(xhalf=5),data=d,
              lower=2,upper=8,
              optimizer="optimize"))

## gives error referring to 'interval' rather than 'upper'/'lower'
## (fit2 <-  mle2(y~dpois(lambda=25/(1+x/xhalf)),
##              start=list(xhalf=5),
##              optimizer="optimize"))
options(old_opts)

library(bbmle)
old_opt <- options(digits=3)
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)


## throws warning on some CRAN system (BDR report 2019-12-20)
## /tests-noLD/bbmle.Rcheck’
## * using R Under development (unstable) (2019-12-19 r77606)
## * using platform: x86_64-pc-linux-gnu (64-bit)
## suppressWarnings() doesn't seem to suppress the warning??
## instead use test from https://www.r-bloggers.com/a-nold-platform-on-r-hub-package-builder/
if (capabilities("long.double")) {
   fit0 <- suppressWarnings(
    mle2(y~dpois(lambda=ymean),start=list(ymean=mean(y)),data=d,
         method="L-BFGS-B",lower=10)
   )
   stopifnot(is.na(confint(fit0)[1]))
}

fit1 <- mle2(y~dpois(lambda=exp(a+b*x)),start=list(a=0,b=2),data=d,
             method="L-BFGS-B",lower=-0.2)

suppressWarnings(confint(fit1))

fit2 <- mle2(y~dpois(lambda=exp(a+b*x)),start=list(a=0,b=2),data=d,
             method="L-BFGS-B")

pp <- suppressWarnings(profile(fit2,prof.lower=-0.2))
stopifnot(min(subset(as.data.frame(pp),param=="b")$par.vals.b)==-0.2)
## note that b does go below -0.2 when profiling a ...
options(old_opt)

## setGeneric("update", function(object, formula.,  ..., evaluate=TRUE)
##           standardGeneric("update"))


## FIXME: compare these two
## setMethod("update", "mle2",
## function (object, ..., evaluate = TRUE)
## {
##   call <- object@call
##   extras <- match.call(expand.dots = FALSE)$...
##   if (length(extras) > 0) {
##     existing <- !is.na(match(names(extras), names(call)))
##     for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
##     if (any(!existing)) {
##       call <- c(as.list(call), extras[!existing])
##       call <- as.call(call)
##     }
##   }
##   if (evaluate) eval(call, parent.frame()) else call
## })

 

## update.default, modified with $ turned to @ as appropriate
setMethod("update", "mle2",
          function (object, formula., evaluate = TRUE, ...) 
          {
            call <- object@call
            extras <- match.call(expand.dots = FALSE)$...
            if (!missing(formula.)) 
              call$minuslogl <- update.formula(formula(object), formula.)
            if (length(extras)) {
              existing <- !is.na(match(names(extras), names(call)))
              for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
              if (any(!existing)) {
                call <- c(as.list(call), extras[!existing])
                call <- as.call(call)
              }
            }
            if (evaluate) 
              eval(call, parent.frame())
            else call
          })

library(bbmle)

funcresp <-
structure(list(Initial = as.integer(c(5, 5, 10, 10, 15, 15, 20, 
20, 30, 30, 50, 50, 75, 75, 100, 100)), Killed = as.integer(c(1, 
2, 5, 6, 10, 9, 7, 10, 11, 15, 5, 21, 32, 18, 25, 35))), .Names = c("Initial", 
"Killed"), class = "data.frame", row.names = c("1", "2", "3", 
"4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", 
"16"))

binomNLL2 = function(p) {
  a = p[1]
  h = p[2]
  ##  cat(a,h,"\n")
  p = a/(1+a*h*N)
  -sum(dbinom(k,prob=p,size=N,log=TRUE))
}

N=0; k=0
parnames(binomNLL2) = c("a","h")
m2a = mle2(binomNLL2,start=c(a=0.5,h=0.0125),
      data=with(funcresp,list(N=Initial,k=Killed)))
p1a = profile(m2a)
c2a = print(confint(p1a),digits=3)

binomNLL2b = function(p,N,k) {
  a = p[1]
  h = p[2]
  ##  cat(a,h,"\n")
  p = a/(1+a*h*N)
  -sum(dbinom(k,prob=p,size=N,log=TRUE))
}
parnames(binomNLL2b) = c("a","h")
m2b = mle2(binomNLL2,start=c(a=0.5,h=0.0125),
  data=with(funcresp,list(N=Initial,k=Killed)))
c2b = confint(m2b)

N=funcresp$Initial; k=funcresp$Killed
m2c = mle2(binomNLL2,start=c(a=0.5,h=0.0125))
c2c = confint(m2c)
print(c2c,digits=3)

## I am experiencing difficulties with one of my modeling function (bbmle::mle2)
## which, like other modeling functions in R, uses match.call() to
## retrieve and save the original function call for future use.
## I'll describe the problem for bbmle and then show that I can
## provoke a similar problem with lm().

## ============
## PART I: mle2()

  library(bbmle)

  x <- 0:10
  y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
  d <- data.frame(x,y)

## The key is to call the modeling function from within another
## function which passes additional arguments via ... 

  ff <- function(d,...) {
    mle2(y~dpois(lambda=ymean),start=list(ymean=mean(y)),data=d,...)
  }

  ff(d)
  try(ff(d,control=list(maxit=1000)))

##   Error in call$control$parscale : 
##      object of type 'symbol' is not subsettable

## This happens when I try:

##  call$control$parscale <- eval.parent(call$control$parscale)

## in 'normal' circumstances call$control and call$control$parscale
## are either NULL or well-specified ...

## Debugging mle2 shows that the results of match.call() are

##   mle2(minuslogl = y ~ dpois(lambda = ymean), start = list(ymean = mean(y)), 
##       data = d, control = ..1)

## ============
## PART II: lm()

## I can find a similar issue with lm(), although admittedly
## I have to work a bit harder/do something a little bit more
## obscure.

  L1 <-  lm(y~1,data=d,tol=1e-6)
  L1$call

  ff2 <- function(d,...) {
    lm(y~1,data=d,...)
  }

  tt <- 1e-6
  L2 <- ff2(d,tol=tt)
  L2$call

  try(update(L2,.~.+x))

## Error in lm.fit(x, y, offset = offset, singular.ok = singular.ok, ...) : 
##   ..1 used in an incorrect context, no ... to look in

 ## similar issue in curve3d().  How does curve() work?

library(bbmle)

set.seed(101)
z = rpois(100,lambda=5)

m1 = mle2(z~dpois(lambda=L),start=list(L=4),data=data.frame(z))
q1 <- qAICc(m1,nobs=100,dispersion=1.2)
qAICc(m1,m1,nobs=100,dispersion=1.2) ## !!
i1 <- ICtab(m1,type="qAICc",dispersion=1.2,nobs=100, base=TRUE)

m2 = glm(z~1,family=poisson)
q2 <- qAICc(m2,nobs=100,dispersion=1.2)

## test that dAIC ignores
m3 <- glm(z~1,family=quasipoisson)
aa <- AICtab(m1,m2,m3,weights=TRUE)
stopifnot(any(!is.na(aa$dAIC)),
          any(!is.na(aa$weight)))

set.seed(101)
x <- rnorm(100)
dd <- data.frame(y=rnorm(100,2+3*x,sd=1),x)
m4A <- lm(y~x,dd)
m4B <- mle2(y~dnorm(mean=a+b*x,sd=exp(logsd)),
           data=dd,
           start=list(a=1,b=1,logsd=0))
## cosmetic differences only
stopifnot(all.equal(AIC(m4A,m4B)[,"AIC"],
                    AIC(m4B,m4A)[,"AIC"]))

## set up a data frame for prediction

set.seed(1001)
f = factor(rep(letters[1:4],each=20))
x = runif(80)
u = rnorm(4)
y = rnorm(80,mean=2+x*(3+u[f]),sd=0.1)
dat = data.frame(f,x,y)

## fit a model ... could easily do by lm() but want to
##   demonstrate the problem

library(bbmle)
m1 = mle2(y~dnorm(a+b*x,sd=exp(logs)),parameters=list(b~f),data=dat,
  start=list(a=0,b=2,logs=-3))

## data frame for prediction
pp0 = expand.grid(x=seq(0,1,length=11),
  f=levels(dat$f))

## combine frame and model data: have to keep the model data
##  around, because it contain other information needed for
##  prediction.

nrow(predict(m1,pp0))

## STUB: starting to play with/think about TMB back-end
form <- quote({
    A <- exp(-alpha*t)
})
recruits ~ dnorm(settlers*A/(1+(beta*settlers*(1-A)/alpha)), sd)
TMB_mle2_function <- function(formula,
                              parameters,
                              links,
                              start,
                              parnames,
                              data=NULL) {

    ## find symbols

    

}

require(bbmle)
mle2a <- function(...)
  mle2(...)

mle2b <- function(...)
  mle2a(...)

## some data
d <- data.frame(x=0:10,y=c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8))
ym <- mean(d$y)

## some fits

(fit0 <- mle2(y~dpois(lambda=ymean),start=list(ymean=ym),data=d)) # okay
predict(fit0)
(fit0.2 <- mle2(y~dpois(lambda=ymean),start=list(ymean=ym),data=d,
                control=list(parscale=2))) # okay
predict(fit0.2)
(fit1 <- mle2a(y~dpois(lambda=ymean),start=list(ymean=ym),data=d)) # okay
(fit1.2 <- mle2a(y~dpois(lambda=ymean),start=list(ymean=ym),data=d,
                 control=list(parscale=2))) # FAILS
(fit1.3 <- mle2b(y~dpois(lambda=ymean),start=list(ymean=ym),data=d,
                 control=list(parscale=2))) # FAILS

### NOT WORKING:
if (FALSE) {
  predict(fit1)
  predict(fit1.2)
  predict(fit1.3)
}

setMethod("confint", "profile.mle2",
function (object, parm, level = 0.95, trace=FALSE, ...)
{
  Pnames <- names(object@profile)
  if (missing(parm)) parm <- Pnames
  if (is.character(parm)) parm <- match(parm,Pnames)
  if (any(is.na(parm))) stop("parameters not found in profile")
  ## Calculate confidence intervals based on likelihood
  ## profiles
  a <- (1 - level)/2
  a <- c(a, 1 - a)
  pct <- paste(round(100 * a, 1), "%")
  ci <- array(NA, dim = c(length(parm), 2),
              dimnames = list(Pnames[parm], pct))
  cutoff <- qnorm(a)
  for (pm in parm) {
    pro <- object@profile[[Pnames[pm]]]
    pv <- pro[,"par.vals"]
    if (is.matrix(pv)) pv <- pv[,Pnames[pm]]
    if (any(diff(pro[,1])<0)) {
      warning(paste("non-monotonic profile (",
                    Pnames[pm],"): reverting from spline to linear approximation ",
              "(consider running 'profile' with manually reduced std.err)", sep=""))
      tt <- approx(pro[,1],pv,xout=cutoff)$y
    } else {
      sp <- spline(x = pv, y = pro[, 1])
      if (any(diff(sp$y)<0)) {
        warning(paste("non-monotonic spline fit to profile (",
                      Pnames[pm],"): reverting from spline to linear approximation",sep=""))
        tt <- approx(pro[,1],pv,xout=cutoff)$y
      } else {
        tt <- try(approx(sp$y, sp$x, xout = cutoff)$y,silent=TRUE)
        if (inherits(tt,"try-error")) tt <- rep(NA,2)
      }
    }
    if (!any(is.na(tt))) {
        ## if NAs present, sort() will drop NAs ...
        tt <- sort(tt)
    }
    ci[Pnames[pm], ] <- tt
  }
  drop(ci)
})

setMethod("confint", "mle2",
function (object, parm, level = 0.95, method,
          trace=FALSE,quietly=!interactive(),
          tol.newmin=0.001,...)
{
  if (missing(method)) method <- mle2.options("confint")
  ## changed coef() calls to object@coef -- really *don't* want fullcoef!
  Pnames <- names(object@coef)
  if (missing(parm))
    parm <- seq(along=Pnames)
  if (is.character(parm)) parm <- match(parm,Pnames)
  if (any(is.na(parm))) stop("parameters not found in model coefficients")
  if (method=="spline") {
    if (!quietly) message("Profiling...\n")
    newpars_found <- FALSE
    prof = try(profile(object,which=parm,tol.newmin=tol.newmin,...))
    if (inherits(prof,"try-error")) stop(paste("Problem with profiling:",prof))
    if (class(prof)=="mle2") newpars_found <- TRUE
    if (newpars_found) {
        ## profiling found a better fit
        message("returning better fit\n")
        return(prof)
    }
    return(confint(prof, parm, level, ...))
  } else {
    B0 <- object@coef
    pnames <- names(B0)
    if (missing(parm))
      parm <- seq(along=pnames)
    if (is.character(parm))
      parm <- match(parm, pnames, nomatch = 0)
    a <- (1 - level)/2
    a <- c(a, 1 - a)
    pct <- paste(round(100 * a, 1), "%")
    pct <- paste(round(100 * a, 1), "%")
    ci <- array(NA, dim = c(length(parm), 2),
                dimnames = list(pnames[parm], pct))
    std.err <- summary(object)@coef[, "Std. Error"]
    if (method=="uniroot") {
      chisqcutoff <- qchisq(level,1)
      call <- object@call
      if (!isTRUE(call$vecpar))
        call$start <- as.list(B0) ## added
      upper <- rep(unlist(eval.parent(call$upper)),length.out=length(pnames))
      lower <- rep(unlist(eval.parent(call$lower)),length.out=length(pnames))
      for (pm in parm) {
        critfun <- function(bi)
          {
            fix <- list(bi)
            names(fix) <- pnames[pm]
            call$fixed <- c(fix,eval(call$fixed))
            if (!is.null(upper) && length(upper)>1) call$upper <- upper[-pm]
            if (!is.null(lower) && length(lower)>1) call$lower <- lower[-pm]
            pfit <- try(eval(call), silent=TRUE)
            if(inherits(pfit, "try-error")) {
              warning(paste("Error encountered in profile (uniroot):",pfit))
              return(NA)
            }
            else {
              zz <- 2*pfit@min - 2*(-logLik(object))
              if (zz > -tol.newmin)
                zz <- max(zz, 0)
              else
                stop(sprintf("profiling has found a better solution (old deviance=%.2f, new deviance=%.2f), so original fit had not converged",2*pfit@min,2*(-logLik(object))))
              z <- zz - chisqcutoff
            }
            if (trace) cat(bi, z, "\n")
            z
          }
        stepfun <- function(step) {
          B0[pm] + sgn * step * std.err[pm]
        }
        invstepfun <- function(out) {
          (out - B0[pm])/(sgn * std.err[pm])
        }
        sgnvec=c(-1,1)
        for (i in 1:2) {
          sgn <- sgnvec[i]
          bnd <- if (sgn<0) {
            if (is.null(lower)) -Inf else lower[pm]
          } else {
            if (is.null(upper)) Inf else upper[pm]
          }
          c0 <- critfun(B0[pm])
          bi <- 
          ctry <- pmin(5,invstepfun(bnd))
          cdel <- -0.25
          c5 <- NA
          while (is.na(c5) && ctry>0 ) {
            c5 <- critfun(stepfun(ctry))
            if (is.na(c5)) {
              if (trace) cat("encountered NA, reducing ctry to",ctry+cdel,"\n")
              ctry <- ctry+cdel
            }
          }
          if (trace) cat(c0,c5,"\n")
          if (is.na(c0*c5) || c0*c5>0) {
            warning(paste("can't find confidence limits in",
                          c("negative","positive")[i],"direction"))
            curci <- NA
            ## FIXME: could try harder!
          } else {
            curci <- uniroot(critfun,c(stepfun(0),stepfun(ctry)))$root
          }
          ci[pnames[pm],i] <- curci
        }
      }
    } else if (method=="quad") {
      for (pm in parm) {
        ci[pnames[pm],] <- qnorm(a,B0[pm],std.err[pm])
      }
    } else stop("unknown method")
    return(drop(ci))
  }
})

## setGeneric("formula", function(x, env = parent.frame(), ...) {
##           standardGeneric("formula")})
## don't know why behaviour of anova() and formula() are different?
## (used setGeneric for anova() without trouble, caused problems here)
## trying to avoid "creating a new generic" message on install?

setMethod("formula", "mle2",
          function(x, env = parent.frame(), ...) {
            as.formula(x@formula)
          })

## stdEr <- function(x, ...) {
##   UseMethod("stdEr")
##  }

setGeneric("stdEr", function(x, ...) { standardGeneric("stdEr")})

setMethod("stdEr","mle2",
          function(x, ...) {
            sqrt(diag(x@vcov)) ## why doesn't vcov(x) work here???
          })

## should this be object@fullcoef or object@coef??? or should
## it have an additional argument --- is that possible?
setMethod("coef", "mle2", function(object,exclude.fixed=FALSE) {
    if (!exclude.fixed) object@fullcoef else object@coef
})
## fullcoef <- function(object) object@fullcoef  ## this should be a method
setMethod("coef", "summary.mle2", function(object) { object@coef })
## hmmm.  Work on this. 'hessian' conflicts with numDeriv definition. Override?
## setMethod("Hessian", sig="mle2", function(object) { object@details$hessian })

setMethod("show", "mle2", function(object){
    cat("\nCall:\n")
    print(object@call.orig)
    cat("\nCoefficients:\n")
    print(coef(object))
    cat("\nLog-likelihood: ")
    cat(round(as.numeric(logLik(object)),2),"\n")
    if (object@optimizer=="optimx" && length(object@method)>1) {
      cat("Best method:",object@details$method.used,"\n")
    }
    if (object@details$conv>0)
      cat("\nWarning: optimization did not converge (code ",
          object@details$convergence,": ",object@details$message,")\n",sep="")
  })

setMethod("show", "summary.mle2", function(object){
    cat("Maximum likelihood estimation\n\nCall:\n")
    print(object@call)
    cat("\nCoefficients:\n")
    printCoefmat(coef(object))
    cat("\n-2 log L:", object@m2logL, "\n")
})

setMethod("show", "profile.mle2", function(object){
    cat("Likelihood profile:\n\n")
    print(object@profile)
  })

setMethod("summary", "mle2", function(object, waldtest=TRUE, ...){
    cmat <- cbind(Estimate = object@coef,
                  `Std. Error` = sqrt(diag(object@vcov)))
    zval <- cmat[,"Estimate"]/cmat[,"Std. Error"]
    pval <- 2*pnorm(-abs(zval))
    coefmat <- cbind(cmat,"z value"=zval,"Pr(z)"=pval)
    m2logL <- 2*object@min
    new("summary.mle2", call=object@call.orig, coef=coefmat, m2logL= m2logL)
})



setMethod("logLik", "mle2",
function (object, ...)
{
    if(length(list(...)))
        warning("extra arguments discarded")
    val <- -object@min
    attr(val, "df") <- length(object@coef)
    attr(val, "nobs") <- attr(object,"nobs")
    class(val) <- "logLik"
    val
  })

setGeneric("deviance", function(object, ...) standardGeneric("deviance"))
setMethod("deviance", "mle2",
function (object, ...)
{
  -2*logLik(object)
})

setMethod("vcov", "mle2", function (object, ...) { object@vcov } )


setGeneric("anova", function(object, ...) standardGeneric("anova"))
setMethod("anova","mle2",
          function(object,...,width=getOption("width"), exdent=10) {
            mlist <- c(list(object),list(...))
            ## get names from previous call
            mnames <- sapply(sys.call(sys.parent())[-1],deparse)
            ltab <- as.matrix(do.call("rbind",
                                      lapply(mlist,
                                             function(x) {
                                               c("Tot Df"=length(x@coef),
                                                 Deviance=-2*logLik(x))
                                             })))
            terms=sapply(mlist,
              function(obj) {
                if (is.null(obj@formula) || obj@formula=="") {
                  mfun <- obj@call$minuslogl
                  mfun <- paste("[",if (is.name(mfun)) {
                    as.character(mfun)
                  } else { "..." },
                                "]",sep="")
                  paste(mfun,": ",paste(names(obj@coef),
                                        collapse="+"),sep="")
                } else {
                  as.character(obj@formula)
                }
              })
            mterms <- paste("Model ",
                            1:length(mnames),": ",mnames,", ",terms,sep="")
            mterms <- strwrapx(mterms,width=width,exdent=exdent,
                               wordsplit="[ \n\t]")
  ## trunc.term <- function(s,len) {
  ##     ## cat("***",nchar(s),length(grep("\\+",s)),"\n",sep=" ")    
  ##     if ((nchar(s)<len) || (length(grep("\\+",s))==0)) return(s)
  ##     ## cat("abc\n")
  ##     lens <- cumsum(sapply(strsplit(s,"\\+")[[1]],nchar)+1)
  ##     paste(substr(s,1,max(lens[lens<len])-1),"+...",sep="")
  ##   }
  ## WRAP here
  heading <- paste("Likelihood Ratio Tests",
                   paste(mterms,
                         collapse="\n"),
                   sep="\n")
  ltab <- cbind(ltab,Chisq=abs(c(NA,diff(ltab[,"Deviance"]))),
                Df=abs(c(NA,diff(ltab[,"Tot Df"]))))
  ltab <- cbind(ltab,"Pr(>Chisq)"=c(NA,pchisq(ltab[,"Chisq"][-1],
                       ltab[,"Df"][-1],lower.tail=FALSE)))
  rownames(ltab) <- 1:nrow(ltab)
  attr(ltab,"heading") <- heading
  class(ltab) <- "anova"
  ltab
})

## translate from profile to data frame, as either
## S3 or S4 method
as.data.frame.profile.mle2 <- function(x, row.names = NULL,
                                       optional = FALSE, ...) {
    m1 <- mapply(function(vals,parname) {
        ## need to unpack the vals data frame so that
        ## parameter names show up properly
        do.call("data.frame",
                c(list(param=rep(parname,nrow(vals))),
                  as.list(vals),focal=list(vals$par.vals[,parname])))
            },
                 x@profile,
                 as.list(names(x@profile)),
                 SIMPLIFY=FALSE)
    m2 <- do.call("rbind",m1)
    m2
}

setAs("profile.mle2","data.frame",
      function(from) {
          as.data.frame.profile.mle2(from)
          })

## causes infinite loop, and unnecessary anyway??
## BIC.mle2 <- stats4:::BIC

#' generate population prediction sample from parameters
#' 
#' This [EXPERIMENTAL] function combines several sampling tricks to compute a version of an importance sample (based on flat priors) for the parameters. 
#' 
#' @param object a fitted \code{mle2} object
#' @param n number of samples to return
#' @param n_imp number of total samples from which to draw, if doing importance sampling
#' @param return_wts return a column giving the weights of the samples, for use in weighted summaries?
#' @param impsamp subsample values (with replacement) based on their weights?
#' @param PDify use Gill and King generalized-inverse procedure to correct non-positive-definite variance-covariance matrix if necessary?
#' @param PDmethod method for fixing non-positive-definite covariance matrices
#' @param rmvnorm_method package to use for generating MVN samples
#' @param tol tolerance for detecting small eigenvalues
#' @param fix_params parameters to fix (in addition to parameters that were fixed during estimation)
#' @param return_all return a matrix including all values, and weights (rather than taking a sample)
#' @export
#' @references Gill, Jeff, and Gary King. "What to Do When Your Hessian Is Not Invertible: Alternatives to Model Respecification in Nonlinear Estimation." Sociological Methods & Research 33, no. 1 (2004): 54-87.
#' Lande, Russ and Steinar Engen and Bernt-Erik Saether, Stochastic Population Dynamics in Ecology and Conservation. Oxford University Press, 2003.

pop_pred_samp <- function(object,
                     n=1000,
                     n_imp=n*10,
                     return_wts=FALSE,
                     impsamp=FALSE,
                     PDify=FALSE,
                     PDmethod=NULL,
                     tol = 1e-6,
                     return_all=FALSE,
                     rmvnorm_method=c("mvtnorm","MASS"),
                     fix_params=NULL) {

    rmvnorm_method <- match.arg(rmvnorm_method)
    
    min_eval <- function(x) {
        ev <- eigen(x,only.values=TRUE)$values
        if (is.complex(ev)) {
            print(x)
            print(ev)
            stop("covariance matrix with complex eigenvalues (!)")
        }
        min(ev)
    }

    ## extract var-cov,
    cc_full <- object@fullcoef ## full parameters
    cc <- object@coef ## varying parameters only
    keep_params <- !names(cc) %in% fix_params

    cc <- cc[keep_params]
    vv <- vcov(object)
    vv <- vv[keep_params,keep_params]

    Lfun <- object@minuslogl
    fixed_pars <- setdiff(names(object@fullcoef),names(cc))
    res <- matrix(NA,nrow=n,ncol=length(cc_full),
                  dimnames=list(NULL,names(cc_full)))
    if (any(is.na(cc))) return(res)  ## bail out if coefs are NA

    ## try to fix bad covariance matrices
    bad_vcov <- any(is.na(vv))
    if (!bad_vcov) {
        min_eig <- min_eval(vv)
    } else {
        min_eig <- NA
    }
    if (is.na(min_eig) || any(min_eig<tol)) {
        if (!PDify) {
            stop("NA or non-positive definitive variance-covariance matrix ",
                 sprintf("(min eig=%f): ",min_eig),
                 "consider PDify=TRUE (and probably impsamp=TRUE)")
        }
        ## use King 1994 to 'posdefify' variance-covariance matrix
        ## (better than e.g. Matrix::nearPD)
        hh <- object@details$hessian[keep_params,keep_params]
        if (any(is.na(hh))) {
            warning("NA values in Hessian set to zero: check results *very* carefully!") 
            hh[is.na(hh)] <- 0 # !! questionable
        }
        if ((is.null(PDmethod) && bad_vcov) ||
            identical(PDmethod,"King")) {
            ## ONLY use this
            warning("using EXPERIMENTAL King et al method")
            ## semi definite: use King et al method
            ## (we may need this when semidefinite, because
            ##  we couldn't invert H in the first place ... nearPD
            ##  wants to take a non-pos-def matrix and PDify it.
            ##  (perhaps we could PDify the Hessian and then invert it ???
            vv <- crossprod(as.matrix(bdsmatrix::gchol(MASS::ginv(hh)),
                                      ones = FALSE))
        } else {
            vv <- as.matrix(Matrix::nearPD(vv)$mat)
        }
    }

    mv_n <- if (impsamp) n_imp else n  ## take more samples if actually sampling

    ## draw MVN samples
    res[,names(cc)] <- mv_vals <- switch(rmvnorm_method,
                                         mvtnorm=mvtnorm::rmvnorm(mv_n, mean=cc, sigma=vv),
                                         MASS=MASS::mvrnorm(mv_n, mu=cc, Sigma=vv))
    ## fill in fixed parameters as necessary
    if (length(fixed_pars)>0) {
        for (p in fixed_pars) {
            res[,p] <- object@fullcoef[p]
        }
    }
    if (!(impsamp || return_wts)) return(res)  ## done
    
    ## compute MV sampling probabilities
    mv_wts <- mvtnorm::dmvnorm(mv_vals,mean=cc,sigma=vv,log=TRUE)
    if (all(is.na(mv_wts)) && length(mv_wts)==1) {
        ## work around emdbook bug
        mv_wts <- rep(NA,length(mv_vals))
        warning("can't compute MV sampling probabilities")
    }
    
    ## compute **log**-likelihoods of each sample point (Lfun is negative LL)
    L_wts0 <- -1*apply(res,1,Lfun)
    ## shift negative log-likelihoods (avoid underflow);
    ## find scaled likelihood
    L_wts <- L_wts0 - mv_wts ## subtract log samp prob
    L_wts <- exp(L_wts - max(L_wts,na.rm=TRUE))
    L_wts <- L_wts/sum(L_wts,na.rm=TRUE)
    eff_samp <- 1/sum(L_wts^2,na.rm=TRUE)  ## check ???
    res <- cbind(res,wts=L_wts)
    attr(res,"eff_samp") <- eff_samp
    ## FIXME: warn if eff_samp is low?
    if (return_all) {
        return(cbind(res,loglik=L_wts0,mvnloglik=mv_wts))
    }
    if (return_wts) return(res)
    ## do importance sampling
    res <- res[sample(seq(nrow(res)),
                              size=n,
                              prob=L_wts,
                              replace=TRUE),]
    return(res)
}
    
## ## copy (!!) dmvnorm from emdbook to avoid cyclic dependency problems
## dmvnorm <- function (x, mu, Sigma, log = FALSE, tol = 1e-06) {
##     if (is.vector(x)) 
##         x = t(as.matrix(x))
##     n = length(mu)
##     if (is.vector(mu)) {
##         p <- length(mu)
##         if (is.matrix(x)) {
##             mu <- matrix(rep(mu, nrow(x)), ncol = p, byrow = TRUE)
##         }
##     }
##     else {
##         p <- ncol(mu)
##     }
##     if (!all(dim(Sigma) == c(p, p)) || nrow(x) != nrow(mu)) 
##         stop("incompatible arguments")
##     eS <- eigen(Sigma, symmetric = TRUE) 
##     ev <- eS$values
##     if (!all(ev >= -tol * abs(ev[1]))) 
##         warning("Sigma is not positive definite, trying anyway")
##     z = t(x - mu)
##     logdetS = try(determinant(Sigma, logarithm = TRUE)$modulus,
##                   silent=TRUE)
##     if (inherits(logdetS,"try-error")) return(rep(NA,nrow(x)))
##     attributes(logdetS) <- NULL
##     iS = MASS::ginv(Sigma)
##     ssq = diag(t(z) %*% iS %*% z)
##     loglik = -(n * (log(2*pi)) +  logdetS + ssq)/2
##     if (log) loglik else exp(loglik)
##   }

## TO DO: roll back into bbmle?
## allow multiple 'transects'?
## (i.e. if two sets of parameters are given ...)
## * robustification
##  print method
## allow manual override of ranges
## allow log-scaling?
## preserve order of parameters in 1D plots

## substitute values of parameters into full parameter vector
mkpar <- function(params,p,i) {
  params[i] <- p
  params
}

## get reasonable range for slice
## document what is done here
## implement upper bound
## more robust approaches;
## try not to assume anything about signs of parameters
## inherit bounds from fitted value
get_trange <- function(pars,  ## baseline parameter values
                       i,     ## focal parameter
                       fun,   ## objective function
                       lower=-Inf, ## lower bound 
                       upper=Inf,  ## upper bound
                       cutoff=10,  ## increase above min z-value
                       maxit=200,  ## max number of iterations
                       steptype=c("mult","addprop"),
                       step=0.1) {
    ## step possibilities: multiplicative
    ## additive (absolute scale)     [not yet implemented]
    addabs <- NULL ## fix false positive test
    steptype <- match.arg(steptype)
    v <- v0 <- fun(pars)
    lowval <- pars[i]
    it <- 1
    if (steptype=="addprop") step <- step*pars[i]
    while (it<maxit && lowval>lower && v<(v0+cutoff)) {
        lowval <- switch(steptype,
                         addabs,
                         addpropn=lowval-step,
                         mult=lowval*(1-step))
        v <- fun(mkpar(pars,lowval,i))
        it <- it+1
    }
    lowdev <- v
    lowit <- it
    upval <- pars[i]
    it <- 1
    v <- v0 <- fun(pars)
    if (upval==0) upval <- 1e-4
    while (it<maxit && v<(v0+cutoff)) {
        upval <- switch(steptype,
                        addabs,
                        addpropn=lowval+step,
                         mult=lowval*(1+step))
        v <- fun(mkpar(pars,upval,i))
        ## cat(it,upper,v,"\n")
        it <- it+1
    }
    updev <- v
    upit <- it
    c(low_val=lowval,up_valr=upval,low_dev=lowdev,up_dev=updev,
      low_it=lowit,up_it=upit)
  }

get_all_trange <- function(params,fun,lower,upper,cutoff=10,...) {
  arglist <- c(list(pars=params,fun=fun,cutoff=cutoff),list(...))
  tranges <- t(mapply(FUN=get_trange,
                      seq(length(params)),
                      lower,
                      upper,
                      MoreArgs=arglist,
                      SIMPLIFY=TRUE))
  rownames(tranges) <- names(params)
  tranges
}

## generic function (S3)
slice <- function (x, dim=1, ...)  {
    UseMethod("slice")
}

slice.mle2 <- function(x,  ...) {
    ff <- x@minuslogl
    ## vectorize objective function: return minus log L
    ## (switching to deviance screws things up???)
    ff2 <- function(p) {
        do.call(ff,as.list(p))
    }
    slice0(coef(x),ff2, ...)
}

slice0 <- function(params,fun,dim=1,params2=NULL,...) {
  if (dim==1) {
    if (is.null(params2)) {
      slice1D(params,fun,...)
    } else {
      slicetrans(params,params2,fun,...)
    }
  } else {
    if (!is.null(params2)) stop("can't do transect in 2D")
    slice2D(params,fun,...)
  }
}


plot.slice <- function(x,...) {
  switch(x$dim,xyplot(x,...),splom(x,...))
}

slicetrans <- function(params, params2, fun, extend=0.1, nt=401,
                       lower=-Inf, upper=Inf) {
  ## make sure 0/1 are included in parameter vector ...
  np <- length(params)
  extend <- rep(extend,length.out=2)
  lower <- rep(lower,length.out=np)
  upper <- rep(upper,length.out=np)
  slicep <- sort(unique(c(0,1,seq(-extend[1],1+extend[2], length=nt))))
  slicepars <- t(sapply(slicep, function(x) (1 - x) * params + x * params2))
  OK <- apply(slicepars,1,function(x) all(x>=lower & x<=upper))
  if (any(!OK)) {
    warning("some parameter sets outside of bounds were removed")
    slicep <- slicep[OK]
    slicepars <- slicepars[OK,]
  }
  v <- apply(slicepars, 1, fun)
  slices <- list(data.frame(var1="trans",x=slicep,z=v))
  r <- list(slices=slices,params=params,params2=params2,dim=1)
  class(r) <- "slice"
  r
}

  
slice1D <- function(params,fun,nt=101,
                    lower=-Inf,
                    upper=Inf,
                    verbose=TRUE,
                    tranges=NULL,
                    ...) {
  npv <- length(params)
  if (is.null(pn <- names(params))) pn <- seq(npv)
  if (is.null(tranges)) {
      tranges <- get_all_trange(params,fun,
                                rep(lower,length.out=npv),
                                rep(upper,length.out=npv),
                                ...)
  }
  slices <- vector("list",npv)
  for (i in 1:npv) {
      tvec <- seq(tranges[i,1],tranges[i,2],length=nt)
      if (verbose) cat(pn[i],"\n")
      vtmp <- sapply(tvec,
                     function(t) {
                         fun(mkpar(params,t,i))})
      slices[[i]] <- data.frame(var1=pn[i],x=tvec,z=vtmp)
  }
  r <- list(slices=slices,ranges=tranges,params=params,dim=1)
  class(r) <- "slice"
  r
}

## OLD slice method
## should probably roll this in as an option to profile
## include attribute, warning? draw differently (leave off
## conf. limit lines)
## slice <- function(fitted, ...) UseMethod("slice")

## 1D slicing implemented as in profile
sliceOld <- function (fitted, which = 1:p, maxsteps = 100,
                       alpha = 0.01, zmax = sqrt(qchisq(1 - alpha/2, p)),
                       del = zmax/5, trace = FALSE,
                       tol.newmin=0.001, ...)
{
    onestep <- function(step)
    {
        bi <- B0[i] + sgn * step * del * std.err[i]
        fix <- list(bi)
        names(fix) <- p.i
        call$fixed <- c(fix,eval(call$fixed))
        call$eval.only = TRUE
        pfit <- try(eval(call), silent=TRUE) ##
        if(inherits(pfit, "try-error")) return(NA)
        else {
            zz <- 2*(pfit@min - fitted@min)
            ri <- pv0
            ri[, names(pfit@coef)] <- pfit@coef
            ri[, p.i] <- bi
            if (zz > -tol.newmin)
                zz <- max(zz, 0)
            else stop("profiling has found a better solution, so original fit had not converged")
            z <- sgn * sqrt(zz)
            pvi <<- rbind(pvi, ri)
            zi <<- c(zi, z) ## NB global set!
        }
        if (trace) cat(bi, z, "\n")
        z
      }
    ## Profile the likelihood around its maximum
    ## Based on profile.glm in MASS
    summ <- summary(fitted)
    std.err <- summ@coef[, "Std. Error"]
    Pnames <- names(B0 <- fitted@coef)
    pv0 <- t(as.matrix(B0))
    p <- length(Pnames)
    prof <- vector("list", length = length(which))
    names(prof) <- Pnames[which]
    call <- fitted@call
    call$minuslogl <- fitted@minuslogl
    for (i in which) {
        zi <- 0
        pvi <- pv0
        p.i <- Pnames[i]
        for (sgn in c(-1, 1)) {
          if (trace)
            cat("\nParameter:", p.i, c("down", "up")[(sgn + 1)/2 + 1], "\n")
          step <- 0
          z <- 0
          ## This logic was a bit frail in some cases with
          ## high parameter curvature. We should probably at least
          ## do something about cases where the mle2 call fails
          ## because the parameter gets stepped outside the domain.
          ## (We now have.)
          call$start <- as.list(B0)
          lastz <- 0
          while ((step <- step + 1) < maxsteps && abs(z) < zmax) {
            z <- onestep(step)
            if(is.na(z)) break
            lastz <- z
          }
          if(abs(lastz) < zmax) {
            ## now let's try a bit harder if we came up short
            for(dstep in c(0.2, 0.4, 0.6, 0.8, 0.9)) {
              z <- onestep(step - 1 + dstep)
              if(is.na(z) || abs(z) > zmax) break
            }
          } else if(length(zi) < 5) { # try smaller steps
            mxstep <- step - 1
            step <- 0.5
            while ((step <- step + 1) < mxstep) onestep(step)
          }
        }
        si <- order(pvi[, i])
        prof[[p.i]] <- data.frame(z = zi[si])
        prof[[p.i]]$par.vals <- pvi[si,, drop=FALSE]
    }
    list(profile = prof, summary = summ)
}



## * is it possible to set up the 2D vectors so they include
##   the baseline value? maybe not easily ...
slice2D <- function(params,
                    fun,
                    nt=31,
                    lower=-Inf,
                    upper=Inf,
                    cutoff=10,
                    verbose=TRUE,
                    tranges=NULL,
                    ...) {
  npv <- length(params)
  if (is.null(pn <- names(params))) pn <- seq(npv)
  if (is.null(tranges)) {
      tranges <- get_all_trange(params,fun,
                                rep(lower,length.out=npv),
                                rep(upper,length.out=npv),
                                cutoff=cutoff,
                                ...)
  }
  slices <- list()
  for (i in 1:(npv-1)) {
    slices[[i]] <- vector("list",npv)
    for (j in (i+1):npv) {
      if (verbose) cat("param",i,j,"\n")
      t1vec <- seq(tranges[i,1],tranges[i,2],length=nt)
      t2vec <- seq(tranges[j,1],tranges[j,2],length=nt)
      mtmp <- matrix(nrow=nt,ncol=nt)
      for (t1 in seq_along(t1vec)) {
        for (t2 in seq_along(t2vec)) {
          mtmp[t1,t2] <- fun(mkpar(params,c(t1vec[t1],t2vec[t2]),
                                   c(i,j)))
        }
      }
      slices[[i]][[j]] <- data.frame(var1=pn[i],var2=pn[j],
                                     expand.grid(x=t1vec,y=t2vec),
                                     z=c(mtmp))
    }
  }
  r <- list(slices=slices,ranges=tranges,params=params,dim=2)
  class(r) <- "slice"
  r
}

## flatten slice:
##  do.call(rbind,lapply(slices,do.call,what=rbind))

slices_apply <- function(s,FUN,...) {
  for (i in seq_along(s)) {
    for (j in seq_along(s[[i]])) {
      if (!is.null(s[[i]][[j]])) {
        s[[i]][[j]] <- FUN(s[[i]][[j]],...)
      }
    }
  }
  s
}

xyplot.slice <- function(x,data,type="l",scale.min=TRUE,...) {
  allslice <- do.call(rbind,x$slices)
  ## correct ordering
  allslice$var1 <- factor(allslice$var1,
                          levels=unique(as.character(allslice$var1)))
  if (scale.min) allslice$z <- allslice$z-min(allslice$z)
  pfun <- function(x1,y1,...) {
      panel.xyplot(x1,y1,...)
      if (is.null(x$params2)) {
          ## regular 1D slice
          panel.abline(v=x$params[panel.number()],col="gray")
      } else {
          ## 'transect' slice
          panel.abline(v=c(0,1),col="gray")
          panel.abline(h=y1[x1 %in% c(0,1)],col="gray")
      }
  }
  xyplot(z~x|var1,data=allslice,type=type,
         scales=list(x=list(relation="free")),
         panel=pfun,...)
}

splom.slice <- function(x,
                        data,
                        scale.min=TRUE,
                        at=NULL,
                        which.x=NULL,
                        which.y=NULL,
                        dstep=4,
                        contour=FALSE,...) {
  if (x$dim==1) stop("can't do splom on 1D slice object")
  smat <- t(x$ranges[,1:2])
  if (scale.min) {
    ## FIXME: something more elegant to flatten slice list?
    all.z <- unlist(sapply(x$slices,
                    function(x) {
                      sapply(x,
                             function(x)
                             if (is.null(x)) NULL else x[["z"]])
                    }))
    min.z <- min(all.z[is.finite(all.z)])
    ## round up to next multiple of 'dstep'
    max.z <- dstep * ((max(all.z[is.finite(all.z)])-
                       min.z) %/% dstep + 1)
    if (missing(at)) {
      at <- seq(0,max.z,by=dstep)
    }
    scale.z <- function(X) {
        X$z <- X$z-min.z
        X
    }
    x$slices <- slices_apply(x$slices,scale.z)
  }
  up0 <- function(x1, y, groups, subscripts, i, j, ...) {
      ## browser()
    sl <- x$slices[[j]][[i]]
    with(sl,panel.levelplot(x=x,y=y,z=z,contour=contour,
                            at=if (!is.null(at)) at else pretty(z),
                            subscripts=seq(nrow(sl))))
    panel.points(x$params[j],x$params[i],pch=16)
    mm <- matrix(sl$z,nrow=length(unique(sl$x)))
    ## FIXME: more robust ...
    wmin <- which(mm==min(mm),arr.ind=TRUE)
    xmin <- unique(sl$x)[wmin[1]]
    ymin <- unique(sl$y)[wmin[2]]
    panel.points(xmin,ymin,pch=1)
  }
  lp0 <- function(...) {
  }
  ## FIXME: use ?draw.colorkey to add a legend ...
  ## FIXME: make diagonal panel text smaller ???
  splom(smat,lower.panel=lp0,diag.panel=diag.panel.splom,
        upper.panel=up0,...)
}

  
## generic profiling code???
##  either need (1) optimizers with 'masks' or (2)

## logic for removing/modifying bounds:
## (1) unbounded opt. will have limits of -Inf/Inf
##   [or missing()]
## (2) bounded opt
## fix length mismatch errors!
k <- 19
N <- 20

uniboundtest <- function() {
  m1 <- mle2(k~dbinom(size=N,prob=p),
             start=list(p=0.5))
  m1b <- mle2(k~dbinom(size=N,prob=p),
             start=list(p=0.5),method="L-BFGS-B",upper=0.999)
  p1 <- profile(m1)
  p1b <- profile(m1b)
}

require(bbmle)
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)
fit <- mle2(y~dpois(lambda=ymax/(1+x/xhalf)), start=list(ymax=25,xhalf=3),data=d)
fit2 <- mle2(y~dpois(lambda=(x+1)*slope), start=list(slope=1),data=d)
BIC(fit)
BIC(fit,fit2)

library(bbmle)
d <- data.frame(x=0:10,
                y=c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8))

maxit <- 1000
mle2(y~dpois(lambda=exp(lymax)/(1+x/exp(lhalf))),
   start=list(lymax=0,lhalf=0),
   data=d,
     control=list(maxit=maxit),
   parameters=list(lymax~1,lhalf~1))

\documentclass{article}
%\VignetteIndexEntry{Examples for enhanced mle code}
%\VignettePackage{bbmle}
%\VignetteDepends{Hmisc}
%\VignetteDepends{emdbook}
%\VignetteDepends{ggplot2}
%\VignetteDepends{lattice}
%\VignetteEngine{knitr::knitr}
\usepackage[utf8]{inputenc} % for UTF-8/single quotes from sQuote()
\usepackage[english]{babel} % for texi2dvi ~ bug
\usepackage{graphicx}
\usepackage{natbib}
\usepackage{array}
\usepackage{color}
\usepackage[colorlinks=true,bookmarks=true]{hyperref}
\hypersetup{linkcolor=purple,urlcolor=blue,citecolor=gray}
\usepackage{url}
\author{Ben Bolker}
\title{Maximum likelihood estimation and analysis
  with the \code{bbmle} package}
\newcommand{\code}[1]{{\tt #1}}
\newcommand{\bbnote}[1]{\color{red} {\em #1} \color{black}}
\date{\today}
\begin{document}
\bibliographystyle{chicago}
%\bibliographystyle{plain}
\maketitle
\tableofcontents

<<knitropts,echo=FALSE,message=FALSE>>=
if (require("knitr")) opts_chunk$set(fig.width=5,fig.height=5,tidy=FALSE,warning=FALSE,error=TRUE)
@ 
<<setup,results="hide",echo=FALSE,message=FALSE>>=
library(Hmisc)
@ 

The \code{bbmle} package, designed to simplify
maximum likelihood estimation and analysis in R,
extends and modifies the \code{mle} function and class
in the \code{stats4} package that comes with R by default.
\code{mle} is in turn a wrapper around the \code{optim}
function in base R.
The maximum-likelihood-estimation function and class
in \code{bbmle} are both called \code{mle2}, to avoid
confusion and conflict with the original functions in
the \code{stats4} package.  The major differences between
\code{mle} and \code{mle2} are:
\begin{itemize}
\item \code{mle2} is more robust, with additional warnings (e.g.
  if the Hessian can't be computed by finite differences,
  \code{mle2} returns a fit with a missing Hessian rather
  than stopping with an error)
\item \code{mle2} uses a \code{data} argument to allow different
  data to be passed to the negative log-likelihood function
\item \code{mle2} has a formula interface like that
 of (e.g.) \code{gls} in the \code{nlme} package.
 For relatively simple models the formula for the
 maximum likelihood can be written in-line, rather than
 defining a negative log-likelihood function.  The formula
 interface also simplifies fitting models with
 categorical variables.  Models fitted using the formula interface
 also have applicable \code{predict} and \code{simulate} methods.
\item \code{bbmle} defines \code{anova}, \code{AIC}, \code{AICc}, 
  and \code{BIC} methods for
  \code{mle2} objects, as well as
  \code{AICtab}, \code{BICtab}, \code{AICctab}
  functions for producing summary tables of information criteria for a 
  set of models.
\end{itemize}

Other packages with similar functionality (extending
GLMs in various ways) are
\begin{itemize}
\item on CRAN: \code{aods3} (overdispersed models such as beta-binomial);
  \code{vgam} (a wide range of models);
  \code{betareg} (beta regression);
  \code{pscl} (zero-inflated, hurdle models);
  \code{maxLik} (another general-purpose maximizer, with
  a different selection of optimizers)
\item In Jim Lindsey's code repository
  (\url{http://popgen.unimaas.nl/~jlindsey/rcode.html}):
  \code{gnlr} and \code{gnlr3}
\end{itemize}
  
\section{Example: \emph{Orobanche}/overdispersed binomial}

This example will use the classic data set on
\emph{Orobanche} germination from \cite{Crowder1978}
(you can also use
\code{glm(...,family="quasibinomial")} or
the \code{aods3} package to analyze these data).

\subsection{Test basic fit to simulated beta-binomial data}

First, generate a single beta-binomially distributed
set of points as a simple test.

Load the \code{emdbook} package
to get functions for the beta-binomial distribution (random-deviate 
function \code{rbetabinom} --- these functions are also available
in Jim Lindsey's \code{rmutil} package).
<<emdbook,message=FALSE>>=
library(emdbook)
@  

Generate random deviates from a random beta-binomial:
<<bbsim>>=
set.seed(1001)
x1 <- rbetabinom(n=1000,prob=0.1,size=50,theta=10)
@ 

Load the package:
<<bbmle,message=FALSE>>=
library(bbmle)
@ 

Construct a simple negative log-likelihood function:
<<likfun1>>=
mtmp <- function(prob,size,theta) {
  -sum(dbetabinom(x1,prob,size,theta,log=TRUE))
}
@ 

Fit the model --- use \code{data} to pass the \code{size}
parameter (since it wasn't hard-coded in the \code{mtmp}
function):
<<fit1,warning=FALSE>>=
suppressWarnings(
    m0 <- mle2(mtmp,start=list(prob=0.2,theta=9),data=list(size=50))
)
@
(here and below, I'm suppressing lots of warnings about {\tt NaNs produced})

The \code{summary} method for \code{mle2} objects
shows the parameters; approximate standard
errors (based on quadratic approximation to the curvature at
the maximum likelihood estimate); and a test
of the parameter difference from zero based on
this standard error and on an assumption 
that the likelihood surface is quadratic
(or equivalently that the sampling distribution
of the estimated parameters is normal).

<<sum1>>=
summary(m0)
@ 

Construct the likelihood profile (you can
apply \code{confint} directly to \code{m0},
but if you're going to work with the likelihood
profile [e.g. plotting, or looking for confidence
intervals at several different $\alpha$ values]
then it is more efficient to compute the profile
once):

<<prof1,warning=FALSE>>=
suppressWarnings(
    p0 <- profile(m0)
)
@ 

Compare the confidence interval estimates based on
inverting a spline fit to the profile (the default);
based on the quadratic approximation at the
maximum likelihood estimate; and based on
root-finding to find the exact point where the
profile crosses the critical level.

<<confint1,warning=FALSE>>=
confint(p0)
confint(m0,method="quad")
confint(m0,method="uniroot")
@ 

All three types of confidence limits are similar.

Plot the profiles:
<<profplot1,fig.height=5,fig.width=10,out.width="\\textwidth">>=
par(mfrow=c(1,2))
plot(p0,plot.confstr=TRUE)
@ 

By default, the plot method for 
likelihood profiles displays the square root of the
the deviance difference
(twice the difference in negative
log-likelihood from the best fit), so it will
be {\sf V}-shaped
for cases where the quadratic approximation works well
(as in this case).
(For a better visual estimate of whether the profile
is quadratic, use the \code{absVal=FALSE} option to the \code{plot}
method.)

You can also request confidence intervals
calculated using \code{uniroot}, which may be more exact when
the profile is not smooth enough to be modeled accurately
by a spline.  However, this method is
also more sensitive to numeric problems.

Instead of defining an
explicit function for \code{minuslogl}, 
we can also use the formula interface.
The formula interface assumes that
the density function given (1) has \code{x} as
its first argument (if the distribution is multivariate,
then \code{x} should be a matrix of observations) and
(2) has a \code{log} argument that will return
the log-probability or log-probability density
if \code{log=TRUE}.  Some of the extended functionality
(prediction etc.) depends on the existence of 
an \code{s}- variant function for the distribution
that returns (at least) the mean and median as
a function of the parameters
(currently defined: \code{snorm}, \code{sbinom},
\code{sbeta}, \code{snbinom}, \code{spois}).
<<fit2,warning=FALSE>>=
m0f <- mle2(x1~dbetabinom(prob,size=50,theta),
            start=list(prob=0.2,theta=9),data=data.frame(x1))
@ 
Note that you must specify the data via the \code{data}
argument when using the formula interface. This may be
slightly more unwieldy than just pulling the data from your
workspace when you are doing simple things, but in the long
run it makes tasks like predicting new responses much simpler.

It's convenient to use the formula interface
to try out likelihood estimation on the
transformed parameters:
<<fit2f>>=
m0cf <- mle2(x1~dbetabinom(prob=plogis(lprob),size=50,theta=exp(ltheta)),
            start=list(lprob=0,ltheta=2),data=data.frame(x1))
confint(m0cf,method="uniroot")
confint(m0cf,method="spline")
@ 

In this case the answers from \code{uniroot}
and \code{spline} (default) methods barely
differ.

\subsection{Real data (\emph{Orobanche}, \cite{Crowder1978})}
Data are copied from the \code{aods3} package
(but a copy is saved with the package to avoid depending on the
 \code{aods3} package):
<<orobdata>>=
load(system.file("vignetteData","orob1.rda",package="bbmle"))
summary(orob1)
@ 

Now construct a negative log-likelihood
function that differentiates among groups:

<<aodlikfun>>=
X <- model.matrix(~dilution, data = orob1)
ML1 <- function(prob1,prob2,prob3,theta,x) {
  prob <- c(prob1,prob2,prob3)[as.numeric(x$dilution)]
  size <- x$n
  -sum(dbetabinom(x$m,prob,size,theta,log=TRUE))
}
@ 

%  Would like to show an intermediate example that does plogis(X %*% beta)
% explicitly but argument processing is messed up for list-like parameters ...
% sigh ...

Results from \cite{Crowder1978}:
<<crowdertab,echo=FALSE,results="asis">>=
crowder.results <- matrix(c(0.132,0.871,0.839,78.424,0.027,0.028,0.032,-34.991,
                            rep(NA,7),-34.829,
                            rep(NA,7),-56.258),
                          dimnames=list(c("prop diffs","full model","homog model"),
                            c("prob1","prob2","prob3","theta","sd.prob1","sd.prob2","sd.prob3","NLL")),
                          byrow=TRUE,nrow=3)
latex(crowder.results,file="",table.env=FALSE,title="model")
@

<<aodfit1,warning=FALSE,depends.on="aodlikfun">>=
(m1 <- mle2(ML1,
            start=list(prob1=0.5,prob2=0.5,prob3=0.5,theta=1),
            data=list(x=orob1)))
@

Or:
<<eval=FALSE>>=
## would prefer ~dilution-1, but problems with starting values ...
(m1B <- mle2(m~dbetabinom(prob,size=n,theta),
             param=list(prob~dilution),
             start=list(prob=0.5,theta=1),
    data=orob1))
@


 
The result warns us that the optimization has not
converged; we also don't match
Crowder's results for $\theta$ exactly.
We can fix both of these problems by setting \code{parscale} appropriately.

Since we don't bound $\theta$ (or below, $\sigma$) we get  a fair number
of warnings with this and the next few fitting and profiling attempts.
We will ignore these for now, since the final results reached are reasonable
(and match or nearly match Crowder's values); the appropriate, careful thing
to do would be either to fit on a transformed scale where all real-valued
parameter values were legal, or to use \code{method="L-BFGS-B"} (or \code{method="bobyqa"} 
with the \code{optimx} package) to bound the parameters appropriately.
You can also use \code{suppressWarnings()} if you're sure you don't
need to know about any warnings (beware: this will suppress \emph{all}
warnings, those you weren't expecting as well as those you were \ldots)

<<suppWarn,echo=FALSE>>=
opts_chunk$set(warning=FALSE)
@ 
<<aodfit2>>=
(m2 <- mle2(ML1,start=as.list(coef(m1)),
          control=list(parscale=coef(m1)),
          data=list(x=orob1)))
@ 

Calculate likelihood profile (restrict the upper limit
of $\theta$, simply because it will make the picture
below a little bit nicer):
<<aodprof2>>=
p2 <- profile(m2,prof.upper=c(Inf,Inf,Inf,theta=2000))
@ 

Get the curvature-based parameter standard
deviations (which Crowder used
rather than computing likelihood profiles):
<<aodstderr>>=
round(stdEr(m2),3)
@ 
We are slightly off Crowder's numbers --- rounding
error?

Crowder also defines a variance (overdispersion) parameter
$\sigma^2=1/(1+\theta)$.
<<aodvar>>=
sqrt(1/(1+coef(m2)["theta"]))
@ 

Using the delta method (via the \code{deltavar}
function in the \code{emdbook} package)
to approximate the standard deviation of
$\sigma$:
<<deltavar>>=
sqrt(deltavar(sqrt(1/(1+theta)),meanval=coef(m2)["theta"],
         vars="theta",Sigma=vcov(m2)[4,4]))
@ 

Another way to fit in terms of $\sigma$ rather than $\theta$
is to compute $\theta=1/\sigma^2-1$ on the fly in a
formula:

<<sigma3>>=
m2b <- mle2(m~dbetabinom(prob,size=n,theta=1/sigma^2-1),
            data=orob1,
            parameters=list(prob~dilution,sigma~1),
            start=list(prob=0.5,sigma=0.1))
## ignore warnings (we haven't bothered to bound sigma<1)
round(stdEr(m2b)["sigma"],3)
p2b <- profile(m2b,prof.lower=c(-Inf,-Inf,-Inf,0))
@ 

As might be expected since the standard deviation
of $\sigma$ is large, the quadratic approximation is
poor:

<<compquad>>=
r1 <- rbind(confint(p2)["theta",],
            confint(m2,method="quad")["theta",])
rownames(r1) <- c("spline","quad")
r1
@ 

Plot the profile:
<<profplottheta>>=
plot(p2, which="theta",plot.confstr=TRUE, show.points = TRUE)
@ 

What does the profile for $\sigma$ look like?

<<profplotsigma>>=
## not working?
## plot(p2b,which="sigma",plot.confstr=TRUE, show.points=TRUE)
par(las = 1, bty = "l")
with(p2b@profile$sigma, plot(par.vals[,"sigma"], abs(z), type = "b"))
@ 

Now fit a homogeneous model:
<<homogmodel>>=
ml0 <- function(prob,theta,x) {
  size <- x$n
  -sum(dbetabinom(x$m,prob,size,theta,log=TRUE))
}
m0 <- mle2(ml0,start=list(prob=0.5,theta=100),
          data=list(x=orob1))
@ 

The log-likelihood matches Crowder's result:
<<logLikcomp>>=
logLik(m0)
@ 

It's easier to 
use the formula interface
to specify all three of the models
fitted by Crowder (homogeneous, probabilities differing
by group, probabilities and overdispersion differing
by group):

<<formulafit>>=
m0f <- mle2(m~dbetabinom(prob,size=n,theta),
            parameters=list(prob~1,theta~1),
            data=orob1,
            start=list(prob=0.5,theta=100))
m2f <- update(m0f,
              parameters=list(prob~dilution,theta~1),
              start=list(prob=0.5,theta=78.424))
m3f <- update(m0f,
              parameters=list(prob~dilution,theta~dilution),
              start=list(prob=0.5,theta=78.424))
@ 

\code{anova} runs a likelihood ratio test on nested
models:
<<anovafit>>=
anova(m0f,m2f,m3f)
@ 

The various \code{ICtab} commands produce tables of
information criteria; by default the results are sorted and
presented as $\Delta$IC; there are various options, including
printing model weights.
<<ICtabfit>>=
AICtab(m0f,m2f,m3f,weights=TRUE)
BICtab(m0f,m2f,m3f,nobs=nrow(orob1),weights=TRUE)
AICctab(m0f,m2f,m3f,nobs=nrow(orob1),weights=TRUE)
@ 
<<reWarn,echo=FALSE>>=
opts_chunk$set(warning=FALSE)
@ 

\section{Example: reed frog size predation}

Data from an experiment by Vonesh \citep{VoneshBolker2005}
<<frogsetup>>=
frogdat <- data.frame(
  size=rep(c(9,12,21,25,37),each=3),
  killed=c(0,2,1,3,4,5,rep(0,4),1,rep(0,4)))
frogdat$initial <- rep(10,nrow(frogdat))
@ 

<<getgg>>=
library(ggplot2)
@ 

<<gg1>>=
gg1 <- ggplot(frogdat,aes(x=size,y=killed))+geom_point()+
      stat_sum(aes(size=..n..))+
      labs(size="#")+scale_x_continuous(limits=c(0,40))+
scale_size(breaks=1:3)
@ 
<<frogfit1,cache=TRUE,warning=FALSE>>=
m3 <- mle2(killed~dbinom(prob=c*(size/d)^g*exp(1-size/d),
  size=initial),data=frogdat,start=list(c=0.5,d=5,g=1))
pdat <- data.frame(size=1:40,initial=rep(10,40))
pdat1 <- data.frame(pdat,killed=predict(m3,newdata=pdat))
@

<<frogfit2,cache=TRUE,warning=FALSE>>=
m4 <- mle2(killed~dbinom(prob=c*((size/d)*exp(1-size/d))^g,
  size=initial),data=frogdat,start=list(c=0.5,d=5,g=1))
pdat2 <- data.frame(pdat,killed=predict(m4,newdata=pdat))
@ 

<<gg1plot>>=
gg1 + geom_line(data=pdat1,colour="red")+
      geom_line(data=pdat2,colour="blue")
@ 

<<frogfit2anal,cache=TRUE,warning=FALSE>>=
coef(m4)
prof4 <- profile(m4)
@ 

Three different ways to draw the profile:

(1) Built-in method (base graphics):
<<basegraphprofplot>>=
plot(prof4)
@ 

(2) Using \code{xyplot} from the \code{lattice} package:
\setkeys{Gin}{width=\textwidth}
<<latticeprof,fig.height=5,fig.width=10,out.width="\\textwidth">>=
prof4_df <- as.data.frame(prof4)
library(lattice)
xyplot(abs(z)~focal|param,data=prof4_df,
      subset=abs(z)<3,
       type="b",
       xlab="",
       ylab=expression(paste(abs(z),
           " (square root of ",Delta," deviance)")),
       scale=list(x=list(relation="free")),
             layout=c(3,1))
@ 

(3) Using \code{ggplot} from the \code{ggplot2} package:
<<ggplotprof,fig.height=5,fig.width=10>>=
ss <-subset(prof4_df,abs(z)<3)
ggplot(ss,
       aes(x=focal,y=abs(z)))+geom_line()+
      geom_point()+
      facet_grid(.~param,scale="free_x")
@ 

\section*{Additions/enhancements/differences from \code{stats4::mle}}
\begin{itemize}
\item{\code{anova} method}
\item{warnings on convergence failure}
\item{more robust to non-positive-definite Hessian;
  can also specify \code{skip.hessian} to skip Hessian
  computation when it is problematic}
\item{when profiling fails because better value is
    found, report new values}
\item{can take named vectors as well as lists as
    starting parameter vectors}
\item{added \code{AICc}, \code{BIC} definitions,
    \code{ICtab} functions}
\item{added \code{"uniroot"} and \code{"quad"}
    options to \code{confint}}
\item{more options for colors and line types etc etc.
The old arguments are:
<<oldargs,eval=FALSE>>=
function (x, levels, conf = c(99, 95, 90, 80, 50)/100, nseg = 50,
          absVal = TRUE, ...) {}
@ 
The new one is:
<<newargs,eval=FALSE>>=
function (x, levels, which=1:p, conf = c(99, 95, 90, 80, 50)/100, nseg = 50,
          plot.confstr = FALSE, confstr = NULL, absVal = TRUE, add = FALSE,
          col.minval="green", lty.minval=2,
          col.conf="magenta", lty.conf=2,
          col.prof="blue", lty.prof=1,
          xlabs=nm, ylab="score",
          show.points=FALSE,
          main, xlim, ylim, ...) {}
@ 
\code{which} selects (by character vector or numbers)
which parameters to plot: \code{nseg} does nothing
(even in the old version); \code{plot.confstr} turns on
the labels for the confidence levels; \code{confstr} gives
the labels; \code{add} specifies whether to add the
profile to an existing plot; \code{col} and \code{lty}
options specify the colors and line types for
horizontal and vertical lines marking the minimum
and confidence vals and the profile curve; \code{xlabs}
gives a vector of x labels; \code{ylab} gives the y label;
\code{show.points} specifies whether to show the raw points
computed.
}
\item{\code{mle.options()}}
\item{\code{data} argument}
\item{handling of names in argument lists}
\item{can use alternative optimizers (\code{nlminb}, \code{nlm}, \code{constrOptim}, \code{optimx},
    \code{optimize})}
\item{uses code from \code{numDeriv} package to compute Hessians rather than
    built-in optimizer code}
\item{by default, uses \code{MASS::ginv} (generalized inverse) rather than \code{solve} to invert
    Hessian (more robust to positive-semidefinite Hessians \ldots)}
\item{can use \code{vecpar=TRUE} (and \code{parnames()}) to use objective functions with parameters
    specified as vectors (for compatibility with \code{optim} etc.)}
\end{itemize}

\section{Newer stuff}

\textbf{To do:}
\begin{itemize}
\item{use \code{predict}, \code{simulate} etc.
    to demonstrate different parametric bootstrap approaches
    to confidence and prediction intervals
    \begin{itemize}
    \item use \code{predict} to get means and standard
      deviations, use delta method?
    \item use \code{vcov}, assuming quadratic profiles,
      with \code{predict(\ldots,newparams=\ldots)}
    \item prediction intervals assuming no parameter uncertainty
      with \code{simulate}
    \item both together \ldots
    \end{itemize}
  }
\end{itemize}


\section{Technical details}

\subsection{Profiling and confidence intervals}

This section describes the algorithm for constructing profiles
and confidence intervals, which is not otherwise documented anywhere
except in the code.  * indicates changes from the version
in \code{stats4:::mle}

\subsubsection{Estimating standard error}

In order to construct the profile for a particular parameter, one
needs an initial estimate of the scale over which to vary that
parameter.  The estimated standard error of the parameter based
on the estimated curvature of the likelihood surface at the MLE
is a good guess.
\begin{itemize}
\item if \code{std.err} is missing, extract the 
  standard error from the summary coefficient table (ultimately computed from 
  \code{sqrt(diag(inverse Hessian))} of the fit)
\item * a user-set value of \code{std.err} overrides this behavior 
  unless the value is specified as \code{NA} (in which
  case the estimate from the previous step is used)
\item * if the standard error value is still \code{NA} (i.e. the
  user did not specify it and the value estimated from the Hessian
  is missing or \code{NA}) use \code{sqrt(1/diag(hessian))}. This
  represents a (fairly feeble) attempt to come up with a plausible number
  when the Hessian is not positive definite but still has positive diagonal
  entries
\item if all else fails, stop and * print an error message that encourages
  the user to specify the values with \code{std.err}
\end{itemize}
  
There may be further tricks that would help guess the appropriate scale:
for example, one could guess on the basis of a comparison between the
parameter values and negative log-likelihoods at the starting and ending points
of the fits.  On the other hand, (a) this would take some effort and
still be subject to failure for sufficiently pathological fits and (b) there
is some value to forcing the user to take explicit, manual steps to remedy
such problems, as they may be signs of poorly defined or buggy log-likelihood
functions.

\subsubsection{Profiling}
  
Profiling is done on the basis of a constructed function that minimizes
the negative log-likelihood for a fixed value of the focal parameter and
returns the signed square-root of the deviance difference from the 
minimum (denoted by $z$).  At the MLE $z=0$ by definition; it should never
be $<0$ unless something has gone wrong with the original fit. The LRT significance
cutoffs for $z$ are equal to the usual two-tailed normal distribution cutoffs
(e.g. $\pm \approx 1.96$ for 95\% confidence regions).
  
In each direction (decreasing and increasing from the MLE for the focal parameter):
\begin{itemize}
\item fix the focal parameter
\item adjust control parameters etc. accordingly (e.g. remove the
  entry for the focal parameter so that the remaining control
  parameters match the non-fixed parameters)
\item{controls on the profiling (which can be set manually, but for which there
    is not much guidance in the documentation):
    \begin{itemize}
    \item \code{zmax} Maximum $z$ to aim for. (Default: \code{sqrt(qchisq(1-alpha/2, p))})
      The default maximum $\alpha$ (type~I error) is 0.01.
      \bbnote{I don't understand this
        criterion. It seems to expand the size of the univariate profile
        to match a cutoff for the multivariate confidence region of
        the model.  The $\chi^2$ cutoff for deviance to get the $(1-\alpha)$ 
        multivariate confidence region (i.e.,
        on all $p$ of the parameters) would be \code{qchisq(1-alpha,p)} --- %
        representing a one-tailed test on the deviance.  Taking the square root
        makes sense, since we are working with the square root of the deviance,
        but I don't understand (1) why we are expanding the region to allow
        for the multivariate confidence region (since we are computing univariate
        profiles) [you could at least argue that this is conservative, making
        the region a little bigger than it needs to be]; (2) why we are
        using $1-\alpha/2$ rather than $1-\alpha$.
      }
      For comparison, \code{MASS::profile.glm} (written by Bates and Venables in
      1996, ported to R by BDR in 1998) uses \code{zmax}=\code{sqrt(qchisq(1-alpha,1))}
      \bbnote{(this makes more sense to me \ldots)}.
      On the other hand, the profiling code in \code{lme4a} (the \code{profile}
      method for \code{merMod}, in \code{profile.R}) uses
      \code{qchisq(1-alphamax, nptot)} \ldots
    \item \code{del} Step size (scaled by standard error) (Default: \code{zmax}/5.)
      Presumably (?) copied from \code{MASS::profile.glm}, 
      which says (in \code{?profile.glm}):
      ``[d]efault value chosen to allow profiling at about 10 parameter
      values.''
    \item \code{maxsteps} Maximum number of profiling steps to try in each direction. (Default: 100)
    \end{itemize}
  }
\item While \verb+step<maxsteps+ and \verb+abs(z) < zmax+, set the value of the focal
  parameter to its MLE + \code{sgn*step*del*std.err} where \code{sgn} represents
  the direction, \code{step} is the current (integer) step, and \code{del} and 
  \code{std.err} are the step size scaling factor and standard error estimate
  discussed above (i.e. take steps of size (\code{del*std.err}) in the appropriate direction);
  evaluate $z$
\item{Stop the profiling: 
    \begin{itemize}
    \item if $z$ doesn't change from the previous step (\verb+stop_flat+) %
      --- unless \verb+try_harder+ is \code{TRUE}
    \item * stop if $z$ is less than \code{tol.newmin} (default: 0.001) units 
      \emph{better} than the MLE fit, i.e. $z<-\mbox{\code{tol.newmin}}$
      (if $-\mbox{\code{tol.newmin}}<z<0$, set $z$ to zero) (\verb+newpars_found+)
    \item if $z$ is \code{NA} (\verb+stop_na+)  --- unless \verb+try_harder+ is \code{TRUE}
    \item if $z$ is beyond \code{zmax} (i.e., we have reached our goal: \verb+stop_cutoff+)
    \item if \code{step==maxsteps}
    \item if the focal parameter has hit its upper/lower bound (\verb+stop_bound+)
    \end{itemize}
  }
\item if we have hit the maximum number of steps but not reached the cutoff
  (\verb+stop_maxstep+ but not \verb+stop_cutoff+), ``try a bit harder'':
  go \emph{almost} one more \code{del*std.err} unit out (in intervals
  of 0.2, 0.4, 0.6, 0.8, 0.9) (\bbnote{also seems reasonable but don't
    know where it comes from})
\item * if we violated the boundary but did not reach the cutoff
  (\verb+!stop_cutoff && stop_bound+), evaluate $z$ at the boundary
\item if we got to the cutoff in $<5$ steps, try smaller steps:
  start at \code{step=0.5} and proceed to
  \code{mxstep-0.5} in unit increments
  (rather than the original scale which went from 0 to \code{mxstep}).
  (\bbnote{
    Again, it seems reasonable, but I don't know what the original justification
    was \ldots})
\end{itemize}

\subsubsection{Confidence intervals}

We are looking for the values where $z$ (signed square root deviance
difference) is equal to the usual two-tailed normal distribution cutoffs
for a specified $\alpha$ level, e.g. $z=\pm 1.96$ for 95\% confidence
intervals (this is equivalent to a one-tailed test on the deviance
difference with the cutoff value for $\chi^2_1$).

\begin{description}
\item[Spline method]{(default)
    \begin{itemize}
    \item If necessary (i.e. if applied to a fitted object and not 
      to an existing profile), construct the profile
    \item * If the profile of the signed square root is non-monotonic,
      warn the user and revert to linear approximation on the profiled points
      to find the cutoffs:
    \item Otherwise, build an interpolation spline of $z$ (signed square root deviance
      difference) based on profiled points (the default is $n=3 \times L$
      where $L$ is the length of the original vector). Then
      use linear approximation on the $y$ ($z$) and $x$ (focal
      parameter value) of the spline
      to find the cutoffs (\bbnote{Why construct a spline and then interpolate linearly?  Why
        not use \code{backSpline} as in the profile plotting code?})
      \end{itemize}
    }
  \item[Quad method]{Use a quadratic approximation based
      on the estimated curvature (this is almost identical to
      using \code{confint.default}, and perhaps obsolete/could
      be replaced by a pointer to \code{confint.default} \ldots)
    }
  \item[Uniroot]{
      For each direction (up and down):
      \begin{itemize}
      \item start by stepping 5 $\sigma$ away from the
        MLE, or to the box constraint on the parameter,
        whichever is closer (\bbnote{this standard error is based on the
          curvature; I should allow it, or the intervals themselves,
          to be overridden via a \code{std.err} or \code{interval}
          parameter})
      \item compute the difference between the deviance and
        the desired deviance cutoff at this point;
        if it is \code{NA}, reduce the distance in steps
        of 0.25 $\sigma$ until it is not, until you reduce
        the distance to zero
      \item if the product of the deviance differences at the MLE
        and at the point you stopped at is \code{NA} or positive
        (indicating that you didn't find a root-crossing in the
        range $[0,5\sigma]$), quit.
      \item otherwise, apply \code{uniroot} across this interval
      \end{itemize}
      
      \code{method="uniroot"} should give the most accurate results, 
      especially when the profile is wonky (it won't care about non-smooth
      profiles), but it will be the slowest --- and different confidence
      levels will have to be computed individually, whereas multiple
      confidence levels can be computed quickly from a single computed
      profile.  A cruder approach would be to use profiling but decrease
      \code{std.err} a lot so that the profile points were very closely
      spaced.
    }
  \end{description}

\subsubsection{Profile plotting}

Plot the signed (or unsigned) square root deviance difference, and ($1-\alpha$) confidence regions/critical
values designated by \code{conf} (default: $\{0.99,0.95,0.9,0.8,0.5\}$).

\begin{itemize}
\item * If the (signed) profile is non-monotonic, simply plot  
  computed points with \code{type="l"} (i.e., with the default linear interpolation)
\item Construct the interpolation spline (using \code{splines:::interpSpline}
  rather than \code{spline} as in the confidence interval method (\bbnote{why this
    difference?})
\item attempt to construct the inverse of the interpolation spline (using \code{backSpline})
\item * if this fails warn the user (assume this was due to non-monotonicity)
  and try to use \code{uniroot} and \code{predict} to find cutoff values
\item otherwise, use the inverse spline to find cutoff values
\end{itemize}
\bbnote{Why is there machinery in the plotting code to find confidence intervals?
  Shouldn't this call \code{confint}, for consistency/fewer points of failure?}

\section*{Bugs, wishes, to do}
\begin{itemize}
\item \textbf{WISH}: further methods and arguments: \code{subset},
  \code{predict}, \code{resid}: \code{sim}?
\item \textbf{WISH}: extend ICtab to allow DIC as well?
\item minor \textbf{WISH}: 
  better methods for extracting \code{nobs} information
  when possible (e.g. with formula interface)
\item \textbf{WISH}: better documentation, especially for S4 methods
\item \textbf{WISH}: variable-length (and shaped) chunks in argument list -- cleaner division
  between linear model specs/list of arguments/vector equivalent
\item \textbf{WISH}: limited automatic differentiation
  (add capability for common distributions)
\item \textbf{WISH}: store \code{objectivefunction}
  and \code{objectivefunctiongr} (vectorized objective/gradient
  functions) in the \code{mle2} object (will break backward compatibility!!);
  add accessors for these and for \code{minuslogl}
\item \textbf{WISH}: document use of the objective function in \code{MCMCpack}
  to do {\emph post hoc} MCMC sampling (or write my own Metropolis-Hastings
  sampler \ldots)
\item \textbf{WISH}: polish profile plotting, with lattice or ggplot2
  methods
\item \textbf{WISH}: add in/document/demonstrate ``slice'' capabilities
\item \textbf{WISH}: refactor profiling to use stored objective functions
  rather than re-calling \code{mle2} with \code{fixed} values mucked around
  with in the calls???  Strip out and make generic for vectorized objective function?
  (\code{profileModel} package only works for glm-like objects, with a linear predictor)
\end{itemize}

\bibliography{mle2}
\end{document}

\documentclass{article}
%\VignettePackage{mle2}
%\VignetteIndexEntry{quasi: notes on quasi-likelihood/qAIC analysis inR}
%\VignetteDepends{MuMIn,AICcmodavg,bbmle}
%\VignetteEngine{knitr::knitr}

\usepackage{graphicx}
\usepackage{hyperref}
\usepackage{url}
\usepackage[utf8]{inputenc}
\newcommand{\code}[1]{{\tt #1}}
\title{Dealing with \code{quasi-}  models in R}
\date{\today}
\author{Ben Bolker}
\begin{document}
\newcommand{\rpkg}[1]{\href{https://CRAN.R-project.org/package=#1}{{\tt #1}}}
\maketitle

\includegraphics[width=2.64cm,height=0.93cm]{cc-attrib-nc.png}
\begin{minipage}[b]{3in}
{\tiny Licensed under the Creative Commons 
  attribution-noncommercial license
(\url{http://creativecommons.org/licenses/by-nc/3.0/}).
Please share \& remix noncommercially,
mentioning its origin.}
\end{minipage}

<<opts,echo=FALSE>>=
if (require("knitr")) opts_chunk$set(tidy=FALSE)
@ 
Computing ``quasi-AIC'' (QAIC), in R is a minor
pain, because  the R Core team (or at least the ones who wrote \code{glm},
\code{glmmPQL}, etc.) 
are purists and don't believe that quasi- models should report a likelihood.
As far as I know, there are three R packages that compute/handle
QAIC: 
\rpkg{bbmle}, \rpkg{AICcmodavg} and \rpkg{MuMIn}.


The basic problem is that quasi- model fits with \code{glm} return
an \code{NA} for the log-likelihood, while the dispersion parameter
($\hat c$, $\phi$, whatever you want to call it)
is only reported for quasi- models.
Various ways to get around this are:
\begin{itemize}
  \item{fit the model twice, once with a regular
      likelihood model (\code{family=binomial}, \code{poisson}, etc.)
      and once with the \code{quasi-} variant --- extract
      the log-likelihood from the former and the dispersion parameter
      from the latter}
    \item{only fit the regular model; extract
      the overdispersion parameter manually
      with
<<dfun>>=
dfun <- function(object) {
  with(object,sum((weights * residuals^2)[weights > 0])/df.residual)
}
@ 
}
\item{use the fact that quasi- fits still contain a deviance,
    even if they set the log-likelihood to \code{NA}.  The deviance
    is twice the negative log-likelihood (it's offset by some constant
    which I haven't figured out yet, but it should still work
    fine for model comparisons)}
\end{itemize}

The whole problem is worse for \code{MASS::glmmPQL}, where (1) the
authors have gone to greater efforts to make sure that the (quasi-)deviance
is no longer preserved anywhere in the fitted model, and (2) they
may have done it for good reason --- it is not clear whether the
number that would get left in the `deviance' slot at the end of
\code{glmmPQL}'s alternating \code{lme} and \code{glm} fits is
even meaningful to the extent that regular QAICs are.  (For 
discussion of a similar situation, see the \code{WARNING}
section of \code{?gamm} in the \code{mgcv} package.)

Example: use the values from one of the examples
in \code{?glm}:

<<dobdata>>=
## Dobson (1990) Page 93: Randomized Controlled Trial :
counts <- c(18,17,15,20,10,20,25,13,12)
outcome <- gl(3,1,9)
treatment <- gl(3,3)
@ 

Fit Poisson and quasi-Poisson models with all combinations
of predictors:

<<fitdob>>=
glmOT.D93 <- glm(counts ~ outcome + treatment, family=poisson)
glmO.D93  <- update(glmOT.D93, . ~ . - treatment)
glmT.D93  <- update(glmOT.D93, . ~ . - outcome)
glmX.D93  <- update(glmT.D93, . ~ . - treatment)
glmQOT.D93 <- update(glmOT.D93, family=quasipoisson)
glmQO.D93 <- update(glmO.D93, family=quasipoisson)
glmQT.D93 <- update(glmT.D93, family=quasipoisson)
glmQX.D93 <- update(glmX.D93, family=quasipoisson)
@ 


Extract log-likelihoods:
<<dobll>>=
(sum(dpois(counts,
          lambda=exp(predict(glmOT.D93)),log=TRUE))) ## by hand
(logLik(glmOT.D93))  ## from Poisson fit
@ 

The deviance (\code{deviance(glmOT.D93)}=\Sexpr{round(deviance(glmOT.D93),3)}
is not the same as $-2L$ (\code{-2*logLik(glmOT.D93)}=\Sexpr{round(-2*c(logLik(glmOT.D93)),3)}),
but the calculated differences in deviance are consistent,
and are also extractable from the quasi- fit even though
the log-likelihood is \code{NA}:
<<dobll2>>=
(-2*(logLik(glmT.D93)-logLik(glmOT.D93)))  ## Poisson fit
(deviance(glmT.D93)-deviance(glmOT.D93))   ## Poisson fit
(deviance(glmQT.D93)-deviance(glmQOT.D93)) ## quasi-fit
@ 


Compare hand-computed dispersion (in two ways)
with the dispersion computed by \code{summary.glm()}
on a quasi- fit:

<<dobdisp>>=
(dfun(glmOT.D93))
(sum(residuals(glmOT.D93,"pearson")^2)/glmOT.D93$df.residual)
(summary(glmOT.D93)$dispersion)
(summary(glmQOT.D93)$dispersion)
@ 


\section*{Examples}

\subsection*{\code{bbmle}}

<<bbmle>>=
library(bbmle)
(qAIC(glmOT.D93,dispersion=dfun(glmOT.D93)))
(qAICc(glmOT.D93,dispersion=dfun(glmOT.D93),nobs=length(counts)))
ICtab(glmOT.D93,glmT.D93,glmO.D93,glmX.D93,
      dispersion=dfun(glmOT.D93),type="qAIC")
ICtab(glmOT.D93,glmT.D93,glmO.D93,glmX.D93,
      dispersion=dfun(glmOT.D93),
      nobs=length(counts),type="qAICc")
detach("package:bbmle")
@ 

\subsection*{\code{AICcmodavg}}

<<AICcmodavg>>=
if (require("AICcmodavg")) {
    aictab(list(glmOT.D93,glmT.D93,glmO.D93,glmX.D93), 
           modnames=c("OT","T","O","X"),
           c.hat=dfun(glmOT.D93))
    detach("package:AICcmodavg")
}
@ 

\subsection*{\code{MuMIn}}

<<MuMin>>=
if (require("MuMIn")) {
    packageVersion("MuMIn")
    ## from ?QAIC
    x.quasipoisson <- function(...) {
        res <- quasipoisson(...)
        res$aic <- poisson(...)$aic
        res
    }
    glmQOT2.D93 <- update(glmOT.D93,family="x.quasipoisson",
                          na.action=na.fail)
    (gg <-  dredge(glmQOT2.D93,rank="QAIC", chat=dfun(glmOT.D93)))
    (ggc <- dredge(glmQOT2.D93,rank="QAICc",chat=dfun(glmOT.D93)))
    detach("package:MuMIn")
}
@ 

Notes: ICtab only gives delta-IC, limited decimal places
(on purpose, but how do you change these defaults if you
want to?).  Need to add 1 to parameters
to account for scale parameter.  When doing corrected-IC you need
to get the absolute number of parameters right, not just the relative
number \ldots Not sure which classes of models each of these
will handle (lm, glm, (n)lme, lme4, mle2 \ldots).  Remember
need to use overdispersion parameter from most complex model.
glmmPQL: needs to be hacked somewhat more severely (does not
contain deviance element, logLik has been NA'd out).

\begin{tabular}{l|ccccccc}
  package & \code{lm} & \code{glm} & \code{(n)lme} & \code{multinom} & \code{polr} & \code{lme4} & \code{mle2} \\
  \hline
  \code{AICcmodavg} & y & y & y & y & y & ? & ? \\
  \code{MuMIn}      & ? & ? & ? & ? & ? & ? & ? \\
  \code{mle2 }      & ? & ? & ? & ? & ? & ? & ?
\end{tabular}
  
\end{document}

\name{predict-methods}
\docType{methods}
\alias{gfun}
\alias{predict-methods}
\alias{predict,mle2-method}
\alias{residuals,mle2-method}
\alias{simulate,mle2-method}
\title{Predicted values from an mle2 fit}
\description{
  Given an \code{mle2} fit and an optional list
  of new data, return predictions (more generally,
  summary statistics of the predicted distribution)
}
\section{Methods}{
\describe{
  \item{x = "mle2"}{an \code{mle2} fit}
}}
\usage{
    \S4method{predict}{mle2}(object, newdata=NULL,
                         location="mean", newparams=NULL, \dots)
    \S4method{simulate}{mle2}(object, nsim,
                         seed, newdata=NULL, newparams=NULL, \dots)
    \S4method{residuals}{mle2}(object,type=c("pearson","response"),
                   location="mean",\dots)
}
\arguments{
  \item{object}{an mle2 object}
  \item{newdata}{optional list of new data}
  \item{newparams}{optional vector of new parameters}
  \item{location}{name of the summary statistic to return}
  \item{nsim}{number of simulations}
  \item{seed}{random number seed}
  \item{type}{residuals type}
  \item{\dots}{additional arguments (for generic compatibility)}
}
\note{For some models (e.g. constant models), \code{predict} may
  return a single value rather than a vector of the appropriate length.}
\examples{
set.seed(1002)
lymax <- c(0,2)
lhalf <- 0
x <- runif(200)
g <- factor(rep(c("a","b"),each=100))
y <- rnbinom(200,mu=exp(lymax[g])/(1+x/exp(lhalf)),size=2)
dat <- data.frame(y,g,x)

fit3 <- mle2(y~dnbinom(mu=exp(lymax)/(1+x/exp(lhalf)),size=exp(logk)),
    parameters=list(lymax~g),
    start=list(lymax=0,lhalf=0,logk=0),
data=dat)

plot(y~x,col=g)
## true curves
curve(exp(0)/(1+x/exp(0)),add=TRUE)
curve(exp(2)/(1+x/exp(0)),col=2,add=TRUE)
## model predictions
xvec = seq(0,1,length=100)
lines(xvec,predict(fit3,newdata=list(g=factor(rep("a",100),levels=c("a","b")),
                                x = xvec)),col=1,lty=2)
lines(xvec,predict(fit3,newdata=list(g=factor(rep("b",100),levels=c("a","b")),
                                x = xvec)),col=2,lty=2)


## comparing automatic and manual predictions
p1 = predict(fit3)
p2A =
with(as.list(coef(fit3)),exp(`lymax.(Intercept)`)/(1+x[1:100]/exp(lhalf)))
p2B =
with(as.list(coef(fit3)),exp(`lymax.(Intercept)`+lymax.gb)/(1+x[101:200]/exp(lhalf)))
all(p1==c(p2A,p2B))
##
simulate(fit3)
}
\keyword{methods}

\name{ICtab}
\alias{ICtab}
\alias{AICtab}
\alias{BICtab}
\alias{AICctab}
\alias{print.ICtab}
\title{Compute table of information criteria and auxiliary info}
\description{
  Computes information criteria for a series of models, optionally
  giving information about weights, differences between ICs, etc.
}
\usage{
ICtab(\dots, type=c("AIC","BIC","AICc","qAIC","qAICc"),
    weights = FALSE, delta = TRUE, base = FALSE,
logLik=FALSE, sort = TRUE,
nobs=NULL, dispersion = 1, mnames, k = 2)
AICtab(\dots,mnames)
BICtab(\dots,mnames)
AICctab(\dots,mnames)
\method{print}{ICtab}(x,\dots,min.weight)
}
\arguments{
  \item{\dots}{a list of (logLik or?) mle objects; in the case of
    \code{AICtab} etc., could also include other arguments to \code{ICtab}}
  \item{type}{specify information criterion to use}
  \item{base}{(logical) include base IC (and log-likelihood) values?}
  \item{weights}{(logical) compute IC weights?}
  \item{logLik}{(logical) include log-likelihoods in the table?}
  \item{delta}{(logical) compute differences among ICs (and log-likelihoods)?}
  \item{sort}{(logical) sort ICs in increasing order?}
  \item{nobs}{(integer) number of observations: required for
    \code{type="BIC"} or \code{type="AICc"} unless objects have
    a \code{\link{nobs}} method}
  \item{dispersion}{overdispersion estimate, for computing qAIC:
    required for \code{type="qAIC"} or \code{type="qAICc"} unless
    objects have a \code{"dispersion"} attribute}
  \item{mnames}{names for table rows: defaults to names of objects passed}
  \item{k}{penalty term (largely unused: left at default of 2)}
  \item{x}{an ICtab object}
  \item{min.weight}{minimum weight for exact reporting (smaller values
    will be reported as "<[min.weight]")}
}
\value{
  A data frame containing:
  \item{IC}{information criterion}
  \item{df}{degrees of freedom/number of parameters}
  \item{dIC}{difference in IC from minimum-IC model}
  \item{weights}{exp(-dIC/2)/sum(exp(-dIC/2))}
}
\note{(1) The print method uses sensible defaults; all ICs are rounded
  to the nearest 0.1, and IC weights are printed using
  \code{\link{format.pval}} to print an inequality for
  values <0.001. (2) The computation of degrees of freedom/number of
  parameters (e.g., whether
  variance parameters are included in the total) varies enormously
  between packages.  As long as the df computations
  for a given set of models is consistent, differences
  don't matter, but one needs to be careful with log likelihoods
  and models taken from different packages.  If necessary
  one can change the degrees of freedom manually by
  saying \code{attr(obj,"df") <- df.new}, where \code{df.new}
  is the desired number of parameters.
  (3) Defaults have changed to \code{sort=TRUE}, \code{base=FALSE},
  \code{delta=TRUE}, to match my conviction that it rarely makes
  sense to report the overall values of information criteria}
\references{Burnham and Anderson 2002}
\author{Ben Bolker}
\examples{
  set.seed(101)
  d <- data.frame(x=1:20,y=rpois(20,lambda=2))
  m0 <- glm(y~1,data=d)
  m1 <- update(m0,.~x)
  m2 <- update(m0,.~poly(x,2))
  AICtab(m0,m1,m2,mnames=LETTERS[1:3])
  AICtab(m0,m1,m2,base=TRUE,logLik=TRUE)
  AICtab(m0,m1,m2,logLik=TRUE)
  AICctab(m0,m1,m2,weights=TRUE)
  print(AICctab(m0,m1,m2,weights=TRUE),min.weight=0.1)
}
\keyword{misc}

\name{call.to.char}
\alias{call.to.char}
\title{Convert calls to character}
\description{
  Utility function (hack) to convert calls such
  as y~x to their character equivalent
}
\usage{
call.to.char(x)
}
\arguments{
  \item{x}{a formula (call)}
}
\details{
  It would be nice if \code{as.character(y~x)}
  gave "y~x", but it doesn't, so this hack achieves
  the same goal
}
\value{
  a character vector of length 1
}
\author{Ben Bolker}
\examples{
as.character(y~x)
call.to.char(y~x)
}
\keyword{misc}

\name{sbinom}
\alias{sbinom}
\alias{spois}
\alias{snbinom}
\alias{snorm}
\alias{sbeta}
\alias{sbetabinom}
\alias{slnorm}
\title{Abstract definitions of distributions}
\description{
  Functions returning values for summary statistics
  (mean, median, etc.) of distributions
}
\usage{
sbeta(shape1, shape2)
sbetabinom(size, prob, theta)
sbinom(size, prob)
snbinom(size, prob, mu)
snorm(mean, sd)
spois(lambda)
slnorm(meanlog, sdlog)
}
\arguments{
  \item{prob}{probability as defined for \code{\link{dbinom}},
    \code{\link{dnbinom}}, or beta-binomial distribution
    (\code{dbetabinom} in the \code{emdbook} package)}
  \item{size}{size parameter as defined for
    \code{\link{dbinom}} or \code{dbetabinom}
    in the \code{emdbook} package, or size/overdispersion parameter
    as in \code{\link{dnbinom}}}
  \item{mean}{mean parameter as defined for \code{\link{dnorm}}}
  \item{mu}{mean parameter as defined for \code{\link{dnbinom}}}
  \item{sd}{standard deviation parameter as defined for \code{\link{dnorm}}}
  \item{shape1}{shape parameter for \code{\link{dbeta}}}
  \item{shape2}{shape parameter for \code{\link{dbeta}}}
  \item{lambda}{rate parameter as defined for \code{\link{dpois}}}
  \item{theta}{overdispersion parameter for beta-binomial
    (see \code{dbetabinom} in the \code{emdbook} package)}
  \item{meanlog}{as defined for \code{\link{dlnorm}}}
  \item{sdlog}{as defined for \code{\link{dlnorm}}}

}
\value{
  \item{title}{name of the distribution}
  \item{[parameters]}{input parameters for the distribution}
  \item{mean}{theoretical mean of the distribution}
  \item{median}{theoretical median of the distribution}
  \item{mode}{theoretical mode of the distribution}
  \item{variance}{theoretical variance of the distribution}
  \item{sd}{theoretical standard deviation of the distribution}
}
\author{Ben Bolker}
\seealso{\code{\link{dbinom}}, \code{\link{dpois}}, \code{\link{dnorm}}, \code{\link{dnbinom}}}
\examples{
  sbinom(prob=0.2,size=10)
  snbinom(mu=2,size=1.2)
}
\note{these definitions are tentative, subject to change
as I figure this out better.  Perhaps construct functions
that return functions? Strip down results? Do more
automatically?}

\keyword{misc}

\name{profile.mle2-class}
\docType{class}
\alias{profile.mle2-class}
\alias{confint,profile.mle2-method}
\alias{confint,mle2-method}
\alias{confint.mle2}  %% bogus but good ref link
\alias{plot,profile.mle2-method}
\alias{plot,profile.mle2,missing-method}
\alias{show,profile.mle2-method}
\alias{plot.profile.mle2}
\title{Methods for likelihood profiles}
\description{Definition of the mle2 likelihood profile class,
  and applicable methods}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("profile.mle2",
    ...)}, but most often by invoking \code{profile} on an "mle2" object.
}
\section{Slots}{
  \describe{
    \item{\code{profile}:}{Object of class \code{"list"}. List of
      profiles, one for each requested parameter. Each profile is a data
      frame with the first column called \code{z} being the signed square
      root of the deviance, and the others being the
      parameters with names prefixed by \code{par.vals.}}
    \item{\code{summary}:}{Object of class \code{"summary.mle2"}. Summary
      of object being profiled.}
  }
}
\section{Methods}{
  \describe{
    \item{confint}{\code{signature(object = "profile.mle2")}: Use profile
      to generate approximate confidence intervals for parameters.}
    \item{plot}{\code{signature(x = "profile.mle2", y = "missing")}: Plot
      profiles for each parameter.}
    \item{summary}{\code{signature(x = "profile.mle2")}: Plot
      profiles for each parameter.}
    \item{show}{\code{signature(object = "profile.mle2")}: Show object.}
  }
}
\usage{
\S4method{plot}{profile.mle2}(x,
   levels, which=1:p, conf = c(99, 95, 90, 80, 50)/100,
   plot.confstr = TRUE,
   confstr = NULL, absVal = TRUE, add = FALSE,
   col.minval="green", lty.minval=2,
   col.conf="magenta", lty.conf=2,
   col.prof="blue", lty.prof=1,
   xlabs=nm, ylab="z",
   onepage=TRUE,
   ask=((prod(par("mfcol")) < length(which)) && dev.interactive() &&
               !onepage),
   show.points=FALSE,
   main, xlim, ylim, \dots)
\S4method{confint}{mle2}(object, parm, level = 0.95, method,
          trace=FALSE,quietly=!interactive(),
          tol.newmin=0.001,\dots)
\S4method{confint}{profile.mle2}(object, parm, level = 0.95, trace=FALSE, \dots)
}
\arguments{
  \item{x}{An object of class \code{profile.mle2}}
  \item{object}{An object of class \code{mle2} or \code{profile.mle2}
    (as appropriate)}
  \item{levels}{levels at which to plot likelihood cutoffs (set by conf
    by default)}
  \item{level}{level at which to compute confidence interval}
  \item{which}{(numeric or character) which parameter profiles to plot}
  \item{parm}{(numeric or character) which parameter(s) to find
    confidence intervals for}
  \item{method}{(character) "spline", "uniroot", or "quad", for
    spline-extrapolation-based (default), root-finding, or quadratic
    confidence intervals.  By default it uses the value of
    \code{mle2.options("confint")} -- the factory setting is "spline".}
  \item{trace}{trace progress of confidence interval calculation when using
    \sQuote{uniroot} method?}
  \item{conf}{(1-alpha) levels at which to plot likelihood
    cutoffs/confidence intervals}
  \item{quietly}{(logical) suppress \dQuote{Profiling ...} message
    when computing profile to get confidence interval?}
  \item{tol.newmin}{see \code{\link{profile-methods}}}
  \item{plot.confstr}{(logical) plot labels showing confidence levels?}
  \item{confstr}{(character) labels for confidence levels (by default, constructed from conf levels)}
  \item{absVal}{(logical) plot absolute values of signed square
    root deviance difference ("V" plot rather than straight-line
    plot)?}
  \item{add}{(logical) add profile to existing graph?}
  \item{col.minval}{color for minimum line}
  \item{lty.minval}{line type for minimum line}
  \item{col.conf}{color for confidence intervals}
  \item{lty.conf}{line type for confidence intervals}
  \item{col.prof}{color for profile}
  \item{lty.prof}{line type for profile}
  \item{xlabs}{x labels}
  \item{ylab}{y label}
  \item{onepage}{(logical) plot all profiles on one page,
    adjusting par(mfcol) as necessary?}
  \item{ask}{(logical) pause for user input between plots?}
  \item{show.points}{(logical) show computed profile points
    as well as interpolated spline?}
  \item{main}{(logical) main title}
  \item{xlim}{x limits}
  \item{ylim}{y limits}
  \item{\dots}{other arguments}
}
\seealso{
  \code{\link{mle2}}, \code{\link{mle2-class}}, \code{\link{summary.mle2-class}} 
}
\details{
  The default confidence interval calculation computes a likelihood
  profile and uses the points therein, or uses the computed points in
  an existing \code{profile.mle2} object, to construct an interpolation
  spline (which by default has three times as many points as were in
  the original set of profile points).  It then uses linear
  interpolation between these interpolated points (!)
}
\examples{
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)
## we have a choice here: (1) don't impose boundaries on the parameters,
##  put up with warning messages about NaN values: 
fit1 <- mle2(y~dpois(lambda=ymax/(1+x/xhalf)),
     start=list(ymax=1,xhalf=1),
     data=d)
p1 <- suppressWarnings(profile(fit1))
plot(p1,main=c("first","second"),
     xlab=c(~y[max],~x[1/2]),ylab="Signed square root deviance",
     show.points=TRUE)
suppressWarnings(confint(fit1)) ## recomputes profile
confint(p1)  ## operates on existing profile
suppressWarnings(confint(fit1,method="uniroot"))
## alternatively, we can use box constraints to keep ourselves
##  to positive parameter values ...
fit2 <- update(fit1,method="L-BFGS-B",lower=c(ymax=0.001,xhalf=0.001))
\dontrun{
p2 <- profile(fit2)
plot(p2,show.points=TRUE)
## but the fit for ymax is just bad enough that the spline gets wonky
confint(p2)  ## now we get a warning
confint(fit2,method="uniroot")
## bobyqa is a better-behaved bounded optimizer ...
##  BUT recent (development, 2012.5.24) versions of
##    optimx no longer allow single-parameter fits!
if (require(optimx)) {
  fit3 <- update(fit1,
      optimizer="optimx",
      method="bobyqa",lower=c(ymax=0.001,xhalf=0.001))
   p3 <- profile(fit3)
   plot(p3,show.points=TRUE)
  confint(p3)
}
}
}
\keyword{classes}

\name{as.data.frame.profile.mle2}
\alias{as.data.frame.profile.mle2}
\alias{coerce,profile.mle2-method}
\alias{coerce,profile.mle2,data.frame-method}
\title{convert profile to data frame}
\description{
  converts a profile of a fitted mle2 object
  to a data frame
}
\usage{
\S3method{as.data.frame}{profile.mle2}(x, row.names=NULL,
optional=FALSE, \dots)
}
\arguments{
  \item{x}{a profile object}
  \item{row.names}{row names (unused)}
  \item{optional}{unused}
  \item{\dots}{unused}
}
\value{
  a data frame with columns
  \item{param}{name of parameter being profiled}
  \item{z}{signed square root of the deviance difference from the
    minimum}
  \item{parameter values}{named par.vals.parname}
  \item{focal}{value of focal parameter: redundant, but included for
    plotting convenience}
}
\examples{
  ## use as.data.frame and lattice to plot profiles
  x <- 0:10
  y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
  library(bbmle)
  LL <- function(ymax=15, xhalf=6) {
        -sum(stats::dpois(y, lambda=ymax/(1+x/xhalf), log=TRUE))
    }
  ## uses default parameters of LL
  fit1 <- mle2(LL)
  p1 <- profile(fit1)
  d1 <- as.data.frame(p1)
  library(lattice)
  xyplot(abs(z)~focal|param,data=d1,
    subset=abs(z)<3,
    type="b",
    xlab="",
    ylab=expression(paste(abs(z),
    " (square root of ",Delta," deviance)")),
    scale=list(x=list(relation="free")))
}
\author{Ben Bolker}
\keyword{misc}

\name{slice}
\alias{slice}
\alias{sliceOld}
\alias{slicetrans}
\alias{slice1D}
\alias{slice2D}
\title{Calculate likelihood "slices"}
\description{
  Computes cross-section(s) of a multi-dimensional likelihood surface
}
\usage{
slice(x, dim=1, ...)
sliceOld(fitted, which = 1:p, maxsteps = 100,
                       alpha = 0.01, zmax = sqrt(qchisq(1 - alpha/2, p)),
                       del = zmax/5, trace = FALSE,
                       tol.newmin=0.001, \dots)
slice1D(params,fun,nt=101,lower=-Inf,
                    upper=Inf,verbose=TRUE, tranges=NULL,\dots)
slice2D(params,fun,nt=31,lower=-Inf,
                    upper=Inf,
                    cutoff=10,verbose=TRUE,
                    tranges=NULL, \dots)
slicetrans(params, params2, fun, extend=0.1, nt=401,
                       lower=-Inf, upper=Inf)
}
\arguments{
  \item{x}{a fitted model object of some sort}
  \item{dim}{dimensionality of slices (1 or 2)}
  \item{params}{a named vector of baseline parameter values}
  \item{params2}{a vector of parameter values}
  \item{fun}{an objective function}
  \item{nt}{(integer) number of slice-steps to take}
  \item{lower}{lower bound(s) (stub?)}
  \item{upper}{upper bound(s) (stub?)}
  \item{cutoff}{maximum increase in objective function to allow
    when computing ranges}
  \item{extend}{(numeric) fraction by which to extend range beyond specified points}
  \item{verbose}{print verbose output?}
  \item{fitted}{A fitted maximum likelihood model of class
    \dQuote{mle2}}
  \item{which}{a numeric or character vector describing which parameters
    to profile (default is to profile all parameters)}
  \item{maxsteps}{maximum number of steps to take looking for an upper
    value of the negative log-likelihood}
  \item{alpha}{maximum (two-sided) likelihood ratio test confidence
    level to find}
  \item{zmax}{maximum value of signed square root of deviance difference
    to find (default value corresponds to a
    2-tailed chi-squared test at level alpha)}
  \item{del}{step size for profiling}
  \item{trace}{(logical) produce tracing output?}
  \item{tol.newmin}{tolerance for diagnosing a new minimum below the
    minimum deviance estimated in initial fit is found}
  \item{tranges}{a two-column matrix giving lower and upper bounds for
    each parameter}
  \item{\dots}{additional arguments (not used)}
}
\value{
  An object of class \code{slice} with
  \describe{
    \item{slices}{a list of individual parameter (or parameter-pair)
    slices, each of which is a data frame with elements
    \describe{
      \item{var1}{name of the first variable}
      \item{var2}{(for 2D slices) name of the second variable}
      \item{x}{parameter values}
      \item{y}{(for 2D slices) parameter values}
      \item{z}{slice values}
      \item{ranges}{a list (?) of the ranges for each parameter}
      \item{params}{vector of baseline parameter values}
      \item{dim}{1 or 2}
    }
  }
  \code{sliceOld} returns instead a list with elements \code{profile}
  and \code{summary} (see \code{\link{profile.mle2}})
}
}
\details{
  Slices provide a lighter-weight way to explore likelihood surfaces
  than profiles, since they vary a single parameter rather than
  optimizing over all but one or two parameters.

\describe{
\item{slice}{is a generic method}
\item{slice1D}{creates one-dimensional slices, by default of all
parameters of a model}
\item{slice2D}{creates two-dimensional slices, by default of all pairs
  of parameters in a model. In each panel the closed point represents
  the parameters given (typically the MLEs), while the open point
  represents the observed minimum value within the 2D slice.
  If everything has gone according to plan, these points should
  coincide (at least up to grid precision).
}
\item{slicetrans}{creates a slice along a transect between two specified
points in parameter space (see \code{calcslice} in the \code{emdbook}
package)}
}
}
\author{Ben Bolker}
\seealso{\code{\link{profile}}}
\examples{
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)
fit1 <- mle2(y~dpois(lambda=exp(lymax)/(1+x/exp(lhalf))),
   start=list(lymax=0,lhalf=0),
   data=d)
s1 <- bbmle::slice(fit1,verbose=FALSE)
s2 <- bbmle::slice(fit1,dim=2,verbose=FALSE)
require(lattice)
plot(s1)
plot(s2)
## 'transect' slice, from best-fit values to another point
st <- bbmle::slice(fit1,params2=c(5,0.5))
plot(st)
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impsamp.R
\name{pop_pred_samp}
\alias{pop_pred_samp}
\title{generate population prediction sample from parameters}
\usage{
pop_pred_samp(object, n = 1000, n_imp = n * 10, return_wts = FALSE,
  impsamp = FALSE, PDify = FALSE, PDmethod = NULL, tol = 1e-06,
  return_all = FALSE, rmvnorm_method = c("mvtnorm", "MASS"),
  fix_params = NULL)
}
\arguments{
\item{object}{a fitted \code{mle2} object}

\item{n}{number of samples to return}

\item{n_imp}{number of total samples from which to draw, if doing importance sampling}

\item{return_wts}{return a column giving the weights of the samples, for use in weighted summaries?}

\item{impsamp}{subsample values (with replacement) based on their weights?}

\item{PDify}{use Gill and King generalized-inverse procedure to correct non-positive-definite variance-covariance matrix if necessary?}

\item{PDmethod}{method for fixing non-positive-definite covariance matrices}

\item{tol}{tolerance for detecting small eigenvalues}

\item{return_all}{return a matrix including all values, and weights (rather than taking a sample)}

\item{rmvnorm_method}{package to use for generating MVN samples}

\item{fix_params}{parameters to fix (in addition to parameters that were fixed during estimation)}
}
\description{
This [EXPERIMENTAL] function combines several sampling tricks to compute a version of an importance sample (based on flat priors) for the parameters.
}
\references{
Gill, Jeff, and Gary King. "What to Do When Your Hessian Is Not Invertible: Alternatives to Model Respecification in Nonlinear Estimation." Sociological Methods & Research 33, no. 1 (2004): 54-87.
Lande, Russ and Steinar Engen and Bernt-Erik Saether, Stochastic Population Dynamics in Ecology and Conservation. Oxford University Press, 2003.
}

\name{parnames}
\alias{parnames}
\alias{parnames<-}
\title{get and set parameter names}
\description{
  Gets and sets the "parnames" attribute on a
  negative log-likelihood function
}
\usage{
parnames(obj)
parnames(obj) <- value
}
\arguments{
  \item{obj}{a negative log-likelihood function}
  \item{value}{a character vector of parameter names}
}
\details{
  The \code{parnames} attribute is used by \code{mle2()}
  when the negative log-likelihood function takes a parameter
  vector, rather than a list of parameters; this allows
  users to use the same objective function for \code{optim()}
  and \code{mle2()}
}
\value{
  Returns the \code{parnames} attribute (a character vector of
  parameter names) or sets it.
}
\author{Ben Bolker}
\examples{
x <- 1:5
set.seed(1001)
y <- rbinom(5,prob=x/(1+x),size=10)
mfun <- function(p) {
  a <- p[1]
  b <- p[2]
  -sum(dbinom(y,prob=a*x/(b+x),size=10,log=TRUE))
}
optim(fn=mfun,par=c(1,1))
parnames(mfun) <- c("a","b")
mle2(minuslogl=mfun,start=c(a=1,b=1),method="Nelder-Mead")
}
\keyword{misc}

\name{mle2-class}
\docType{class}
\alias{mle2-class}
\alias{coef,mle2-method}
\alias{show,mle2-method}
\alias{slice,mle2-method}
\alias{summary,mle2-method}
\alias{update,mle2-method}
\alias{vcov,mle2-method}
\alias{deviance,mle2-method}
\alias{coerce,mle,mle2-method}
\alias{formula,mle2-method}
\alias{stdEr}
\alias{stdEr,mle2-method}
\title{Class "mle2". Result of Maximum Likelihood Estimation.}
\description{This class encapsulates results of a generic maximum
  likelihood procedure.}
\section{Objects from the Class}{
  Objects can be created by calls of the form \code{new("mle2", \dots)}, but
  most often as the result of a call to \code{\link{mle2}}.
}
\section{Slots}{
  \describe{
    \item{\code{call}:}{(language) The call to \code{\link{mle2}}.}
    \item{\code{call.orig}:}{(language) The call to \code{\link{mle2}},
      saved in its original form (i.e. without data arguments
      evaluated).}
    \item{\code{coef}:}{(numeric) Vector of estimated parameters.}
    \item{\code{data}:}{(data frame or list) Data with which to evaluate the negative log-likelihood function}
    \item{\code{fullcoef}:}{(numeric) Fixed and estimated parameters.}
    \item{\code{vcov}:}{(numeric matrix) Approximate variance-covariance
      matrix, based on the second derivative matrix at the MLE.}
    \item{\code{min}:}{(numeric) Minimum value of objective function =
      minimum negative log-likelihood.}
    \item{\code{details}:}{(list) Return value from \code{\link{optim}}.}
    \item{\code{minuslogl}:}{(function) The negative log-likelihood
      function.}
    \item{\code{optimizer}:}{(character) The optimizing function used.}
    \item{\code{method}:}{(character) The optimization method used.}
    \item{\code{formula}:}{(character) If a formula was specified, a
      character vector giving the formula and parameter specifications.}
  }
}
\section{Methods}{
  \describe{
    \item{coef}{\code{signature(object = "mle2")}: Extract coefficients.
      If \code{exclude.fixed=TRUE} (it is \code{FALSE} by default),
      only the non-fixed parameter values are returned.}
    \item{confint}{\code{signature(object = "mle2")}: Confidence
      intervals from likelihood profiles, or quadratic approximations,
      or root-finding.}
    \item{show}{\code{signature(object = "mle2")}: Display object
      briefly.}
    \item{show}{\code{signature(object = "summary.mle2")}: Display object briefly.}
    \item{summary}{\code{signature(object = "mle2")}: Generate object summary.}
    \item{update}{\code{signature(object = "mle2")}:  Update fit.}
    \item{vcov}{\code{signature(object = "mle2")}: Extract
      variance-covariance matrix.}
    \item{formula}{\code{signature(object="mle2")}: Extract formula}
    \item{plot}{\code{signature(object="profile.mle2,missing")}: Plot
      profile. }
  }
}
\section{Details on the confint method}{
  When the parameters in the original fit are constrained using
  \code{lower} or \code{upper}, or when \code{prof.lower} or
  \code{prof.upper} are set, and the confidence intervals lie
  outside the constraint region, \code{confint} will return \code{NA}.
  This may be too conservative -- in some cases, the appropriate
  answer would be to set the confidence limit to the lower/upper
  bound as appropriate -- but it is the most general answer.

  (If you have a strong opinion about the need for a new
  option to \code{confint} that sets the bounds to the limits
  automatically, please contact the package maintainer.)
}
\examples{
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
lowerbound <- c(a=2,b=-0.2)
d <- data.frame(x,y)
fit1 <- mle2(y~dpois(lambda=exp(a+b*x)),start=list(a=0,b=2),data=d,
method="L-BFGS-B",lower=c(a=2,b=-0.2))
(cc <- confint(fit1,quietly=TRUE))
## to set the lower bounds to the limit
na_lower <- is.na(cc[,1])
cc[na_lower,1] <- lowerbound[na_lower]
cc
}
\keyword{classes}

\name{mle2}
\alias{mle2}
\alias{mle}
\alias{calc_mle2_function}
\title{Maximum Likelihood Estimation}
\description{
  Estimate parameters by the method of maximum likelihood.
}
\usage{
mle2(minuslogl, start, method, optimizer,
    fixed = NULL, data=NULL,
    subset=NULL,
default.start=TRUE, eval.only = FALSE, vecpar=FALSE,
parameters=NULL,
parnames=NULL,
skip.hessian=FALSE,
hessian.opts=NULL,
use.ginv=TRUE,
trace=FALSE,
browse_obj=FALSE,
gr=NULL,
optimfun,\dots)
calc_mle2_function(formula,parameters, links, start,
   parnames, use.deriv=FALSE, data=NULL,trace=FALSE)
}
\arguments{
  \item{minuslogl}{Function to calculate negative log-likelihood,
  or a formula}
  \item{start}{Named list. Initial values for optimizer}
  \item{method}{Optimization method to use. See \code{\link{optim}}.}
  \item{optimizer}{Optimization function to use. Currently available
    choices are "optim" (the default), "nlm", "nlminb", "constrOptim",
    "optimx", and "optimize". If "optimx" is used, (1) the \code{optimx}
    package must be explicitly loaded with \code{\link{load}} or
    \code{\link{require}}(\emph{Warning:} Options other than the
    default may be poorly tested, use with caution.)
  }
  \item{fixed}{Named list.  Parameter values to keep fixed during
    optimization.}
  \item{data}{list of data to pass to negative log-likelihood function: must
    be specified if \code{minuslogl} is specified as a formula}
  \item{subset}{logical vector for subsetting data (STUB)}
  \item{default.start}{Logical: allow default values of \code{minuslogl}
    as starting values?}
  \item{eval.only}{Logical: return value of \code{minuslogl(start)}
    rather than optimizing}
  \item{vecpar}{Logical: is first argument a vector of all parameters?
    (For compatibility with \code{\link{optim}}.)
  If \code{vecpar} is \code{TRUE}, then you should use
  \code{\link{parnames}} to define the parameter names for the
  negative log-likelihood function.}
  \item{parameters}{List of linear models for parameters.
    \emph{MUST BE SPECIFIED IN THE SAME ORDER as the start vector
      (this is a bug/restriction that I hope to fix soon, but in
      the meantime beware)}}
  \item{links}{(unimplemented) specify transformations of parameters}
  \item{parnames}{List (or vector?) of parameter names}
  \item{gr}{gradient function}
  \item{\dots}{Further arguments to pass to optimizer}
  \item{formula}{a formula for the likelihood (see Details)}
  \item{trace}{Logical: print parameter values tested?}
  \item{browse_obj}{Logical: drop into browser() within the objective function?}
  \item{skip.hessian}{Bypass Hessian calculation?}
  \item{hessian.opts}{Options for Hessian calculation, passed through to
    the \code{\link[numDeriv]{hessian}} function}
  \item{use.ginv}{Use generalized inverse (\code{\link[MASS]{ginv}}) to
    compute approximate variance-covariance}
  \item{optimfun}{user-supplied optimization function. Must take exactly
    the same arguments and return exactly the same structure as \code{\link{optim}}.}
  \item{use.deriv}{(experimental, not yet implemented): construct symbolic
     derivatives based on formula?}
 }
 \section{Warning}{Do not use a higher-level variable named \code{.i} in
   \code{parameters} -- this is reserved for internal use.
 }
\details{
  The \code{\link{optim}} optimizer is used to find the minimum of the
  negative log-likelihood.  An approximate covariance matrix for the
  parameters is obtained by inverting the Hessian matrix at the optimum.

  The \code{minuslogl} argument can also specify a formula,
  rather than an objective function, of the
  form \code{x~ddistn(param1,...,paramn)}.  In this case
  \code{ddistn} is taken to be a probability or density
  function, which must have (literally) \code{x} as its
  first argument (although this argument may be interpreted as
  a matrix of multivariate responses) and which must have
  a \code{log} argument that can be used to specify the
  log-probability or log-probability-density is required.
  If a formula is specified, then \code{parameters} can contain
  a list of linear models for the parameters.

  If a formula is given and non-trivial linear models are given
  in \code{parameters} for some of the variables, then
  model matrices will be generated using \code{model.matrix}.
  \code{start} can be given:
  \itemize{
    \item as a list containing lists, with each list corresponding
    to the starting values for a particular parameter;
    \item just for the higher-level parameters, in which case
    all of the additional parameters generated by \code{model.matrix}
    will be given starting values of zero (unless a no-intercept
    formula with \code{-1} is specified, in which case all the
    starting values for that parameter will be set equal)
    \item (to be implemented!) as an exhaustive (flat) list
    of starting values (in the order given by \code{model.matrix})
  }
  
  The \code{trace} argument applies only when a formula is specified.
  If you specify a function, you can build in your own \code{print()}
  or \code{cat()} statement to trace its progress.  (You can also
  specify a value for \code{trace} as part of a \code{control}
  list for \code{optim()}: see \code{\link{optim}}.)

  The \code{skip.hessian} argument is useful if the function is
  crashing with a "non-finite finite difference value" error when trying
  to evaluate the Hessian, but will preclude many subsequent
  confidence interval calculations.  (You will know the Hessian
  is failing if you use \code{method="Nelder-Mead"} and still
  get a finite-difference error.)
  
  If convergence fails, see the manual page of the
  relevant optimizer (\code{\link{optim}} by default,
  but possibly \code{\link{nlm}}, \code{\link{nlminb}},
  \code{\link[optimx]{optimx}}, or \code{\link{constrOptim}}
  if you have set the value of \code{optimizer})
  for the meanings of the error codes/messages.
}
\value{
  An object of class \code{"mle2"}.
}
\note{
  Note that the \code{minuslogl} function should
  return the negative log-likelihood, -log L (not
  the log-likelihood, log L, nor the deviance, -2 log L). It
  is the user's responsibility
  to ensure that the likelihood is correct, and that
  asymptotic likelihood inference is valid (e.g.
  that there are "enough" data and that the
  estimated parameter values do not lie on the
  boundary of the feasible parameter space).

  If \code{lower}, \code{upper}, \code{control$parscale},
  or \code{control$ndeps} are specified for \code{optim}
  fits, they must be named vectors.

  The requirement that \code{data} be specified when using
  the formula interface is relatively new: it saves many
  headaches on the programming side when evaluating the
  likelihood function later on (e.g. for profiling or
  constructing predictions).  Since \code{data.frame} uses
  the names of its arguments as column names by default, it
  is probably the easiest way to package objects that are
  lying around in the global workspace for use in \code{mle2}
  (provided they are all of the same length).

  When \code{optimizer} is set to "optimx" and multiple
  optimization methods are used (i.e. the \code{methods}
  argument has more than one element, or \code{all.methods=TRUE}
  is set in the control options), the best (minimum
  negative log-likelihood) solution will be saved,
  regardless of reported convergence status
  (and future operations such as profiling on the fit
  will only use the method that found the best result).
  
}
\seealso{
  \code{\link{mle2-class}}
}
\examples{
x <- 0:10
y <- c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8)
d <- data.frame(x,y)

## in general it is best practice to use the `data' argument,
##  but variables can also be drawn from the global environment
LL <- function(ymax=15, xhalf=6)
    -sum(stats::dpois(y, lambda=ymax/(1+x/xhalf), log=TRUE))
## uses default parameters of LL
(fit <- mle2(LL))
fit1F <- mle2(LL, fixed=list(xhalf=6))
coef(fit1F)
coef(fit1F,exclude.fixed=TRUE)

(fit0 <- mle2(y~dpois(lambda=ymean),start=list(ymean=mean(y)),data=d))
anova(fit0,fit)
summary(fit)
logLik(fit)
vcov(fit)
p1 <- profile(fit)
plot(p1, absVal=FALSE)
confint(fit)

## use bounded optimization
## the lower bounds are really > 0, but we use >=0 to stress-test
## profiling; note lower must be named
(fit1 <- mle2(LL, method="L-BFGS-B", lower=c(ymax=0, xhalf=0)))
p1 <- profile(fit1)

plot(p1, absVal=FALSE)
## a better parameterization:
LL2 <- function(lymax=log(15), lxhalf=log(6))
    -sum(stats::dpois(y, lambda=exp(lymax)/(1+x/exp(lxhalf)), log=TRUE))
(fit2 <- mle2(LL2))
plot(profile(fit2), absVal=FALSE)
exp(confint(fit2))
vcov(fit2)
cov2cor(vcov(fit2))

mle2(y~dpois(lambda=exp(lymax)/(1+x/exp(lhalf))),
   start=list(lymax=0,lhalf=0),
   data=d,
   parameters=list(lymax~1,lhalf~1))

\dontrun{
## try bounded optimization with nlminb and constrOptim
(fit1B <- mle2(LL, optimizer="nlminb", lower=c(lymax=1e-7, lhalf=1e-7)))
p1B <- profile(fit1B)
confint(p1B)
(fit1C <- mle2(LL, optimizer="constrOptim", ui = c(lymax=1,lhalf=1), ci=2,
   method="Nelder-Mead"))

set.seed(1001)
lymax <- c(0,2)
lhalf <- 0
x <- sort(runif(200))
g <- factor(sample(c("a","b"),200,replace=TRUE))
y <- rnbinom(200,mu=exp(lymax[g])/(1+x/exp(lhalf)),size=2)
d2 <- data.frame(x,g,y)

fit3 <- mle2(y~dnbinom(mu=exp(lymax)/(1+x/exp(lhalf)),size=exp(logk)),
    parameters=list(lymax~g),data=d2,
    start=list(lymax=0,lhalf=0,logk=0))
}
}
\keyword{models}

\name{BIC-methods}
\docType{methods}
%\alias{BIC}
\alias{BIC-methods}
\alias{AIC-methods}
\alias{AICc-methods}
\alias{logLik-methods}
\alias{AICc}
\alias{AIC,mle2-method}
\alias{AICc,mle2-method}
\alias{AICc,logLik-method}
\alias{AICc,ANY-method}
\alias{AICc,ANY,mle2,logLik-method}
\alias{qAICc}
\alias{qAICc-methods}
\alias{qAICc,ANY-method}
\alias{qAICc,mle2-method}
\alias{qAICc,logLik-method}
\alias{qAIC}
\alias{qAIC-methods}
\alias{qAIC,ANY-method}
\alias{qAIC,mle2-method}
\alias{qAIC,logLik-method}
%\alias{BIC,logLik-method}
%\alias{BIC,ANY-method}
%\alias{BIC,mle2-method}
%\alias{BIC,ANY,mle2,logLik-method}
\alias{qAIC,ANY,mle2,logLik-method}
\alias{qAICc,ANY,mle2,logLik-method}
\alias{logLik,mle2-method}
\alias{anova,mle2-method}
\title{Log likelihoods and model selection for mle2 objects}
\description{
  Various functions for likelihood-based and information-theoretic
  model selection of likelihood models
}
\section{Methods}{
\describe{
    \item{logLik}{\code{signature(object = "mle2")}: Extract maximized
      log-likelihood.}
    \item{AIC}{\code{signature(object = "mle2")}: Calculate
      Akaike Information Criterion}
    \item{AICc}{\code{signature(object = "mle2")}: Calculate
      small-sample corrected Akaike Information Criterion}
    %\item{BIC}{\code{signature(object = "mle2")}: Calculate
    %Bayesian (Schwarz) Information Criterion}
    %\item{BIC}{\code{signature(object = "logLik")}: Calculate
    %Bayesian (Schwarz) Information Criterion}
    %\item{BIC}{\code{signature(object = "ANY")}: Calculate
    %Bayesian (Schwarz) Information Criterion}
    \item{anova}{\code{signature(object="mle2")}: Likelihood Ratio Test
      comparision of different models}
  }
}
\usage{
%\S4method{BIC}{ANY,mle2,logLik}(object,...)
\S4method{AICc}{ANY,mle2,logLik}(object,...,nobs,k=2)
\S4method{qAIC}{ANY,mle2,logLik}(object,...,k=2)
\S4method{qAICc}{ANY,mle2,logLik}(object,...,nobs,k=2)
}
\arguments{
  \item{object}{A \code{logLik} or \code{mle2} object}
  \item{...}{An optional list of additional \code{logLik}
    or \code{mle2} objects (fitted to the same data set).}
  \item{nobs}{Number of observations (sometimes 
    obtainable as an attribute of
    the fit or of the log-likelihood)}
  \item{k}{penalty parameter (nearly always left at its default value of 2)}
}
\details{
  Further arguments to \code{BIC} can be specified
  in the \code{...} list: \code{delta} (logical)
  specifies whether to include a column for delta-BIC
  in the output.
}
\value{
  A table of the BIC values, degrees of freedom,
  and possibly delta-BIC values relative to the
  minimum-BIC model
}
\note{This is implemented in an ugly way and could
  probably be improved!}
\examples{
  d <- data.frame(x=0:10,y=c(26, 17, 13, 12, 20, 5, 9, 8, 5, 4, 8))
  (fit <- mle2(y~dpois(lambda=ymax/(1+x/xhalf)),
      start=list(ymax=25,xhalf=3),data=d))
  (fit2 <- mle2(y~dpois(lambda=(x+1)*slope),
      start=list(slope=1),data=d))
  BIC(fit)
  BIC(fit,fit2)
  }
\keyword{methods}

\name{namedrop}
\alias{namedrop}
\title{drop unneeded names from list elements}
\description{
  goes through a list (containing a combination
  of single- and multiple-element vectors) and
  removes redundant names that will make trouble
  for mle
}
\usage{
namedrop(x)
}
\arguments{
  \item{x}{a list of named or unnamed, typically numeric, vectors}
}
\details{
  examines each element of \code{x}.  If the element has length
  one and is a named vector, the name is removed; if \code{length(x)}
  is greater than 1, but all the names are the same, the vector
  is renamed 
}
\value{
  the original list, with names removed/added
}
\author{Ben Bolker}
\examples{
x = list(a=c(a=1),b=c(d=1,d=2),c=c(a=1,b=2,c=3))
names(unlist(namedrop(x)))
names(unlist(namedrop(x)))
}
\keyword{misc}

\name{slice.mle2-class}
\docType{class}
\alias{slice.mle2-class}
\title{likelihood-surface slices}
\description{evaluations of log-likelihood along transects in parameter space}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("slice.mle2", ...)}.
The objects are similar to likelihood profiles, but don't involve
any optimization with respect to the other parameters.
}
\section{Slots}{
  \describe{
    \item{\code{profile}:}{Object of class \code{"list"}. List of
      slices, one for each requested parameter. Each slice is a data
      frame with the first column called \code{z} being the signed square
      root of the -2 log likelihood ratio, and the others being the
      parameters with names prefixed by \code{par.vals.}}
    \item{\code{summary}:}{Object of class \code{"summary.mle2"}. Summary
      of object being profiled.}
  }
}
\section{Methods}{
  \describe{
    \item{plot}{\code{signature(x = "profile.mle2", y = "missing")}: Plot
      profiles for each parameter.}
  }
}
\seealso{
  \code{\link{profile.mle2-class}}
}
\keyword{classes}

\name{mle2.options}
\alias{mle2.options}
\title{Options for maximum likelihood estimation}
\description{
  Query or set MLE parameters
}
\usage{
mle2.options(...)
}
\arguments{
  \item{\dots}{names of arguments to query,
    or a list of values to set}
}
\details{
  \describe{
    \item{optim.method}{name of optimization method (see
      \code{\link{optim}} for choices)}
    \item{confint}{name of confidence interval method: choices
      are "spline", "uniroot", "hessian" corresponding
      to spline inversion, attempt to find best answer via uniroot,
      information-matrix approximation}
    \item{optimizer}{optimization function to use by default
(choices: "optim", "nlm", "nlminb", "constrOptim")}
}
}
\value{
  Values of queried parameters, or (invisibly) the full list of parameters
}
\seealso{
  \code{\link{mle2-class}}
}
\keyword{models}

\name{dnorm_n}
\alias{dnorm_n}
\title{
  Normal distribution with profiled-out standard deviation
}
\description{
  Returns the Normal probability densities for a distribution
  with the given mean values and the standard deviation equal
  to the root mean-squared deviation between x and mu
}
\usage{
dnorm_n(x, mean, log = FALSE)
}
\arguments{
  \item{x}{numeric vector of data}
  \item{mean}{numeric vector or mean values}
  \item{log}{logical: return the log-density?}
}
\details{
  This is a convenience function, designed for the case
  where you're trying to compute a MLE for the mean but
  don't want to bother estimating the MLE for the standard
  deviation at the same time
}
\value{
  Numeric vector of probability densities
}
\examples{
set.seed(101)
x <- rnorm(5,mean=3,sd=2)
dnorm_n(x,mean=3,log=TRUE)
}
\keyword{distribution}

\name{get.mnames}
\alias{get.mnames}
\title{extract model names}
\description{
  given a list of models, extract the names (or "model n")
}
\usage{
get.mnames(Call)
}
\arguments{
  \item{Call}{a function call (usually a list of models)}
}
\value{
  a vector of model names
}
\author{Ben Bolker}
\keyword{misc}

\name{relist2}
\alias{relist2}
\title{reconstruct the structure of a list}
\description{
  reshapes a vector according to a list template
}
\usage{
relist2(v, l)
}
\arguments{
  \item{v}{vector, probably numeric, of values to reshape}
  \item{l}{template list giving structure}
}
\details{
  attempts to coerce \code{v} into a list with the same
  structure and names as \code{l}
}
\value{
  a list with values corresponding to v and structure corresponding to l
}
\author{Ben Bolker}
\examples{
  l = list(b=1,c=2:5,d=matrix(1:4,nrow=2))
  relist2(1:9,l)
}
\keyword{misc}
